﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Reflection;
using Styx.CommonBot.CharacterManagement;

namespace KeepOnMoving.Settings
{
    using static KeepOnMoving;

    /*
    class ClassProfileComboBoxEntry
    {
        public ClassProfileComboBoxEntry(ClassProfile classProfile)
        {
            ClassProfile = classProfile;
        }

        public ClassProfile ClassProfile { get; private set; }
        public bool FullName { get; set; }

        public override string ToString()
        {
            string className = ClassProfileLocalization.GetClassName(ClassProfile.Class);
            string specName = ClassProfileLocalization.GetSpecName(ClassProfile.GetSpec());

            if (FullName)
                return string.Format("{0} - {1} ({2})", className, specName, ClassProfile.Name);

            return string.Format("{0} - {1}", className, specName);
        }
    }
    */

    public partial class GUI : Form
    {
        private static KeepOnMoving KeepOnMoving = new KeepOnMoving();

        public GUI()
        {
            InitializeComponent();
			this.Icon = new Icon(Path.Combine(Application.StartupPath, "Plugins/KeepOnMoving/KoM.ico"), new Size(256, 256));
        }

        public static void SetLabelColumnWidth(PropertyGrid grid, int width)
        {
            if (grid == null)
                return;

            FieldInfo fi = grid.GetType().GetField("gridView", BindingFlags.Instance | BindingFlags.NonPublic);
            if (fi == null)
                return;

            Control view = fi.GetValue(grid) as Control;
            if (view == null)
                return;

            MethodInfo mi = view.GetType().GetMethod("MoveSplitterTo", BindingFlags.Instance | BindingFlags.NonPublic);
            if (mi == null)
                return;
            mi.Invoke(view, new object[] { width });
        }

		public static void ChangeDescriptionHeight(PropertyGrid grid, int height)
		{
            if (grid == null)
                throw new ArgumentNullException("grid");

            foreach (Control control in grid.Controls)
            {
                if (control.GetType().Name == "DocComment")
                {
                    FieldInfo fieldInfo = control.GetType().BaseType.GetField("userSized",
                                                                              BindingFlags.Instance | BindingFlags.NonPublic);
                    fieldInfo.SetValue(control, true);
                    control.Height = height;
                    return;
                }
            }
		}

        private void GUI_Load(object sender, EventArgs e)
        {
            pgSettings.SelectedObject = KOMSettings.Instance;
            KOMSettings.Instance.Load();
			SetLabelColumnWidth(pgSettings, 250);
			ChangeDescriptionHeight(pgSettings, 80);
            CreateDescription(this, new EventArgs());

            pgSettings.Controls[0].Controls[1].TextChanged += CreateDescription;
        }

        private void GUI_FormClosing(object sender, FormClosingEventArgs e)
        {
            KOMSettings.Instance.Save();
            KeepOnMoving.PrintSettings();
        }

        private void pgSettings_Click(object sender, EventArgs e)
        {
        }

        private void CreateDescription(object sender, EventArgs e)
        {
            var _descriptionControl = pgSettings.Controls[0].Controls[1];
            if (_descriptionControl.Text.Contains(" --- "))
            {
                var _parts = Regex.Split(_descriptionControl.Text, " --- ");

                var _textWidth = TextRenderer.MeasureText(_parts[0], _descriptionControl.Font).Width;
                var _linesNum = (int)Math.Ceiling(_textWidth / (double)_descriptionControl.Width);
                var _lineBreaks = new String(Convert.ToChar("\n"), 4 - _linesNum);

                var _defaultText = String.Format("<Default: {0}>", _parts[1]);
                var _defaultWidth = TextRenderer.MeasureText(_defaultText, _descriptionControl.Font).Width;

                var _lastLine = " " + _defaultText;
                while (TextRenderer.MeasureText(_lastLine, _descriptionControl.Font).Width < _descriptionControl.Width)
                    _lastLine = " " + _lastLine;
                _lastLine = _lastLine.Substring(1);

                _descriptionControl.Text = _parts[0] +
                                        _lineBreaks +
                                        _lastLine;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            KOMSettings.Instance.Save();
            KeepOnMoving.PrintSettings();
            this.Dispose();
        }

        private void updateToolStripMenuItem_Click(object sender, EventArgs e)
        {
			Process.Start("https://www.thebuddyforum.com/threads/plugin-keeponmoving-unstuck-handling-and-more-in-development.249513/#post-2223798");
        }

        private void donateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=G77TYH3R6Q2R2");
        }

        /*
         * Source: https://dotnet-snippets.de/snippet/strings-komprimieren-und-dekomprimieren/1058
         */

        private string Compress(string _string)
        {
            var _buffer = Encoding.UTF8.GetBytes(_string);
            var _memoryStream = new MemoryStream();
            using (var _gZipStream = new GZipStream(_memoryStream, CompressionMode.Compress, true))
            {
                _gZipStream.Write(_buffer, 0, _buffer.Length);
            }

            _memoryStream.Position = 0;

            var _compressedData = new byte[_memoryStream.Length];
            _memoryStream.Read(_compressedData, 0, _compressedData.Length);

            var _gZipBuffer = new byte[_compressedData.Length + 4];
            Buffer.BlockCopy(_compressedData, 0, _gZipBuffer, 4, _compressedData.Length);
            Buffer.BlockCopy(BitConverter.GetBytes(_buffer.Length), 0, _gZipBuffer, 0, 4);
            return Convert.ToBase64String(_gZipBuffer);
        }

        /*
         * Source: https://dotnet-snippets.de/snippet/strings-komprimieren-und-dekomprimieren/1058
         */

        public static string Decompress(string _string)
        {
            try
            {
                var _gZipBuffer = Convert.FromBase64String(_string);
                using (var _memoryStream = new MemoryStream())
                {
                    int _dataLength = BitConverter.ToInt32(_gZipBuffer, 0);
                    _memoryStream.Write(_gZipBuffer, 4, _gZipBuffer.Length - 4);

                    var _buffer = new byte[_dataLength];

                    _memoryStream.Position = 0;
                    using (var _gZipStream = new GZipStream(_memoryStream, CompressionMode.Decompress))
                    {
                        _gZipStream.Read(_buffer, 0, _buffer.Length);
                    }

                    return Encoding.UTF8.GetString(_buffer);
                }
            }
            catch(Exception)
            {
                return null;
            }
        }

        private void exportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var _settings = KOMSettings.Instance.GetSettings();

            string _string = null;
            foreach (var _setting in _settings)
                _string += String.Format("{0}={1}&", 
                                            _setting.Key, 
                                            _setting.Value.ToString()
                                                        .Replace("&", " ")
                                                        .Replace("=", " "));

            Clipboard.SetText(Compress(_string));
            MessageBox.Show("Exported settings to clipboard", "Export settings", MessageBoxButtons.OK);
        }

        private void LoadFromString(string _string)
        {
            var _list = _string.Split('&').ToList();
            _list.RemoveAt(_list.Count() - 1);

            var _content = new List<object>();
            foreach (var _e in _list)
            {
                var _parts = _e.Split('=');
                _content.Add(new System.Xml.Linq.XElement(_parts[0], _parts[1]));
            }

            var _settings = new System.Xml.Linq.XElement("KOMSettings", _content);
            KOMSettings.Instance.LoadFromXML(_settings);
        }

        private void importToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var _result = MessageBox.Show("Import settings from clipboard?", "Import settings", MessageBoxButtons.YesNo);

            if (_result == DialogResult.Yes)
            {
                var _string = Decompress(Clipboard.GetText());
                if (_string == null)
                {
                    MessageBox.Show("Data seems to be corrupted", "Error", MessageBoxButtons.OK);
                    return;
                }                

                LoadFromString(_string);
                KOMSettings.Instance.Save();

                KeepOnMoving.PrintSettings(false, true);
                MessageBox.Show("Successfully imported", "Import settings", MessageBoxButtons.OK);
                this.Dispose();
                KeepOnMoving.OnButtonPress();
            }
        }
    }
}
