﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Reflection.Emit;
using System.ComponentModel;
using Styx;
using Styx.Common;
using Styx.CommonBot.CharacterManagement;
using Styx.Helpers;
using Styx.WoWInternals.WoWObjects;
using DefaultValue = Styx.Helpers.DefaultValueAttribute;

namespace KeepOnMoving.Settings
{
    enum QuestingPack
    {
        None = 0,
        Old,
        New
    }

    enum PandarenFaction
    {
        NotYetChosen = 0,
        Horde,
        Alliance
    }

    enum TravelForm
    {
        None = 0,
        GroundMountOnly, 
        FlyingMountOnly,
        Both
    }

    enum GatheringProfessions
    {
        None = 0,
        Herbalism,
        HerbalismAndMining,
        // HerbalismAndSkinning,
        Mining,
        // MiningAndSkinning,
        // Skinning
    }

    enum TrainFlying
    {
        None = 0,
        ArtisanOnly,
        ArtisanAndMaster,
    }

    enum HandleNodes
    {
        BotBaseOnly = 0,
        PluginOnly,
        Both,
    }

    enum Fragments
    {
        None        = 0,
        Dwarven     = 87533,
        Draenei     = 87534,
        Fossil      = 87535,
        NightElf    = 87536,
        Nerubian    = 87537,
        Orc         = 87538,
        Tolvir      = 87539,
        Troll       = 87540,
        Vyrkul      = 87541
    }

    internal class KOMSettings : Styx.Helpers.Settings
    {
        private static KOMSettings _instance;
        private static LocalPlayer Me => StyxWoW.Me;
        public static KOMSettings Instance => _instance ?? (_instance = new KOMSettings());
        private static new string SettingsPath => string.Format("{0}\\Settings\\KeepOnMoving\\{1}_{2}", Utilities.AssemblyDirectory, Me.RealmName.Replace(" ", ""), Me.Name);
        public Dictionary<object, object> _settingsDict = new Dictionary<object, object>();

        private KOMSettings()
            : base(SettingsPath + ".config")
        {
        }

        [Setting]
        [Styx.Helpers.DefaultValue(30)]
        [Category("- MOTIONLESS Settings")]
        [DisplayName("Timer")]
        [Description("Amount of seconds not having moved before motionless procedure begins (\"0\" to disable) [NOTE: This setting will not take effect if you are using \"Questing Profile Pack\" shipped with HB or from store] --- 30")]
        public int Motionless_Timer { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- MOTIONLESS Settings")]
        [DisplayName("Objective Timer")]
        [Description("Amount of seconds objective not having changed before motionless procedure begins (\"0\" to disable) --- 0")]
        public int Motionless_ObjectiveTimer { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- STUCK Settings")]
        [DisplayName("Force Dismount")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to immediately dismount when ground mounted and stuck is detected --- False")]
        public bool Stuck_Dismount { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(30)]
        [Category("- STUCK Settings")]
        [DisplayName("Speed Tolerance")]
        [Description("When char's speed falls below this percentage of default speed, StuckCount will be increased for this tick (1 - 100) [NOTE: Decrease to slow down, increase to speed up stuck detection] --- 30")]
        public int Stuck_SpeedTolerance { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(7)]
        [Category("- STUCK Settings")]
        [DisplayName("StuckCount Threshold")]
        [Description("If StuckCount reaches this value, char is considered as being stuck and unstuck handling is triggered [NOTE: Increase to slow down, decrese to speed up stuck detection] --- 5")]
        public int Stuck_StuckCountThreshold { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- STUCK Settings")]
        [DisplayName("Disable Stuck Handling")]
        [Description("Prevents ⌠ KEEP oи MOVING ⌡ from handling stucks --- False")]
        public bool Stuck_Disable { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- DIALOG Settings")]
        [DisplayName("Decline Duel Invites")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to decline duel invites --- False")]
        public bool Dialog_DeclineDuelInvites { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- DIALOG Settings")]
        [DisplayName("Decline Party Invites")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to decline party invites --- False")]
        public bool Dialog_DeclinePartyInvites { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- DIALOG Settings")]
        [DisplayName("Decline Guild Invites")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to decline guild invites --- False")]
        public bool Dialog_DeclineGuildInvites { get; set; }

        [Setting]
        [DefaultValue(GatheringProfessions.None)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Set Gathering Professions")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to handle certain tasks concerning gathering professions --- None")]
        public GatheringProfessions GatheringProfessions_Professions { get; set; }

        [Setting]
        [DefaultValue(HandleNodes.PluginOnly)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Handle Nodes")]
        [Description("Set who is allowed to handle nodes. --- PluginOnly")]
        public HandleNodes GatheringProfessions_HandleNodes { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Train Skill(s)")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to train skill(s) --- True")]
        public bool GatheringProfessions_TrainSkill { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Harvest Mobs")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to harvest mobs --- False")]
        public bool GatheringProfessions_HarvestMobs { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Go to Pandaria")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to go to Pandaria when skill value becomes suitable --- False")]
        public bool Archaeology_GoToPandaria { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Go to Draenor")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to go to Draenor when skill value becomes suitable --- False")]
        public bool Archaeology_GoToDraenor { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Go to Broken Isles")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to go to Broken Isles when skill value becomes suitable --- False")]
        public bool Archaeology_GoToBrokenIsles { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Train Archaeology")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to train at trainer. [NOTE: \"Set Hearthstone\" needs to be enabled] --- True")]
        public bool Archaeology_Train { get; set; }

        [Setting]
        [DefaultValue(Fragments.None)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Buy Fragments")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to buy fragments from Brann Bronzebeard in Pandaria. [NOTE: \"Open Items\" should be enabled as fragments couldn't be used otherwise] --- None")]
        public Fragments Archaeology_Fragments { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- JUMP Settings")]
        [DisplayName("Enable Jump")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to jump randomly between minimum and maximum pause --- False")]
        public bool Jump_Enable { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(5000)]
        [Category("- JUMP Settings")]
        [DisplayName("Maximum Pause")]
        [Description("Maximum pause between jumps in milliseconds --- 3000")]
        public int Jump_MaxPause { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(1000)]
        [Category("- JUMP Settings")]
        [DisplayName("Minimum Pause")]
        [Description("Minimum pause between jumps in milliseconds --- 1000")]
        public int Jump_MinPause { get; set; }

        [Setting]
        [DefaultValue(PandarenFaction.NotYetChosen)]
        [Category("- QUEST Settings")]
        [DisplayName("Set Pandaren Faction")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to skip message box for choosing faction by loading subprofile instead; this takes effect for starting quests only --- NotYetChosen")]
        public PandarenFaction Quest_PandarenFaction { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- LOG Settings")]
        [DisplayName("Enable Runtime Logging")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to log coroutine runtimes. Enable this setting when debugging perfomance related issues only --- False")]
        public bool Log_Runtime { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Sort Bags")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to sort bags --- True")]
        public bool Bag_Sort { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Mail Items")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to mail items (low level items like flasks, etc. included) --- True")]
        public bool Bag_MailItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Combine Items")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to combine items (like pre herbs/ores) --- True")]
        public bool Bag_CombineItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Open Items")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to open items (like clams) --- True")]
        public bool Bag_OpenItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Learn Toy Items")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to learn known toy items --- True")]
        public bool Bag_LearnToys { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Remove Already Known Toy Items")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to remove already known toy items --- True")]
        public bool Bag_RemoveToys { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- BAG Settings")]
        [DisplayName("Remove Low Level Quest Items")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to remove low level quest items --- False")]
        public bool Bag_RemoveQuestItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Manage Containers")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to equip container items or exchange with better ones --- True")]
        public bool Bag_ManageContainers { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Get New Mails")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to get new mails (attachments like items or gold included) --- True")]
        public bool Bag_GetMails { get; set; }

        /*
        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- GENERAL Settings")]
        [DisplayName("Set Favourite GroundMount")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set ground mount to this mount name as soon as it is available in HB settings(\"0\" to disable)\nNOTE: Mount id could be used as well\n\n                                  <Default: 0>")]
        public string G_GroundMount { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- GENERAL Settings")]
        [DisplayName("Set Favourite FlyingMount")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set flying mount to this mount name as sonn as it is available in HB settings(\"0\" to disable)\nNOTE: Mount id could be used as well\n\n                                  <Default: 0>")]
        public string G_FlyingMount { get; set; }
        

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Set Hearthstone")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set hearthstone either to Orgrimmar/Stormwind or Shrine of Two Moons/Seven Stars [NOTE: This setting is crucial for some features to work properly, e.g. training skills --- True")]
        public bool Hearth_SetHearthstone { get; set; }
        */

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Class Specific: Death Knight")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use Death Knight's \"Death Gate\" --- True")]
        public bool Hearth_DeathKnight { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Class Specific: Druid")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use Druid's \"Teleport: Moonglade\" --- True")]
        public bool Hearth_Druid { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Class Specific: Monk")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use Monk's \"Zen Pilgrimage\" --- True")]
        public bool Hearth_Monk { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Hearthstone\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use \"Hearthstone\". [NOTE: Training skills for archaeology/gathering professions or questing progress require hearthstone to be enabled] --- True")]
        public bool Hearth_Hearthstone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Orgrimmar\"/\"Stormwind Portal Stone\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use \"Orgrimmar\"/\"Stormwind Portal Stone\" --- True")]
        public bool Hearth_PortalStone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Garrison Hearthstone\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use \"Garrison Hearthstone\" --- True")]
        public bool Hearth_GarrisonHearthstone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Dalaran Hearthstone\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use \"Dalaran Hearthstone\" --- True")]
        public bool Hearth_DalaranHearthstone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Flight Master's Whistle\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use \"Flight Master's Whistle\" --- True")]
        public bool Hearth_FlightmastersWhistle { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- HEARTH Settings")]
        [DisplayName("Stop Bot Instead of Hearthing")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop Bot instead of hearthing --- False")]
        public bool Hearth_PreStop { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- HEARTH Settings")]
        [DisplayName("Stop Bot After Hearthing")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop Bot after hearthing --- False")]
        public bool Hearth_PostStop { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- HEARTH Settings")]
        [DisplayName("Disable Hearthing")]
        [Description("Disable Hearthing. [NOTE: Training skills for archaeology/gathering professions or questing progress require hearthing to be enabled] --- False")]
        public bool Hearth_Disable { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Prevent Disable Addons")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to prevent new questing pack to disable game addons --- False")]
        public bool Quest_PreventDisableAddons { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Force \"KillBetweenHotspots\" to \"false\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to force \"KillBetweenHotspots\" to \"false\" although questing profile sets it to \"true\". This will speed up leveling from 12-58 dramatically if you are using heirlooms --- False")]
        public bool Quest_KillBetweenHotspots { get; set; }

        [Setting]
        [DefaultValue(TravelForm.Both)]
        [Category("- GENERAL Settings")]
        [DisplayName("TravelForm As Mount")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set TravelForm as Mount in HB settings as soon as spell is learnt. --- Both")]
        public TravelForm General_TravelForm { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GENERAL Settings")]
        [DisplayName("Speed Looting")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to loot while being in combat. --- True")]
        public bool General_SpeedLoot { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GENERAL Settings")]
        [DisplayName("Prevent Refreshing Taxi Nodes")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to prevent refreshing taxi nodes. --- False")]
        public bool General_PreventTaxi { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GENERAL Settings")]
        [DisplayName("Infinite Mode")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to restart bot after having stopped. Can only be turned off by changing this setting or unticking plugin --- False")]
        public bool General_InfiniteMode { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- QUEST Settings")]
        [DisplayName("Prevent Hovering")]
        [Description("Prevents hovering with flying mount and switches to ground mount instead (happens in zones when flying skill is already learnt but profiles are coded for running/walking) --- True")]
        public bool Quest_PreventHovering { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- GENERAL Settings")]
        [DisplayName("MountUp Distance")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to force MountUp when distance to next hotspot is greater than this value (\"0\" to disable) --- 0")]
        public int General_MountUpDistance { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(180)]
        [Category("- GENERAL Settings")]
        [DisplayName("Resurrection Timer")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to force resurrection at graveyard after this amount of seconds being dead (\"0\" to disable) --- 180")]
        public int General_ResurrectionTimer { get; set; }

        [Setting]
        [DefaultValue(QuestingPack.Old)]
        [Category("- QUEST Settings")]
        [DisplayName("Questing Pack")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to use new or old questing pack [NOTE: Copy files of new pack to folder \"Questing Profile Pack\"] --- Old")]
        public QuestingPack Quest_QuestingPack { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- QUEST Settings")]
        [DisplayName("Prefer AutoLoader")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to prefer AutoLoader if using default questing profiles. --- True")]
        public bool Quest_AutoLoader { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Abandon Low Level Quests")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to abandon low level quests. [NOTE: As ⌠ KEEP oи MOVING ⌡ will skip unnecessary parts (e.g. if you have heirlooms equipped), HB might not be able to abandon/complete them itself) --- False")]
        public bool Quest_AbandonQuests { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Low Graphic Mode")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set RenderScale to 0.1, maximum FPS and maximum background FPS to 20 --- False")]
        public bool Performance_LowGraphicMode { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(100)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Default RenderScale")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set RenderScale to this value whenever it stops/pauses (1 - 200) --- 100")]
        public double Performance_DefaultRenderScale { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Default Maximum FPS")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set maximum FPS to this value whenever it stops/pauses (\"0\" to disable) --- 0")]
        public double Performance_MaxFps { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Default Maximum Background FPS")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to set maximum background FPS to this value whenever it stops/pauses (\"0\" to disable) --- 0")]
        public double Performance_MaxFpsBk { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GENERAL Settings")]
        [DisplayName("Update Sound")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to play update sound OnEnable when new version is available --- True")]
        public bool General_UpdateSound { get; set; }

        [Setting]
        [DefaultValue(TrainFlying.None)]
        [Category("- QUEST Settings")]
        [DisplayName("Learn \"Artisan\"- and \"Master Riding\"")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to learn \"Artisan\"- (280%) and \"Master Riding\" (310%). [NOTE: Depending on your reputation with OG/SW faction you will need 4000-5000 gold for each] --- None")]
        public TrainFlying Quest_TrainFlying { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- QUEST Settings")]
        [DisplayName("Set Specialisation")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to change specialization to defined index (\"0\" to disable) --- 0")]
        public int Quest_Specialisation { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Stop At SkillMax")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop HB when gather professions are max skilled --- False")]
        public bool Quest_StopSkillMax { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- QUEST Settings")]
        [DisplayName("Stop At Level")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop HB when this level is reached (\"0\" to disable) --- 0")]
        public int Quest_StopLevel { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- QUEST Settings")]
        [DisplayName("Stop With Item")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop HB when item with this ID is received [NOTE: Separate multiple IDs with char you prefer] (\"0\" to disable) --- 0")]
        public string Quest_StopItem { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- QUEST Settings")]
        [DisplayName("Stop With Quest")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop HB when quest with this ID is received [NOTE: Separate multiple IDs with char you prefer] (\"0\" to disable) --- 0")]
        public string Quest_StopQuest { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- QUEST Settings")]
        [DisplayName("Stop Action")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to start this HBRelog profile when stopping point is reached (\"0\" to disable) --- 0")]
        public string Quest_StopAction { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Stop HBRelog Profile")]
        [Description("Allow ⌠ KEEP oи MOVING ⌡ to stop HBRelog profile when stopping point is reached --- False")]
        public bool Quest_StopHBRelog { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(15)]
        [Category("- FPS STABILISER Settings")]
        [DisplayName("FPS Threshold")]
        [Description("If FPS (frames per seconds) drop below this value, ⌠ KEEP oи MOVING ⌡ will decrease TPS (ticks per seconds) to minimum (\"0\" to disable) --- 15")]
        public int FpsStabiliser_FpsThreshold { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(5)]
        [Category("- FPS STABILISER Settings")]
        [DisplayName("Minimum TPS")]
        [Description("If FPS (frames per seconds) drop below threshold, ⌠ KEEP oи MOVING ⌡ will decrease TPS (ticks per seconds) to this value --- 5")]
        public int FpsStabiliser_MinTps { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(15)]
        [Category("- FPS STABILISER Settings")]
        [DisplayName("Maximum TPS")]
        [Description("If FPS (frames per seconds) are above threshold, ⌠ KEEP oи MOVING ⌡ will slowly increase TPS (ticks per seconds) to this value --- 15")]
        public int FpsStabiliser_MaxTps { get; set; }
    }
}