﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Windows.Media;
using System.Media;
using System.Numerics;
using System.Xml;

using Buddy.Coroutines;
using CommonBehaviors.Actions;
using Bots.DungeonBuddy.Helpers;
using Bots.Grind;
using Bots.ArchaeologyBuddy;

using Styx;
using Styx.TreeSharp;
using Styx.Helpers;
using Styx.Plugins;
using Styx.Pathing;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using static Styx.CommonBot.BotEvents;
using Styx.CommonBot.CharacterManagement;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Frames;
using Styx.CommonBot.Profiles;
using Styx.CommonBot.Profiles.Quest.Order;
using Styx.CommonBot.ObjectDatabase;
using Styx.CommonBot.POI;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using static Styx.WoWInternals.WoWMovement;
using Styx.WoWInternals.World;
using Styx.WoWInternals.DB;
using Styx.WoWInternals.DBC;

using System.Windows.Forms;
using System.Net;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Runtime.CompilerServices;
using System.Security.Cryptography;
using System.Text;

namespace KeepOnMoving
{
    using Settings;

    enum Stuck
    {
        Off = 0,
        On,
        Stage1,
        Stage2,
        Stage3,
        Stage4,
        Stage5,
        Stage6, // RELOAD
        Stage7  // HEARTH
    }

    enum Motionless
    {
        Off = 0,
        On,
        Stage1, // UNSTUCK
        Stage2, // RELOAD
        Stage3  // HEARTH
    }

    enum Hearth
    {
        Off = 0,
        On,
        Capital,
        Pandaria,
        Draenor,
        BrokenIsles
    }

    enum Dialog
    {
        None = 0,
        Guild,
        Party,
        Duel
    }

    public class KOMWebClient : WebClient
    {
        protected override WebRequest GetWebRequest(Uri _uri)
        {
            WebRequest _webRequest = base.GetWebRequest(_uri);
            _webRequest.Timeout = 4 * 1000;
            return _webRequest;
        }
    }

    public class KeepOnMoving : HBPlugin
    {
        private Form _gui;
        public override string Name => "KeepOnMoving";
        public override string Author => "mh66";
        public override Version Version => new Version(3, 1, 37);
        public override bool WantButton => true;
        public override string ButtonText => "Settings";
        private KOMSettings Settings => KOMSettings.Instance;

        private void Log(string _text, params object[] _arguments)
        {
            var _color = Color.FromRgb(28, 117, 240);
            Bots.Professionbuddy.PBLog.Log(_color, "          ⌠ KEEP oи MOVING ⌡ ", Colors.White, _text, _arguments);
        }

        public void DevLog(string _text, params object[] _arguments)
        {
            var _color = Color.FromRgb(125, 175, 255);
            Logging.WriteDiagnostic(_color, $"          ***** ⌠ KEEP oи MOVING ⌡ {_text}", _arguments);
        }

        /*
         *      TEST TWO DIFFERENT CODE RUNTIMES
         *
            ClearLog();
            var _d = DateTime.Now;
            var _runs = 1000;

            for(int i=1; i <= _runs; i++)
            {
	            // FIRST CODE
            }

            var _firstRuntime = DateTime.Now.Subtract(_d).TotalMilliseconds / _runs;

            _d = DateTime.Now;
            for(int i=1; i <= _runs; i++)
            {
	            // SECOND CODE
            }

            var _secondRuntime = DateTime.Now.Subtract(_d).TotalMilliseconds / _runs;

            Log("1 : {0} ({1}% of 2) \n2 : {2} ({3}% of 1)",
		            _firstRuntime,
		            Math.Round(_firstRuntime / _secondRuntime * 100),
		            _secondRuntime,
		            Math.Round(_secondRuntime / _firstRuntime * 100));
        */

        private void RuntimeLog(string _option = null, [CallerLineNumber] int _lineNumber = 0)
        {
            if (!Settings.Log_Runtime)
                return;

            var _colorGrey = Color.FromRgb(100, 100, 100);
            var _colorGreen = Color.FromRgb(0, 255, 100);

            if (_option == "Start")
            {
                Logging.WriteDiagnostic(_colorGrey, "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                _timePerformanceRunSum = DateTime.Now;
            }
            else if (_option == "Sum")
            {
                var _totalMs = Math.Round(DeltaTime(_timePerformanceRunSum, "ms"));
                var _tps = TreeRoot.TicksPerSecond;
                var _msPerTick = Math.Round(1000.0 / TreeRoot.TicksPerSecond);
                var _deltaMs = Math.Round(_msPerTick - _totalMs);

                if (_deltaMs < 0 && Settings.FpsStabiliser_FpsThreshold != 0)
                {
                    DevLog("DeltaMS < 0: {0}", _deltaMs);
                    TreeRoot.TicksPerSecond = (byte)Settings.FpsStabiliser_MinTps;
                }

                Logging.WriteDiagnostic(_colorGrey, "                          {0} ({1})", _totalMs,
                                                                                           (_deltaMs > 0)
                                                                                                ? $"+ {_deltaMs}"
                                                                                                : $"{_deltaMs}");
                Logging.WriteDiagnostic(_colorGrey, "                              {0} TPS", _tps);
                Logging.WriteDiagnostic(_colorGrey, "                              {0} FPS", StyxWoW.WoWClient.Fps);
                Logging.WriteDiagnostic(_colorGrey, "                              {0} ms", _msPerTick);
            }
            else
            {
                if (!_lineRuntimeSum.ContainsKey(_lineNumber))
                    _lineRuntimeSum.Add(_lineNumber, DeltaTime(_timePerformanceRun, "ms"));
                else
                {
                    if (_option == "##")
                        _lineRuntimeSum[_lineNumber] += Math.Round(DeltaTime(_timePerformanceRun, "ms") / TreeRoot.TicksPerSecond);
                    else
                        _lineRuntimeSum[_lineNumber] += DeltaTime(_timePerformanceRun, "ms");
                }

                var _sortedDictionary = from _entry in _lineRuntimeSum
                                        orderby _entry.Value ascending
                                        select _entry;

                var _index = _sortedDictionary.ToList().IndexOf(new KeyValuePair<int, double>(_lineNumber, _lineRuntimeSum[_lineNumber])) + 1;
                float _count = _sortedDictionary.Count();

                var _highlightedEntries = 6;
                var _factor = _index / _count;
                _factor = (_index < _count - _highlightedEntries - 1)
                                ? Math.Max(0, _factor - 0.4f)
                                : _factor;

                var _newR = _colorGrey.R + (_colorGreen.R - _colorGrey.R) * _factor;
                var _newG = _colorGrey.G + (_colorGreen.G - _colorGrey.G) * _factor;
                var _newB = _colorGrey.B + (_colorGreen.B - _colorGrey.B) * _factor;

                var _colorPercentage = Color.FromRgb(Convert.ToByte(_newR),
                                                     Convert.ToByte(_newG),
                                                     Convert.ToByte(_newB));

                var _pre = (_option == null)
                                ? null
                                : _option.Replace(">", "   ")
                                         .Replace("#", null);

                Logging.WriteDiagnostic(_colorPercentage, _pre + "{0}: [{1}] {2}", _lineNumber,
                                                                                   Math.Round(_lineRuntimeSum[_lineNumber]),
                                                                                   DeltaTime(_timePerformanceRun, "ms"));
            }
            _timePerformanceRun = DateTime.Now;
        }

        private static LocalPlayer Me => StyxWoW.Me;

        private Vector3 _lastLocation = Vector3.Zero;
        private Vector3 _lastStuckLocation = Vector3.Zero;
        private double _stuckCount = 0;
        private double _lastStuckCount = 0;
        private double _intervalLastFlightorLog = 0;
        private int _stableStuckCount = 0;
        private int _disableAddonsSetting = -1;

        private string _lastFrameText = null;
        private string _lastObjective = null;
        private string _lastLogEntry = null;
        private string _mountErrorMessage = Lua.GetReturnVal<string>("return SPELL_FAILED_NO_MOUNTS_ALLOWED", 0);
        private string _startsQuestText = Lua.GetReturnVal<string>("return ITEM_STARTS_QUEST", 0);
        private Dialog DialogState = Dialog.None;
        private Stuck StuckState = Stuck.Off;
        private Hearth HearthState = Hearth.Off;
        private Motionless MotionlessState = Motionless.Off;

        private DateTime _timeTPSchanged = DateTime.MinValue;
        private DateTime _timePreMotionless = DateTime.Now;
        private DateTime _timeDied = DateTime.MinValue;
        private DateTime _timeUnghost = DateTime.MinValue;
        private DateTime _timeGarrisonCacheInteracted = DateTime.MinValue;
        private DateTime _timeRun = DateTime.MinValue;
        private DateTime _timeLastJump = DateTime.MinValue;
        private DateTime _timePerformanceRun = DateTime.MinValue;
        private DateTime _timePerformanceRunSum = DateTime.MinValue;
        private DateTime _timeFrameChange = DateTime.MinValue;
        private DateTime _timeStartFlyingMountRun = DateTime.MinValue;
        private DateTime _timeAtTransportBugLocation = DateTime.MinValue;
        private DateTime _timeMovingInto = DateTime.MinValue;
        private DateTime _timeMountError = DateTime.MinValue;
        private DateTime _timeFlightorFailed = DateTime.MinValue;
        private DateTime _timeLastLocation = DateTime.MinValue;
        private DateTime _timeLastTurn = DateTime.MinValue;
        private DateTime _timeLastMoved = DateTime.MinValue;
        private DateTime _timeLastObjectiveChange = DateTime.MinValue;
        private DateTime _timeReload = DateTime.MinValue;
        private DateTime _timeTransportDismount = DateTime.MinValue;
        private DateTime _timeInterrupted = DateTime.MinValue;
        private DateTime _timeSucceeded = DateTime.MinValue;
        private DateTime _timeMountUp = DateTime.MinValue;
        private DateTime _timeDialogOpened = DateTime.MinValue;
        private DateTime _timeDialogClosed = DateTime.MinValue;
        private DateTime _timePathDistancesRequestCount = DateTime.MinValue;
        private DateTime _timeEnterVehicle = DateTime.MinValue;
        private DateTime _timeExitVehicle = DateTime.MinValue;
        private DateTime _timeLastCTM = DateTime.MinValue;
        private DateTime _timeMapChanged = DateTime.MinValue;
        private DateTime _timeLastFlightorLog = DateTime.MinValue;
        private DateTime _timeGettingOff = DateTime.MinValue;

        private double _deltaNextJump = 0;
        private double _failedReviveCount = 0;
        private double _failedMountUpCount = 0;
        private double _failedPathDistancesRequestCount = 0;

        private double _accountExpansionLevel = Lua.GetReturnVal<int>("return GetAccountExpansionLevel()", 0);
        private double _renderScale = Lua.GetReturnVal<double>("return GetCVar('RenderScale')", 0);
        private double _maxFps = Lua.GetReturnVal<double>("return GetCVar('MaxFPS')", 0);
        private double _maxFpsBk = Lua.GetReturnVal<double>("return GetCVar('maxFPSBk')", 0);
        private double _autoLoot = Lua.GetReturnVal<double>("return GetCVar('AutoLootDefault')", 0);
        private double _spec = Lua.GetReturnVal<int>("return GetSpecialization()", 0);
        private bool _declineGuildInvite = Lua.GetReturnVal<bool>("return GetAutoDeclineGuildInvites()", 0);
        private bool _init = false;

        private bool _checkedItems = false;
        private bool _checkedQuests = false;
        private bool _checkedGatheringSkills = false;
        private bool _checkedArchaeologySkill = false;
        private bool _meleeDismount = false;
        private bool _reloadProfile = false;
        private bool _profileChange = false;
        private string _lastProfile = null;

        private SkillLine _lowestNodeSkill = 0;
        private ReadOnlyCollection<uint> _completedQuests = QuestLog.Instance.GetCompletedQuests();
        private List<PlayerQuest> _allQuests = QuestLog.Instance.GetAllQuests();
        private Dictionary<WoWGuid, double> _ignoredNodeGuids = new Dictionary<WoWGuid, double>();
        private Dictionary<WoWGuid, DateTime> _ignoredFlightmasters = new Dictionary<WoWGuid, DateTime>();
        private Dictionary<int, double> _lineRuntimeSum = new Dictionary<int, double>();
        private Dictionary<int, bool> _toys = new Dictionary<int, bool>();
        private List<object[]> _stuckLocations = new List<object[]>();
        private List<object[]> _trainerData = new List<object[]>();

        private Random _rnd = new Random();
        private Composite _KOM;
        private WaitTimer _nodeTimer = new WaitTimer(TimeSpan.FromSeconds(5));
        private WaitTimer _logTimer = new WaitTimer(TimeSpan.FromSeconds(10));
        private WaitTimer _nodeRefreshTimer = new WaitTimer(TimeSpan.FromMilliseconds(1000));

        private readonly SoundPlayer _soundPlayer = new SoundPlayer();

        public override void OnButtonPress()
        {
            if (_gui == null || _gui.IsDisposed || _gui.Disposing)
                _gui = new GUI();
            if (_gui != null || _gui.IsDisposed)
                _gui.ShowDialog();
        }

        private async Task<bool> Sleep(int _maxSleep = 1000, int _minSleep = 250)
        {
            await Coroutine.Sleep(_rnd.Next(_minSleep, _maxSleep));
            return true;
        }

        private async Task<bool> ClickButton(string _buttonName, int _sleep = 1500)
        {
            await Sleep(_sleep);
            if (IsFrameVisible($"{_buttonName}"))
                Lua.DoString($"{_buttonName}:Click()");

            return true;
        }

        private List<string> _defaultFrames = new List<string>
        {
            "QuestFrame",
            "CinematicFrame",
            "GossipFrame",
            "TaxiFrame",
            "MailFrame",
            "LootFrame",
            "TrainerFrame",
            "FlightMapFrame"
        };

        private bool IsFrameVisible(string _frameName)
        {
            if (_defaultFrames.Contains(_frameName) &&
                DeltaTime(_timeDialogOpened, "s") > 10)
                return false;

            switch (_frameName)
            {
                case "CinematicFrame":
                    return StyxWoW.WoWClient.IsCinematicPlaying;
                    break;
                case "QuestFrame":
                    return QuestFrame.Instance != null && QuestFrame.Instance.IsVisible;
                    break;
                case "GossipFrame":
                    return GossipFrame.Instance != null && GossipFrame.Instance.IsVisible;
                    break;
                case "MailFrame":
                    return MailFrame.Instance != null && MailFrame.Instance.IsVisible;
                    break;
                case "LootFrame":
                    return LootFrame.Instance != null && LootFrame.Instance.IsVisible;
                    break;
                case "TrainerFrame":
                    return TrainerFrame.Instance != null && TrainerFrame.Instance.IsVisible;
                    break;
                case "TaxiFrame":
                    return TaxiFrame.Instance != null && TaxiFrame.Instance.IsVisible;
                    break;
                case "FlightMapFrame":
                    return FlightMapFrame.Instance != null && FlightMapFrame.Instance.IsVisible;
                    break;
                default:
                    return new Frame(_frameName).IsVisible;
                    break;
            }
        }

        /*
        Assembly _assembly = typeof(Styx.CommonBot.Frames.Frame).Assembly;
        Type _type = _assembly.GetType("Styx.CommonBot.Frames." + _frameName);

        if(_type != null)
        {
            dynamic _obj = Activator.CreateInstance(_type);
            return _obj != null && _obj.IsVisible;
        */

        private string _mapPosition
        {
            get
            {
                var _mapCoordinates = Lua.GetReturnValues("return GetPlayerMapPosition('player')");
                if (_mapCoordinates.Count == 0)
                {
                    DevLog("IsInInstance: {0}", Me.IsInInstance);
                    return " -/- ";
                }

                var _x = Math.Round(Convert.ToDouble(_mapCoordinates[0]) * 100f, 1);
                var _y = Math.Round(Convert.ToDouble(_mapCoordinates[1]) * 100f, 1);

                return $"{_x}, {_y}";
            }
        }

        private double DeltaTime(DateTime _time, string _unit)
        {
            var _delta = DateTime.Now.Subtract(_time);

            switch (_unit)
            {
                case "ms":
                    return _delta.TotalMilliseconds;
                    break;
                case "s":
                    return _delta.TotalSeconds;
                    break;
                case "m":
                    return _delta.TotalMinutes;
                    break;
                case "h":
                    return _delta.TotalHours;
                    break;
            }

            return 0;
        }

        private WoWGameObject _deathGate
        {
            get
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                            .FirstOrDefault(o => o.Entry == 190942 &&
                                                 o.IsValid &&
                                                 o.CreatedByGuid == Me.Guid);
            }
        }

        private WoWGameObject _garrisonCache
        {
            get
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                            .FirstOrDefault(o => ((Me.IsHorde)
                                                        ? o.Entry == 237191 || o.Entry == 237724
                                                        : o.Entry == 236916 || o.Entry == 237722) &&
                                                 o.IsValid);
            }
        }

        private void Reset(bool _fullReset = false)
        {
            DevLog("Reset{0}", (_fullReset)
                                    ? " > FullReset"
                                    : "");

            if (_fullReset)
            {
                _init = false;

                _ignoredNodeGuids.Clear();
                _ignoredFlightmasters.Clear();
                _stuckLocations.Clear();

                _renderScale = Lua.GetReturnVal<double>("return GetCVar('RenderScale')", 0);
                _maxFps = Lua.GetReturnVal<double>("return GetCVar('MaxFPS')", 0);
                _maxFpsBk = Lua.GetReturnVal<double>("return GetCVar('maxFPSBk')", 0);
                _spec = GetSpecIndex();

                _completedQuests = QuestLog.Instance.GetCompletedQuests();
                _allQuests = QuestLog.Instance.GetAllQuests();
            }

            HearthState = Hearth.Off;
            StuckState = Stuck.Off;
            MotionlessState = Motionless.Off;

            DialogState = Dialog.None;

            _checkedItems = false;
            _checkedQuests = false;
            _checkedGatheringSkills = false;

            _stuckCount = 0;
            _lastStuckCount = 0;
            _stableStuckCount = 0;

            _failedMountUpCount = 0;
            _failedPathDistancesRequestCount = 0;

            _timeAtTransportBugLocation = DateTime.MinValue;
            _timeFlightorFailed = DateTime.MinValue;
            _timeLastMoved = DateTime.MinValue;
            _timeLastObjectiveChange = DateTime.MinValue;
            _timeDialogOpened = DateTime.MinValue;
            _timeDialogClosed = DateTime.MinValue;

            _lastProfile = null;
        }

        private bool breakOut
        {
            get
            {
                return !Me.IsAlive || !_init || (Me.Combat && Me.HealthPercent <= 70);
            }
        }

        private WoWUnit GetNpc(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                        u.Entry == _id &&
                        u.IsValid &&
                        u.IsAlive
                     orderby
                        u.Distance
                     select u);

                if (_units.Count() > 0)
                    return _units.First();
            }

            return null;
        }

        private WoWGameObject GetObject(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                IEnumerable<WoWGameObject> _objects =
                    (from o in ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                     where
                             o.Entry == _id &&
                             o.CanUseNow() &&
                             o.IsValid
                     orderby
                        o.Distance
                     select o);

                if (_objects.Count() > 0)
                    return _objects.First();
            }

            return null;
        }

        private WoWItem GetItem(int _id)
        {
            IEnumerable<WoWItem> _items =
                (from i in Me.BagItems
                 where
                         i.Entry == _id &&
                         i.IsValid
                 orderby
                    i.StackCount
                 select i);

            return (_items.Count() > 0)
                        ? _items.First()
                        : null;
        }

        private int GetItemNum(int _id)
        {
            return (int)Me.BagItems.Where(i => i.Entry == _id &&
                                               i.IsValid)
                                   .Sum(i => i.StackCount);
        }

        private int _garrisonResourcesCount
        {
            get
            {
                return (int)WoWCurrency.GetCurrencyByType(WoWCurrencyType.GarrisonResources).Amount;
            }
        }

        private int _gatheringItemCount
        {
            get
            {
                return (int)Me.BagItems
                    .Where(i => i.IsValid &&
                                (i.ItemInfo.TradeGoodsClass == WoWItemTradeGoodsClass.MetalStone ||
                                 i.ItemInfo.TradeGoodsClass == WoWItemTradeGoodsClass.Herb))
                    .Sum(i => i.StackCount);
            }
        }

        private WoWGameObject _mailbox
        {
            get
            {
                IEnumerable<WoWGameObject> _objects =
                    (from o in ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                     where
                        o.SubType == WoWGameObjectType.Mailbox &&
                        o.IsValid &&
                        o.CanUse()
                     orderby
                        o.Distance
                     select o);

                return (_objects.Count() > 0)
                            ? _objects.First()
                            : null;
                /*
                var _factionId = (_first != null)
                                        ? _first.FactionTemplate.Faction.Id
                                        : 0;
                var _moreThanNeutral = (_factionId != 0)
                                            ? Me.GetReputationLevelWith(_factionId) >= WoWUnitReaction.Friendly
                                            : false;
                var _isMyFaction = (Me.IsHorde)
                                        ? _factionId == 66 || _moreThanNeutral
                                        : _factionId == 189 || _moreThanNeutral;
                return (_first != null && _isMyFaction)
                            ? _first
                            : null;
                */
            }
        }

        private MailFrame.InboxMailItem _newMail
        {
            get
            {
                IEnumerable<MailFrame.InboxMailItem> _mails =
                    (from m in MailFrame.Instance.GetAllMails()
                     where
                        (!m.WasRead ||
                        m.Copper > 0 ||
                        m.ItemCount > 0) &&
                        m.ItemCount < Me.FreeBagSlots
                     select m);

                return (_mails.Count() > 0)
                                ? _mails.First()
                                : null;
            }
        }

        private WoWUnit _lootableMob
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                        u.CanLoot &&
                        u.WithinInteractRange &&
                        u.Lootable &&
                        !u.Looting &&
                        u.IsValid
                     orderby
                        u.Distance
                     select u);

                return (_units.Count() > 0)
                                ? _units.First()
                                : null;
            }
        }

        private WoWUnit _interactableMob
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                        !u.CanLoot &&
                        u.CanSkin &&
                        u.CanInteractNow &&
                        // u.WithinInteractRange &&
                        !u.Looting &&
                        !u.Lootable &&
                        u.IsValid
                     orderby
                        u.Distance
                     select u);

                return (_units.Count() > 0)
                                ? _units.First()
                                : null;
            }
        }

        private float GetPathDistance(WoWObject _object)
        {
            int i = 1;
            float _pathDistance = 0;
            while (i <= 5)
            {
                _pathDistance = Navigator.LookupPathInfo(_object).Distance;
                if (_pathDistance != 0)
                    return _pathDistance;
                i++;
            }

            return 0;
        }

        private WoWGameObject _node
        {
            get
            {
                _lowestNodeSkill = (_lowestNodeSkill == 0)
                                        ? GetLowestNodeSkill()
                                        : _lowestNodeSkill;

                if (_lowestNodeSkill == 0)
                    return null;

                var _nodes = ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                                .Where(o => (o.IsMineral || o.IsHerb) &&
                                             o.CanLoot &&
                                             o.SubType == WoWGameObjectType.GatheringNode &&
                                             (
                                                !_ignoredNodeGuids.ContainsKey(o.Guid) ||
                                                _ignoredNodeGuids[o.Guid] > 0
                                             ));

                if (_nodes.Count() == 0)
                    return null;

                /*
                foreach (var _node in _nodes)
                {
                    if (!IsNodeValuable(_node))
                        _ignoredNodeGuids[_node.Guid] = 0;
                }
                */

                _nodes = _nodes.Where(o => IsNodeValuable(o));
                if (_nodes.Count() == 0)
                    return null;

                _nodes = (Me.IsFlying || (Flightor.CanFly && Me.IsOutdoors))
                                ? _nodes.OrderBy(o => o.Distance)
                                : _nodes.Where(o => GetPathDistance(o) > 0 &&
                                                    GetPathDistance(o) < 150)
                                        .OrderBy(o => GetPathDistance(o));

                if (_nodes.Count() == 0)
                    return null;

                var _herbs = _nodes.Where(o => o.IsHerb);
                var _minerals = _nodes.Where(o => o.IsMineral);

                if (GetLowestNodeSkill() == SkillLine.Herbalism && _herbs.Count() > 0 && !_nodes.First().IsHerb)
                    return _herbs.First();
                else if (GetLowestNodeSkill() == SkillLine.Mining && _minerals.Count() > 0 && !_nodes.First().IsMineral)
                    return _minerals.First();

                return _nodes.First();
            }
        }

        private async Task<bool> DruidForm()
        {
            RuntimeLog(">>>");

            if (Me.Class != WoWClass.Druid)
                return false;

            RuntimeLog(">>>");

            if (Flightor.CanFly && _preventFlying)
                return false;

            RuntimeLog(">>>");

            if (!Mount.CanMount() && (Me.Shapeshift == ShapeshiftForm.Cat ||
                                      !SpellManager.HasSpell(768)))
                return false;

            RuntimeLog(">>>");

            if (Mount.CanMount() && (Me.Shapeshift == ShapeshiftForm.Travel ||
                                     !SpellManager.HasSpell(783)))
                return false;

            RuntimeLog(">>>");

            var _flyingMountSpellId = CharacterSettings.Instance.FlyingMountSpellId;
            if (Mount.CanMount() && Flightor.CanFly && _flyingMountSpellId != 0) //&& _flyingMountSpellId != 783 )
                return false;

            RuntimeLog(">>>");

            var _groundMountSpellId = CharacterSettings.Instance.GroundMountSpellId;
            if (Mount.CanMount() && !Flightor.CanFly && _groundMountSpellId != 0) //&& _groundMountSpellId != 783 )
                return false;

            RuntimeLog(">>>");

            await CommonCoroutines.StopMoving();
            if (Mount.CanMount())
            {
                DevLog("DruidForm > TravelForm");
                await Coroutine.Wait(5000, () => SpellManager.CanCast(783));
                SpellManager.CastSpellById(783);
                await Coroutine.Wait(2000, () => Me.Shapeshift == ShapeshiftForm.Travel);

                if (Flightor.CanFly)
                    await Move("JF", 250, true);

                return true;
            }
            else
            {
                DevLog("DruidForm > CatForm");
                await Coroutine.Wait(5000, () => SpellManager.CanCast(768));
                SpellManager.CastSpellById(768);
                await Coroutine.Wait(2000, () => Me.Shapeshift == ShapeshiftForm.Cat);

                return true;
            }
        }

        private bool _preventFlying
        {
            get
            {
                return DeltaTime(_timeFlightorFailed, "s") <= 30;
            }
        }

        private bool _preventMount
        {
            get
            {
                return DeltaTime(_timeMovingInto, "s") < 5 ||
                       DeltaTime(_timeMountError, "s") < 10 ||
                       (_failedMountUpCount >= 2 && DeltaTime(_timeMountUp, "ms") < 6000) ||
                       (
                            DeltaTime(_timeTransportDismount, "s") < 30 &&
                            !Me.IsOnTransport
                       );
            }
        }

        private async Task<bool> MountUp(Nullable<Vector3> _location = null, double _distanceTreshold = 20, bool _forceGroundMount = false)
        {
            RuntimeLog(">>");

            if (_preventMount)
                return false;

            RuntimeLog(">>");

            if (Me.Mounted && !Mount.Current.IsFlyingMount && _forceGroundMount)
                return false;

            if (Me.Mounted && !Flightor.CanFly && !Mount.Current.IsFlyingMount)
                return false;

            RuntimeLog(">>");

            if (Me.Mounted && Flightor.CanFly && Mount.Current.IsFlyingMount)
            {
                if (_forceGroundMount)
                    await CommonCoroutines.Dismount(null, false);
                return false;
            }

            RuntimeLog(">>");

            if (Me.Class == WoWClass.Druid && !Mount.CanMount() &&
                (
                    Me.Shapeshift == ShapeshiftForm.Cat ||
                    !SpellManager.CanCast(768)
                ))
                return false;

            RuntimeLog(">>");

            if (Me.Class == WoWClass.Druid && Me.Shapeshift == ShapeshiftForm.Travel)
                return false;

            RuntimeLog(">>");

            if (Me.Class != WoWClass.Druid && (Me.Combat || !Mount.CanMount()))
                return false;

            RuntimeLog(">>");

            if (_location != null && ((Vector3)_location).Distance(Me.Location) < _distanceTreshold)
                return false;

            RuntimeLog(">>");

            if (!await DruidForm() || _forceGroundMount)
            {
                if (Flightor.CanFly && !_preventFlying && !_forceGroundMount)
                {
                    DevLog("MountUp > FlyingMount");
                    while ((!Me.Mounted || !Mount.Current.IsFlyingMount) && !breakOut)
                    {
                        if (_preventMount)
                        {
                            DevLog("PreventMount: {0}", _preventMount);
                            return false;
                        }

                        DevLog("WHILE: MountUp > FlyingMount");
                        await CommonCoroutines.StopMoving();
                        await CommonCoroutines.SummonFlyingMount();
                        await Coroutine.Wait(2000, () => Me.Mounted || _preventMount);
                        await Coroutine.Yield();
                    }
                    await Move("J", 200);

                    return true;
                }
                else
                {
                    DevLog("MountUp > GroundMount");
                    while (!Me.Mounted && Mount.CanMount() && !breakOut)
                    {
                        if (_preventMount)
                        {
                            DevLog("PreventMount: {0}", _preventMount);
                            return false;
                        }

                        DevLog("WHILE: MountUp > GroundMount");
                        await CommonCoroutines.StopMoving();
                        await CommonCoroutines.SummonGroundMount();
                        await Coroutine.Wait(2000, () => Me.Mounted || _preventMount);
                        await Coroutine.Yield();
                    }

                    return true;
                }
            }

            _stuckCount = 0;
            return true;
        }

        private bool IsQuestItem(WoWItem _item)
        {
            if (_item == null)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null)
                return false;

            var _questId = _itemInfo.BeginQuestId;
            if (_questId != 0)
            {
                var _quest = Quest.FromId((uint)_questId);
                if (_quest.Level > 0 &&
                   _quest.Level <= Me.Level - 5)
                    return true;
            }

            var _itemClass = _itemInfo.ItemClass;
            if (_itemClass == null)
                return false;

            if (_itemClass != WoWItemClass.Quest)
                return false;

            if (_itemInfo.Level == 1)
                return false;

            if (_itemInfo.Level > Me.Level - 5)
                return false;

            /*

            if (_itemClass != WoWItemClass.Quest &&
                _itemClass != WoWItemClass.Miscellaneous)
                return false;

            if (_itemClass == WoWItemClass.Miscellaneous &&
                !StartsQuest(_item))
                return false;
            */

            return true;
        }

        private bool IsOpenable(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            if (_item.Entry == 35792)   // QuestItem that needs to be opened by questing profile
                return false;

            if (_item.IsOpenable)
                return true;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _isOpenable = _effects.Any(e => e != null &&
                                                e.Spell != null &&
                                                e.Spell.SpellEffects.Count() > 0 &&
                                                e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.CreateRandomItem));
            if (_isOpenable)
                return true;

            return false;
        }

        private bool IsCombinable(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _effect = _item.Effects.FirstOrDefault(e => e.TriggerType == ItemEffectTriggerType.OnUse);
            if (_effect == null)
                return false;

            var _spell = _effect.Spell;
            if (_spell == null || _spell.CreatesItemId == 0)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null || _itemInfo.MaxStackSize != 200)
                return false;

            var _createdItemInfo = ItemInfo.FromId(_spell.CreatesItemId);
            if (_createdItemInfo == null || !_createdItemInfo.IsCraftingReagent)
                return false;

            var _reagents = _spell.InternalInfo.SpellReagents;
            if (_reagents == null || !_reagents.HasValue)
                return false;

            var _requiredReagentCount = (uint)_reagents.Value.ReagentCount[0] + 1;
            var _itemCount = Me.BagItems
                                .Where(i => i.Entry == _item.Entry)
                                .Sum(i => i.StackCount);
            if (_itemCount >= _requiredReagentCount)
                return true;

            return false;
        }

        /*
        private bool _startNewExpansion
        {
            get
            {
                return ProfileManager.XmlLocation != null && ProfileManager.XmlLocation.ToLower().Contains("questing profile pack") &&
                       ((Me.Level == 58 && (Me.CurrentMap.ExpansionId != 1 && !_inCapital)) ||
                        (Me.Level == 68 && Me.CurrentMap.ExpansionId == 1) ||
                        (Me.Level == 80 && Me.CurrentMap.ExpansionId == 2) ||
                        (Me.Level == 85 && !_inCapital && ((Me.IsHorde)
                                                                ? !IsQuestCompleted(29611)
                                                                : !IsQuestCompleted(29547))) ||
                        (Me.Level == 90 && Me.CurrentMap.ExpansionId == 4) ||
                        (Me.Level == 100 && Me.CurrentMap.ExpansionId == 5 && _accountExpansionLevel >= 6));
            }
        }
        */

        private async Task<bool> HandleTPS()
        {
            if (Settings.FpsStabiliser_FpsThreshold != 0)
            {
                if (DeltaTime(_timeTPSchanged, "ms") >= 100)
                {
                    if (StyxWoW.WoWClient.Fps <= Settings.FpsStabiliser_FpsThreshold)
                    {
                        if (TreeRoot.TicksPerSecond > Settings.FpsStabiliser_MinTps)
                        {
                            TreeRoot.TicksPerSecond = Convert.ToByte(Settings.FpsStabiliser_MinTps);
                            _timeTPSchanged = DateTime.Now;
                        }
                    }
                    else if (TreeRoot.TicksPerSecond < Settings.FpsStabiliser_MaxTps &&
                             DeltaTime(_timeTPSchanged, "ms") >= 500)
                    {
                        TreeRoot.TicksPerSecond += 1;
                        _timeTPSchanged = DateTime.Now;
                    }
                }
            }

            if (DeltaTime(_timeRun, "ms") < 1000)
                return false;

            return true;
        }

        private async Task<bool> MoveTo(Vector3 _location, bool _useTaxi = false, bool _preventBlackspot = false)
        {

            if (Me.HasAura("StagForm") && Flightor.CanFly)
                Me.GetAuraByName("Travel Form").TryCancelAura();

            await MountUp(_location);
            await HandleJump();
            await HandleTPS();
            await HandleStuck();

            if (_preventBlackspot)
                await HandleNavigationBlackspots();

            // Prevent Motionless
            _timeLastMoved = DateTime.Now;
            _timeLastObjectiveChange = DateTime.Now;

            if (_useTaxi)
                await TakeTaxi(_location);

            var _moveToParameters = new MoveToParameters(_location);
            // _moveToParameters.AllowFlightPaths = _useTaxi;3

            var _flyToParameters = new FlyToParameters(_location);
            _flyToParameters.CheckIndoors = true;
            _flyToParameters.MinHeight = 20;
            _flyToParameters.GroundNavParameters.AllowFlightPaths = _useTaxi;

            /*
            var _moveResult = (Flightor.CanFly && Me.IsOutdoors && !Me.Combat)
                                        ? Flightor.MoveTo(_flyToParameters)
                                        : Navigator.MoveTo(_location);

            if (_moveResult == MoveResult.PathGenerationFailed)
                return false;
            */

            if (Flightor.CanFly && DeltaTime(_timeFlightorFailed, "s") > 30)
            {
                if (Flightor.MoveTo(_flyToParameters) == MoveResult.PathGenerationFailed)
                {
                    _timeFlightorFailed = DateTime.Now;
                    await CommonCoroutines.LandAndDismount(null, true);
                }
            }
            else
            {
                if (Navigator.MoveTo(_moveToParameters) == MoveResult.PathGenerationFailed)
                    return false;
            }

            return true;
        }

        private async Task<bool> PreventTaxiRefreshing()
        {
            if (!Settings.General_PreventTaxi)
                return false;

            if (KnownFlightNodesManager.ElapsedSinceUpdate.TotalHours > 1)
                KnownFlightNodesManager.UpdateKnownFlightNodes(new byte[0]);

            return true;
        }

        private async Task<bool> HandleTaxiNodes()
        {
            await PreventTaxiRefreshing();

            if (_flightmaster == null)
                return false;

            if (_flightmaster.InteractType == WoWInteractType.TaxiNotEligible)
                return false;

            var _elapsedTime = KnownFlightNodesManager.ElapsedSinceUpdate;
            if (_flightmaster.InteractType != WoWInteractType.TaxiPathAvailable &&
                _elapsedTime.TotalHours < 23)
                return false;

            if (Me.IsFlying && GameWorld.TraceLine(Me.Location, _flightmaster.Location, TraceLineHitFlags.Collision))
                return false;

            if (_ignoredFlightmasters.ContainsKey(_flightmaster.Guid) &&
                DeltaTime(_ignoredFlightmasters[_flightmaster.Guid], "m") < 60)
                return false;

            if (_inShrine)
                return false;

            if (_flightmaster.InteractType == WoWInteractType.TaxiPathAvailable)
            {
                Log("Getting taxi node");
            }
            else
            {
                Log("Refreshing taxi nodes");
                DevLog("ElapsedTime: {0}h", Math.Round(_elapsedTime.TotalHours));
                DevLog("InteractType: {0}", _flightmaster.InteractType);
            }

            var _flightmasterLocation = _flightmaster.Location;
            var _flightmasterEntry = _flightmaster.Entry;
            var _flightmasterName = _flightmaster.Name;
            var _flightmasterGuid = _flightmaster.Guid;
            var _startTime = DateTime.Now;

            while (!IsFrameVisible("TaxiFrame") && !IsFrameVisible("FlightMapFrame") && !breakOut)
            {
                DevLog("WHILE: HandleTaxiNodes");

                if (DeltaTime(_startTime, "s") > 40 &&
                    (_flightmaster == null || !_flightmaster.WithinInteractRange))
                {
                    DevLog("     MaxTime reached");
                    DevLog("     Blacklisting for 1 hour");
                    DevLog("     Name: {0} (Id: {1})", _flightmasterName, _flightmasterEntry);
                    if (!_ignoredFlightmasters.ContainsKey(_flightmasterGuid))
                        _ignoredFlightmasters.Add(_flightmasterGuid, DateTime.Now);
                    else
                        _ignoredFlightmasters[_flightmasterGuid] = DateTime.Now;
                    return false;
                }

                if (_flightmaster == null || !_flightmaster.WithinInteractRange)
                {
                    if (!await MoveTo(_flightmasterLocation))
                    {
                        DevLog("     PathGenerationFailed");
                        DevLog("     Blacklisting for 1 hour");
                        DevLog("     Name: {0} (Id: {1})", _flightmasterName, _flightmasterEntry);
                        if (!_ignoredFlightmasters.ContainsKey(_flightmasterGuid))
                            _ignoredFlightmasters.Add(_flightmasterGuid, DateTime.Now);
                        else
                            _ignoredFlightmasters[_flightmasterGuid] = DateTime.Now;
                        return false;
                    }
                }
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _flightmaster.Interact();

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _taxiEntry = GossipFrame.Instance.GossipOptionEntries.FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Taxi);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_taxiEntry.Index);
                        await Coroutine.Wait(5000, () => IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"));
                    }
                }
                await Coroutine.Yield();
            }
            if (IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"))
                KnownFlightNodesManager.UpdateKnownFlightNodes(WoWTaxi.GetKnownFlightNodesMask());

            return true;
        }

        private async Task<bool> HandleKillBetweenHotspots()
        {
            if (!Settings.Quest_KillBetweenHotspots || !GlobalSettings.Instance.KillBetweenHotspots)
                return false;

            DevLog("KillBetweenHotspots");
            GlobalSettings.Instance.KillBetweenHotspots = false;
            GlobalSettings.Instance.Save();

            return true;
        }

        private async Task<bool> HandleNewMails()
        {
            RuntimeLog(">");

            if (!Settings.Bag_GetMails)
                return false;

            RuntimeLog(">");

            if (_mailbox == null)
                return false;

            RuntimeLog(">");

            if (!IsFrameVisible("MiniMapMailFrame") && _newMail == null)
                return false;

            RuntimeLog(">");

            if (IsFrameVisible("MailFrame"))
                return false;

            RuntimeLog(">");

            if (Me.FreeBagSlots <= 5)
                return false;

            Log("Getting new mails");
            var _mailboxLoc = _mailbox.Location;

            while (!IsFrameVisible("MailFrame") && !breakOut)
            {
                DevLog("WHILE: HandleNewMails > GoTo");

                if (_mailbox == null || !_mailbox.WithinInteractRange)
                    await MoveTo(_mailboxLoc);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _mailbox.Interact();
                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("MailFrame"));
                }

                await Coroutine.Yield();
            }
            await Coroutine.Wait(2000, () => _newMail != null);

            var _autoEquip = CharacterSettings.Instance.AutoEquip;
            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = false;

            while (_newMail != null && IsFrameVisible("MailFrame") && !breakOut)
            {
                DevLog("WHILE: HandleNewMails > Read");

                var _index = _newMail.Index;

                await Sleep();
                await MailFrame.Instance.GetMailAttachmentsCoroutine(_index);
                if (_newMail != null && _newMail.Index == _index)
                {
                    while (!IsFrameVisible("MailItem" + _index + "Button"))
                    {
                        await ClickButton("InboxNextPageButton");
                        await Coroutine.Yield();
                    }
                    await ClickButton($"MailItem{_index}Button");
                    await Coroutine.Wait(2 * 1000, () => IsFrameVisible("OpenMailFrame"));
                    Lua.DoString("OpenMailFrame:Hide()");
                }

                await Coroutine.Yield();
            }

            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = true;

            return true;
        }

        private async Task<bool> Move(string _direction, int _duration = 0, bool _waitUntilFinished = false)
        {
            WoWMovement.MovementDirection _movementDirection = WoWMovement.MovementDirection.None;

            if (_direction.Contains("A"))
                _movementDirection |= WoWMovement.MovementDirection.AutoRun;
            if (_direction.Contains("B"))
                _movementDirection |= WoWMovement.MovementDirection.Backwards;
            if (_direction.Contains("D"))
                _movementDirection |= WoWMovement.MovementDirection.Descend;
            if (_direction.Contains("F"))
                _movementDirection |= WoWMovement.MovementDirection.Forward;
            if (_direction.Contains("J"))
                _movementDirection |= WoWMovement.MovementDirection.JumpAscend;
            if (_direction.Contains("L"))
                _movementDirection |= WoWMovement.MovementDirection.StrafeLeft;
            if (_direction.Contains("l"))
                _movementDirection |= WoWMovement.MovementDirection.TurnLeft;
            if (_direction.Contains("R"))
                _movementDirection |= WoWMovement.MovementDirection.StrafeRight;
            if (_direction.Contains("r"))
                _movementDirection |= WoWMovement.MovementDirection.TurnRight;

            if (_duration > 0)
                WoWMovement.Move(_movementDirection, _duration);
            else
                WoWMovement.Move(_movementDirection);

            if (_waitUntilFinished)
                await Coroutine.Sleep(_duration);

            return true;
        }

        private async Task<bool> Turn(string _direction, int _duration, int _turnPause)
        {
            if (DeltaTime(_timeLastTurn, "ms") < _turnPause)
                return false;

            _timeLastTurn = DateTime.Now;
            await Move($"{_direction.ToLower()}", _duration);

            return true;
        }

        private double _defaultSpeed
        {
            get
            {
                double _factor = 1;

                if (Me.Mounted)
                {
                    if (Me.IsFlying)
                        switch (Me.GetSkill(SkillLine.Riding).CurrentValue)
                        {
                            case 225:
                                _factor = 2.5;
                                break;
                            case 300:
                                _factor = 3.8;
                                break;
                            case 375:
                                _factor = 4.1;
                                break;
                        }
                    else
                        switch (Me.GetSkill(SkillLine.Riding).CurrentValue)
                        {
                            case 75:
                                _factor = 1.6;
                                break;
                            default:
                                _factor = 2;
                                break;
                        }
                }

                if (Me.IsSwimming)
                {
                    _factor = (Me.Class == WoWClass.Druid && Me.Shapeshift == ShapeshiftForm.Aqua)
                                        ? 1.3
                                        : 0.5;
                }

                return _factor * 7;
            }
        }

        private List<int> _stuckIntervals = new List<int>
            {
                5,
                10,
                14,
                18,
                23,
                68, // RELOAD
                    // HEARTH
            };

        private async Task<bool> JumpToHearthStage()
        {
            DevLog("Hearth disabled: {0}", Settings.Hearth_Disable);

            if (InVehicle())
            {
                DevLog("WHILE: JumpToHearthStage > VehicleExit");
                Lua.DoString("VehicleExit()");
                await Coroutine.Sleep(2000);
            }

            if (MotionlessState == Motionless.Off)
            {
                Log("     Stage 7");
                StuckState = Stuck.Off;
            }
            else
            {
                Log("     Stage 3");
                MotionlessState = Motionless.Off;
            }

            HearthState = Hearth.On;
            return true;
        }

        private double _speed
        {
            get
            {
                var _distance = Me.Location.Distance(_lastLocation);
                var _time = DeltaTime(_timeLastLocation, "ms");
                var _speed = _distance / _time * 1000f;

                return Math.Round(_speed, 1);
            }

        }

        private async Task<bool> IsStuck()
        {
            if (WoWMovement.ClickToMoveInfo.IsClickMoving)
                _timeLastCTM = DateTime.Now;

            if (Me.IsMoving && _speed < (Settings.Stuck_SpeedTolerance / 100f) * _defaultSpeed &&
                DeltaTime(_timeLastCTM, "ms") < 750 &&
                DeltaTime(_timeMountUp, "ms") > 1500)
                _stuckCount++;

            if (_lastStuckCount == _stuckCount)
            {
                _stableStuckCount++;

                if (_stableStuckCount >= 3)
                    _stuckCount = 0;
            }
            else
                _stableStuckCount = 0;
            _lastStuckCount = _stuckCount;

            if (_stuckCount < Settings.Stuck_StuckCountThreshold)
            {
                _lastLocation = Me.Location;
                _timeLastLocation = DateTime.Now;

                if (Settings.Stuck_StuckCountThreshold >= 5 &&
                    _stuckCount >= Settings.Stuck_StuckCountThreshold - 2 &&
                    Me.Mounted && !Me.IsFlying)
                {
                    var _myLocation = Me.Location;
                    var _clickLocation = (WoWMovement.ClickToMoveInfo.IsClickMoving)
                                                        ? WoWMovement.ClickToMoveInfo.ClickPos
                                                        : _myLocation;
                    var _deltaHeight = _clickLocation.Z - _myLocation.Z;
                    var _deltaFrontHeight = (_myLocation.Distance2D(_clickLocation) / 3) * _deltaHeight;
                    var _inFront = WoWMathHelper.CalculatePointInFront(_myLocation + new Vector3(0, 0, 2 + _deltaFrontHeight),
                                                                       Me.Rotation, 3);
                    var _isPassage = !GameWorld.TraceLine(_myLocation, _inFront, TraceLineHitFlags.Collision);

                    var _preStuckCount = 0;
                    var _stuckLocationEntry = _stuckLocations.FirstOrDefault(l => Me.Location.Distance((Vector3)(l[0])) < 10);
                    if (_stuckLocationEntry != null)
                        _preStuckCount = (int)_stuckLocationEntry[1];

                    if (_isPassage || _preStuckCount >= 1)
                    {
                        DevLog("PreventStuck");
                        PrintInfo();
                        await CommonCoroutines.LandAndDismount(null, true);
                        await Move("F", 500, true);
                    }
                }

                return false;
            }
            else if (IsFrameVisible("CinematicFrame"))
            {
                _stuckCount = 0;
                return false;
            }

            return true;
        }

        private string PrintMatrix(Matrix4x4 _matrix)
        {
            return "<Matrix " +
                    _matrix.ToString()
                            .Replace("{ {", "")
                            .Replace("} }", "")
                            .Replace("} {", " ")
                            .Replace(":", "=\"")
                            .Replace(" M", "\" M") +
                   "\" />";
        }

        private Vector3 GetLastStuckPoint()
        {
            return (Vector3)((_stuckLocations[_stuckLocations.Count - 2])[0]);
        }

        private void PrintInfo(bool _stuck = true, int _preStuckCount = 0, double _deltaLastUnstuck = 0)
        {
            var _logString
                        = "\n\n\n" +
                           "----- KoM-LOG ----- \n" +
                           "\n" +
                          $"Location: {Me.Location} ({_mapPosition}) \n";

            if (_stuck && _stuckLocations.Count > 1)
                _logString += $"LastStuckLocation: {GetLastStuckPoint()} (distance {Math.Round(Me.Location.Distance(GetLastStuckPoint()), 1)} yards)\n";

            if (_preStuckCount > 0 || _deltaLastUnstuck > 0)
                _logString += $"PreStuckCount: {_preStuckCount}\n" +
                              $"DeltaLastUnstuck: {TimeString(_deltaLastUnstuck)}\n";


            if (_stuck)
            {
                var _closestObject = ObjectManager.GetObjectsOfType<WoWGameObject>()
                                            .OrderBy(o => o.Distance)
                                            .FirstOrDefault();

                _logString += "\n" +
                              $"ClickPos: {WoWMovement.ClickToMoveInfo.ClickPos} \n" +
                              $"Zone: {Me.RealZoneText} (ZoneId: {Me.ZoneId}) > {Me.MinimapZoneText}" + ((Me.SubZoneId != 0)
                                                                                                            ? $" (SubZoneId: {Me.SubZoneId})"
                                                                                                            : "") + "\n" +
                              $"Map: {Me.MapName} (MapId: {Me.MapId}) \n" +
                               "\n" +

                              $"We are stuck! (TPS: {TreeRoot.TicksPerSecond}, FPS: {StyxWoW.WoWClient.Fps}, Latency: {StyxWoW.WoWClient.Latency}, Map: {Me.MapName}, loc: {Me.Location})!\n" +

                              ((_closestObject != null)
                                        ? $"Nearest game object, distance {Math.Round(_closestObject.Distance, 1)} yards \n" +
                                            $"<!-- {_closestObject.Name} in {Me.RealZoneText} zone and on {Me.MapName} map -->\n" +
                                            $"<GameObject model=\"#{_closestObject.ModelFileId}\">\n" +
                                            $"   {PrintMatrix(_closestObject.GetWorldMatrix())}\n" +
                                             "</GameObject>"
                                        : "") + "\n" +
                               "\n" +
                              $"Profile: {ProfileManager.CurrentProfile.Name} \n" +
                              $"BotPoi: {Styx.CommonBot.POI.BotPoi.Current} \n" +
                              $"TreeRoot: {TreeRoot.GoalText} \n" +
                               "\n" +
                              $"DefaultSpeed: {_defaultSpeed} \n" +
                              $"     CurrentSpeed: {_speed} \n" +
                              $"     {Math.Round(_speed / _defaultSpeed * 100)} % of DefaultSpeed \n" +
                              $"FailedMountUpCount: {_failedMountUpCount} \n" +
                              $"FailedPathDistancesRequestCount: {_failedPathDistancesRequestCount} \n" +
                              $"SpeedTolerance: {Settings.Stuck_SpeedTolerance} \n" +
                              $"StuckCountThreshold: {Settings.Stuck_StuckCountThreshold} \n" +
                              $"Dismount: {Settings.Stuck_Dismount} \n" +
                               "\n" +
                               "--- Testing: \n" +
                              $"<MoveTo X=\"{Me.Location.X}\" Y=\"{Me.Location.Y}\" Z=\"{Me.Location.Z}\" /> \n" +
                              $"<Code File=\"MyCTM\" X=\"{WoWMovement.ClickToMoveInfo.ClickPos.X}\" Y=\"{WoWMovement.ClickToMoveInfo.ClickPos.Y}\" Z=\"{WoWMovement.ClickToMoveInfo.ClickPos.Z}\" /> \n" +
                               "\n" +
                               "----- KoM-LOG -----\n\n";
            }
            else
            {
                _logString += "\n" +
                              $"Zone: {Me.RealZoneText} (ZoneId: {Me.ZoneId}) > {Me.MinimapZoneText} (SubZoneId: {Me.SubZoneId})\n" +
                              $"Map: {Me.MapName} (MapId: {Me.MapId}) \n" +
                               "\n" +
                              $"Profile: {ProfileManager.CurrentProfile.Name} \n" +
                              $"BotPoi: {Styx.CommonBot.POI.BotPoi.Current} \n" +
                              $"TreeRoot: {TreeRoot.GoalText} \n" +
                               "\n" +
                              $"Timer: {Settings.Motionless_Timer} \n" +
                              $"ObjectiveTimer: {Settings.Motionless_ObjectiveTimer} \n" +
                               "\n" +
                               "----- KoM-LOG -----\n\n";
            }

            DevLog(_logString);
        }

        private async Task<bool> DoUnstuck(int _preStuckCount = 1, bool _motionless = false)
        {
            var _horizontalDirections = new List<string>
            {
                "L",
                "R"
            };

            var _verticalDirections = new List<string>
            {
                "J",
                "D"
            };

            var _randomNr = (_rnd.Next(2) == 0)
                                ? 0
                                : 1;
            var _horizontalDirection = _horizontalDirections[_randomNr];
            var _horizontalCounterDirection = _horizontalDirections[(_randomNr + 1) % 2];
            var _verticalDirection = "J";
            var _verticalCounterDirection = "J";

            if (Me.IsFlying ||
               (Flightor.CanFly && Me.Mounted && Mount.Current.IsFlyingMount))
            {
                var _randomNrVertical = (GameWorld.TraceLine(Me.Location, Me.Location + new Vector3(0, 0, 20f), TraceLineHitFlags.Collision))
                                                ? 1
                                                : 0;
                _verticalDirection = _verticalDirections[_randomNr];
                _verticalCounterDirection = _verticalDirections[(_randomNr + 1) % 2];
            }

            var _timeStartStuck = DateTime.Now;
            var _lastStage = Stuck.On;
            var _turnPause = 100;
            _lastStuckLocation = Me.Location;

            Log("STUCK! Unstucking...");

            while (Me.Location.Distance(_lastStuckLocation) < (_preStuckCount + 1) * _defaultSpeed)
            {
                DevLog("WHILE: DoUnstuck");

                if (!_init)
                    return false;

                if (Me.IsAlive && Me.Combat && Me.HealthPercent <= 70)
                    return false;


                var _currentStageEntry = _stuckIntervals.FirstOrDefault(s => DeltaTime(_timeStartStuck, "s") < s);
                StuckState = (Stuck)(_stuckIntervals.IndexOf(_currentStageEntry) + 2);

                if ((int)StuckState >= 5 && IsPlayerInRange)
                {
                    DevLog("PlayerInRange");
                    await JumpToHearthStage();
                    return false;
                }

                if (StuckState != _lastStage)
                {
                    Log("     {0}", StuckState.ToString().Replace("e", "e "));
                    _turnPause = 0;
                    _lastStage = StuckState;
                }

                _turnPause = (_speed < (Settings.Stuck_SpeedTolerance / 100f) * _defaultSpeed)
                                    ? 100
                                    : _turnPause + 100;

                if ((int)StuckState <= 5)
                    await Move("A");

                switch (StuckState)
                {
                    case Stuck.Stage1:
                        await Move($"{_verticalDirection}{_horizontalDirection}", 100);
                        await Turn($"{_horizontalCounterDirection}", 50, _turnPause);
                        break;

                    case Stuck.Stage2:
                        await Move($"{_verticalCounterDirection}{_horizontalCounterDirection}", 100);
                        await Turn($"{_horizontalDirection}", 50, _turnPause);
                        break;

                    case Stuck.Stage3:
                        if (Me.Mounted && !Me.IsFlying)
                            await CommonCoroutines.LandAndDismount(null, true);

                        await Move($"{_verticalDirection}{_horizontalDirection}", 100);
                        await Turn($"{_horizontalCounterDirection}", 100, _turnPause);
                        break;

                    case Stuck.Stage4:
                        await Move($"{_verticalCounterDirection}{_horizontalCounterDirection}", 100);
                        await Turn($"{_horizontalDirection}", 100, _turnPause);
                        break;

                    case Stuck.Stage5:
                        if (MotionlessState != Motionless.Off)
                            return false;

                        await Move("B", 5000, true);
                        break;

                    case Stuck.Stage6:
                        _timeReload = DateTime.Now;
                        ReloadProfile();
                        return false;
                        break;
                }

                _lastLocation = Me.Location;
                _timeLastLocation = DateTime.Now;
                await Coroutine.Yield();
            }

            await CommonCoroutines.StopMoving();
            return true;
        }

        private async Task<bool> HandleStuck()
        {
            if (HearthState != Hearth.Off)
                return false;

            if (Settings.Stuck_Disable)
                return false;

            Navigator.NavigationProvider.ClearStuckInfo();

            if (DeltaTime(_timeMountUp, "ms") < 1500)
                return false;

            if (Me.Stunned)
                return false;

            /*
            if (!Me.IsAlive)
                return false;
                */

            if (Me.IsActuallyInCombat)
            {
                _stuckCount = 0;
                return false;
            }

            if (Me.OnTaxi || Me.IsOnTransport)
                return false;

            if (!await IsStuck() &&
                StuckState == Stuck.Off &&
                _failedMountUpCount < 3 &&
                _failedPathDistancesRequestCount < 5)
                return false;

            if (Me.IsAlive && await HandleReloadStage())
            {
                await JumpToHearthStage();
                return false;
            }

            if (StuckState == Stuck.Stage6)
                return false;

            var _preStuckCount = 0;
            var _stuckLocationEntry = _stuckLocations.FirstOrDefault(l => Me.Location.Distance((Vector3)(l[0])) < 10);
            if (_stuckLocationEntry == null)
            {
                _stuckLocations.Add(new object[] { Me.Location, 0, DateTime.MinValue });
                _stuckLocationEntry = _stuckLocations.Last();

                PrintInfo();
            }
            else
            {
                var _timeLastUnstuck = (DateTime)_stuckLocationEntry[2];
                var _deltaLastUnstuck = DeltaTime(_timeLastUnstuck, "s");
                if (_deltaLastUnstuck >= 30)
                {
                    _stuckLocationEntry[1] = 0;
                    _stuckLocationEntry[2] = DateTime.MinValue;
                }

                _preStuckCount = (int)_stuckLocationEntry[1];
                PrintInfo(true, _preStuckCount, _deltaLastUnstuck);

                if (_preStuckCount >= 3)
                {
                    DevLog("PreStuckCount >= 3");
                    Log("STUCK! Unstucking...");
                    await JumpToHearthStage();
                    return false;
                }
            }

            if (Settings.Stuck_Dismount && Me.Mounted && !Me.IsFlying)
            {
                DevLog("Stuck_Dismount enabled");
                await CommonCoroutines.LandAndDismount(null, true);
            }

            if (!await DoUnstuck(_preStuckCount))
                return false;

            _stuckLocationEntry[2] = DateTime.Now;
            _stuckLocationEntry[1] = (int)(_stuckLocationEntry[1]) + 1;

            Reset();
            return true;
        }

        private List<object[]> _classHearthData = new List<object[]>
        {
            new object[] { WoWClass.Monk,           "Zen Pilgrimage",       126892, 126896, new Vector3(3826.87f, 1795.74f, 950.3f) },
            new object[] { WoWClass.Druid,          "Teleport: Moonglade",  18960,  18960,  new Vector3(7995.56f, -2679.75f, 512.49f) },
            new object[] { WoWClass.DeathKnight,    "Death Gate",           50977,  50977,  new Vector3(2359.64f, -5662.41f, 382.2605f) }
        };

        private async Task<bool> CastClassHearth()
        {
            if ((int)HearthState >= 2)
                return false;

            var _classHearthEntry = _classHearthData.FirstOrDefault(e => Me.Class == (WoWClass)(e[0]) &&
                                                                         SpellManager.HasSpell((int)e[2]) &&
                                                                         SpellManager.HasSpell((int)e[3]) &&
                                                                         GetCondition(((WoWClass)(e[0])).ToString(), true));
            if (_classHearthEntry == null)
                return false;

            var _spellName = (string)_classHearthEntry[1];
            var _portSpellId = (int)_classHearthEntry[2];
            var _returnSpellId = (int)_classHearthEntry[3];
            var _portLocation = (Vector3)_classHearthEntry[4];
            var _preMapId = Me.CurrentMap.ExpansionId;

            Log("HEARTH! Taking me home...");
            Log("     {0}", _spellName);

            await CommonCoroutines.LandAndDismount(null, true);
            await CommonCoroutines.StopMoving();

            while (StyxWoW.IsInWorld && Me.Location.Distance(_portLocation) > 60 && !breakOut)
            {
                DevLog("WHILE: HandleClassHearth > Port");
                if (Me.Class == WoWClass.DeathKnight
                            ? _deathGate == null
                            : true)
                {
                    if (!Me.IsCasting && SpellManager.CanCast(_portSpellId))
                    {
                        SpellManager.CastSpellById(_portSpellId);
                        await Coroutine.Wait(1000, () => Me.IsCasting);
                        await Coroutine.Wait(10 * 1000, () => !Me.IsCasting);

                        await Coroutine.Wait(2000, () => (Me.Class == WoWClass.DeathKnight)
                                                                    ? _deathGate != null
                                                                    : !StyxWoW.IsInWorld);
                        if (StyxWoW.IsInWorld && (Me.Class == WoWClass.DeathKnight
                                                            ? _deathGate == null
                                                            : true))
                            await Move("B", 200, true);
                    }
                }
                else
                {
                    await Sleep();
                    _deathGate.Interact();
                    await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                }
                await Coroutine.Yield();
            }
            await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld || !_init);

            if (_preMapId != Me.CurrentMap.ExpansionId)
            {
                await WaitForMapChange();
                _preMapId = Me.CurrentMap.ExpansionId;
            }

            if (Me.Class == WoWClass.DeathKnight)
                Log("     Waiting for \"Death Gate\" cooldown...");

            while (StyxWoW.IsInWorld && Me.Location.Distance(_portLocation) < 60 && !breakOut)
            {
                DevLog("WHILE: HandleClassHearth > Return");
                if (Me.Class != WoWClass.DeathKnight || _deathGate == null)
                {
                    if (!Me.IsCasting && SpellManager.CanCast(_returnSpellId))
                    {
                        SpellManager.CastSpellById(_returnSpellId);

                        if(await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                            if(await Coroutine.Wait(10 * 1000, () => !Me.IsCasting))
                                if(await Coroutine.Wait(2 * 1000, () => (Me.Class == WoWClass.DeathKnight)
                                                                            ? _deathGate != null
                                                                            : !StyxWoW.IsInWorld))

                        if (StyxWoW.IsInWorld && (Me.Class != WoWClass.DeathKnight || _deathGate == null) )
                            await Move("B", 200, true);
                    }
                }
                else
                {
                    await Sleep();
                    _deathGate.Interact();
                    await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                }
                await Coroutine.Yield();
            }
            await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld || !_init);

            if (_preMapId != Me.CurrentMap.ExpansionId)
                await WaitForMapChange();

            return true;
        }

        private async Task<bool> UseHearthItem()
        {
            var _hearthItemEntries = _hearthItemData.Where(e => Me.BagItems.Any(i => i.Entry == (int)(e[1])) &&
                                                                GetCondition((string)(e[0])) &&
                                                                GetCondition((string)(e[0]), true));
            var _hearthItemEntry = _hearthItemEntries.FirstOrDefault();
            if (_hearthItemEntry == null)
                return false;

            if (_hearthItemEntries.Count() > 1)
                _hearthItemEntry = _hearthItemEntries.FirstOrDefault(e => Me.BagItems.FirstOrDefault(i => i.Entry == (int)(e[1])).CooldownTimeLeft.TotalSeconds < 30);

            if (_hearthItemEntry == null)
                _hearthItemEntry = _hearthItemEntries.FirstOrDefault();

            var _itemName = (string)_hearthItemEntry[0];
            var _itemId = (int)_hearthItemEntry[1];
            var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        i.Entry == _itemId);

            if (HearthState == Hearth.Capital)
                Log("Going to {0}", (Me.IsHorde)
                                        ? "Orgrimmar"
                                        : "Stormwind");
            else if ((int)HearthState >= 3)
                Log("Going to {0}", HearthState.ToString().Replace("nI", "n I"));

            Log("HEARTH! Taking me home...");
            if (_item.CooldownTimeLeft.TotalSeconds > 0)
            {
                Log("     Waiting for \"{0}\" cooldown...", _itemName);

                var _seconds = _item.CooldownTimeLeft.TotalSeconds;
                Log("          {0} left", TimeString(_seconds));

                Func<bool> _waitForCooldown = () =>
                {
                    if (!_init)
                        return true;

                    if (breakOut)
                        return true;

                    var _timeLeft = Math.Round(_item.CooldownTimeLeft.TotalSeconds);
                    if (_timeLeft % 10 == 0 && _logTimer.IsFinished)
                    {
                        Log("          {0} left", TimeString(_timeLeft));
                        _logTimer.Reset();
                    }

                    return _item.CooldownTimeLeft.TotalSeconds == 0;
                };
                await Coroutine.Wait(30 * 60 * 1000, _waitForCooldown);
            }

            await CommonCoroutines.LandAndDismount(null, true);
            await CommonCoroutines.StopMoving();

            Log("     Using \"{0}\"", _itemName);
            var _portLocation = Me.Location;
            var _preZone = Me.ZoneId;
            while (StyxWoW.IsInWorld && Me.Location.Distance(_portLocation) < 60 && !breakOut)
            {
                DevLog("WHILE: UseHearthItem");

                if (InHearthArea())
                {
                    DevLog("     InHearthArea");
                    return true;
                }

                if (Me.Combat)
                    return false;

                await CommonCoroutines.StopMoving();
                if(await UseItem(_item))
                    if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                        if (await Coroutine.Wait(10 * 1000, () => !Me.IsCasting))
                            await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                if (StyxWoW.IsInWorld && !Me.IsCasting)
                    await Move("B", 400, true);

                await Coroutine.Yield();
            }

            if (!breakOut)
            {
                await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld || !_init);
                await Coroutine.Wait(6 * 1000, () => _preZone != Me.ZoneId);
            }

            return true;
        }

        private bool GetCondition(string _optionName, bool _returnSetting = false)
        {
            if (_returnSetting)
            {
                switch (_optionName)
                {
                    case "Monk":
                        return Settings.Hearth_Monk;
                        break;
                    case "Druid":
                        return Settings.Hearth_Druid;
                        break;
                    case "DeathKnight":
                        return Settings.Hearth_DeathKnight;
                        break;
                    case "Flightmaster's Whistle":
                        return Settings.Hearth_FlightmastersWhistle;
                        break;
                    case "Dalaran Hearthstone":
                        return Settings.Hearth_DalaranHearthstone;
                        break;
                    case "Garrison Hearthstone":
                        return Settings.Hearth_GarrisonHearthstone;
                        break;
                    case "Stormwind Portal Stone":
                        return Settings.Hearth_PortalStone;
                    case "Orgrimmar Portal Stone":
                        return Settings.Hearth_PortalStone;
                    case "Hearthstone":
                        return Settings.Hearth_Hearthstone;
                        break;
                    case "Pandaria":
                        return Settings.Archaeology_GoToPandaria;
                        break;
                    case "Draenor":
                        return Settings.Archaeology_GoToDraenor;
                        break;
                    case "BrokenIsles":
                        return Settings.Archaeology_GoToBrokenIsles;
                        break;
                }
            }
            else
            {
                switch (_optionName)
                {
                    case "Flightmaster's Whistle":
                        return Me.CurrentMap.ExpansionId == 6 &&
                                Me.MapId != 1014 &&
                                Me.MapId != 1021 &&
                                Me.BagItems.FirstOrDefault(i => i.Entry == 141605).Usable &&
                                HearthState != Hearth.Capital &&
                                HearthState != Hearth.Pandaria &&
                                HearthState != Hearth.Draenor &&
                                HearthState != Hearth.BrokenIsles;
                        break;

                    case "Dalaran Hearthstone":
                        return (
                                    HearthState == Hearth.BrokenIsles ||
                                    Me.CurrentMap.ExpansionId == 6
                               ) &&
                               HearthState != Hearth.Capital &&
                               HearthState != Hearth.Pandaria &&
                               HearthState != Hearth.Draenor;
                        break;

                    case "Garrison Hearthstone":
                        return (
                                   HearthState == Hearth.Draenor ||
                                   (
                                       Me.CurrentMap.ExpansionId == 5 &&
                                       (
                                           (
                                               ProfileManager.XmlLocation != null &&
                                               ProfileManager.XmlLocation.ToLower().Contains("questing profile pack") &&
                                               Me.Level < 100
                                           ) ||
                                           (
                                               ProfileManager.XmlLocation != null &&
                                               !ProfileManager.XmlLocation.ToLower().Contains("questing profile pack")
                                           )
                                       )
                                   )
                               ) &&
                               HearthState != Hearth.Capital &&
                               HearthState != Hearth.Pandaria &&
                               HearthState != Hearth.BrokenIsles;
                        break;

                    case "Orgrimmar Portal Stone":
                        return (int)HearthState <= 3;
                        break;

                    case "Stormwind Portal Stone":
                        return (int)HearthState <= 3;
                        break;

                    case "Hearthstone":
                        if (Me.CurrentMap.ExpansionId >= 5 &&
                            Me.MapId != HearthstoneMapId()) /* &&
                            UnfavourableAreaSet()) */
                            return false;

                        if ((int)HearthState >= 3 &&
                            (int)HearthState + 1 != HearthstoneMapId())
                            return false;

                        return true;
                        break;
                }
            }

            return false;
        }

        private List<object[]> _hearthItemData = new List<object[]>
        {
            new object[] {"Flightmaster's Whistle", 141605 },
            new object[] {"Dalaran Hearthstone",    140192 },
            new object[] {"Garrison Hearthstone",   110560 },
            new object[] {"Orgrimmar Portal Stone", 132119 },
            new object[] {"Stormwind Portal Stone", 132120 },
            new object[] {"Hearthstone",            6948 },
        };

        /*
        private bool UnfavourableAreaSet()
        {
            var _area = AreaTable.FromId(Me.HearthstoneAreaId);
            if (_area == null)
                return true;

            if (Me.HearthstoneAreaId != _pandariaHearthstoneAreaId &&
               ((Me.IsHorde)
                    ? _area.MapId != 1
                    : _area.MapId != 0))
                return true;

            return false;
        }
        */

        private int HearthstoneMapId()
        {
            var _area = AreaTable.FromId(Me.HearthstoneAreaId);
            if (_area == null)
                return 0;

            return (int)_area.MapId;
        }

        private bool InHearthArea()
        {
            var _area = AreaTable.FromId(Me.HearthstoneAreaId);
            if (_area == null)
                return false;

            return Me.SubZoneId == _area.AreaId;
        }

        private async Task<bool> HandleHearth()
        {
            if (Settings.Hearth_Disable)
                return false;

            if (!Me.IsAlive)
                return false;

            if (Me.Combat)
                return false;

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return false;

            RuntimeLog(">");

            /*
            if (await HandleHearthstoneArea(true))
                return true;

            RuntimeLog(">");

            if (Settings.Hearth_Hearthstone &&
               Settings.Hearth_SetHearthstone)
            {
                if (UnfavourableAreaSet())
                {
                    Log("Hearthstone is set to a highly unfavourable area. Place char in {0} and restart bot.",
                        (Me.IsHorde)
                              ? "Orgrimmar"
                              : "Stormwind");
                    Settings.General_InfiniteMode = false;
                    Log("     STOPPING bot...");
                    DevLog("HearthstoneAreaId: {0}", Me.HearthstoneAreaId);

                    var _area = AreaTable.FromId(Me.HearthstoneAreaId);
                    if (_area == null)
                        DevLog("Hearthstone not set");
                    else
                    {
                        DevLog("AreaName: {0}", _area.AreaName);
                        DevLog("MapId: {0}", _area.MapId);
                    }
                    TreeRoot.Stop("KeepOnMoving");
                }
                
                if (_startNewExpansion)
                {
                    if (await HandleCompletedQuests(true))
                        return false;

                    DevLog("Hearth.Capital: StartNewExpansion");
                    Log("Preparing for starting new expansion");
                    HearthState = Hearth.Capital;
                }
            }
            */

            if (HearthState == Hearth.Off)
                return false;

            if (Settings.Hearth_PreStop)
            {
                Reset(true);
                Settings.General_InfiniteMode = false;
                Log("HEARTH! Taking me home...");
                Log("     STOPPING bot...");
                TreeRoot.Stop("KeepOnMoving");
            }

            var _preMapId = Me.CurrentMap.ExpansionId;
            if (!await CastClassHearth())
            {
                if (!await UseHearthItem())
                {
                    Log("HEARTH! Taking me home...");
                    Log("     Couldn't do my job as no hearth option seems to be available");
                    Log("     STOPPING bot...");

                    foreach (var _entry in _classHearthData)
                    {
                        DevLog("{0}", (string)_entry[1]);
                        DevLog("     AppropriateClass: {0}", Me.Class == (WoWClass)_entry[0]);
                        DevLog("     SettingEnabled: {0}", GetCondition(((WoWClass)_entry[0]).ToString(), true));
                        DevLog("     HasPortSpell: {0}", (int)_entry[2]);
                        DevLog("     HasReturnSpell: {0}", (int)_entry[3]);
                    }

                    foreach (var _entry in _hearthItemData)
                    {
                        DevLog("{0}", (string)_entry[0]);
                        DevLog("     HasItem: {0}", Me.BagItems.Any(i => i.Entry == (int)_entry[1]));
                        DevLog("     SettingEnabled: {0}", GetCondition((string)_entry[0]), true);
                        DevLog("     ConditionsMeet: {0}", GetCondition((string)_entry[0]));
                    }

                    Settings.General_InfiniteMode = false;
                    TreeRoot.Stop("KeepOnMoving");
                }
            }

            /*
            if (HearthState == Hearth.Capital && !_inCapital)
            {
                if (_inShrine)
                {
                    var _portalId = (Me.IsHorde)
                                        ? 215127
                                        : 215119;
                    var _portal = GetObject(_portalId);
                    var _portalLocation = (Me.IsHorde)
                                                ? new Vector3(1735.073f, 873.5676f, 487.4114f)
                                                : new Vector3(828.2266f, 176.9051f, 519.9392f);

                    while (!_inCapital && !breakOut)
                    {
                        DevLog("WHILE: HandleHearth");

                        await TakePortal(_portalId, _portalLocation);
                        await Coroutine.Yield();
                    }
                }
                else if ((Me.IsHorde)
                            ? Me.MapId == 1
                            : Me.MapId == 0)
                    await HandleHearthstoneArea(true, true);
            }
            */

            if (!breakOut)
            {
                if (_preMapId != Me.CurrentMap.ExpansionId)
                    await WaitForMapChange();

                Reset();
                if (Settings.Hearth_PostStop)
                {
                    Settings.General_InfiniteMode = false;
                    Log("     STOPPING bot...");
                    TreeRoot.Stop("KeepOnMoving");
                }

                ReloadProfile();
            }
            return true;
        }

        private async Task<bool> HandleGarrisonResources()
        {
            if (Me.IsHorde
                    ? Me.ZoneId != 7004
                    : Me.ZoneId != 7078)
                return false;

            if (_garrisonCache == null)
                return false;

            if (DeltaTime(_timeGarrisonCacheInteracted, "m") < 60)
                return false;

            Log("Getting garrison resources");
            var _garrisonCacheLocation = _garrisonCache.Location;
            var _preGarrisonResourcesCount = _garrisonResourcesCount;
            while (_preGarrisonResourcesCount == _garrisonResourcesCount && _garrisonCache != null && !breakOut)
            {
                DevLog("WHILE: GetGarrisonResources");

                if (_garrisonCache == null || !_garrisonCache.WithinInteractRange)
                    await MoveTo(_garrisonCacheLocation);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _garrisonCache.Interact();
                }
                await Coroutine.Yield();
            }

            _timeGarrisonCacheInteracted = DateTime.Now;
            return true;
        }

        private async Task<bool> HandleTrainFlying()
        {
            if (Settings.Quest_TrainFlying.ToString() == "None")
                return false;

            if (Me.Level < 70)
                return false;

            if (Me.GetSkill(SkillLine.Riding).CurrentValue < 225 ||
                Me.GetSkill(SkillLine.Riding).CurrentValue >= 375)
                return false;

            if (Me.GetSkill(SkillLine.Riding).CurrentValue == 300 &&
                !Settings.Quest_TrainFlying.ToString().Contains("Master"))
                return false;

            if (Me.GetSkill(SkillLine.Riding).CurrentValue == 300 &&
                Settings.Quest_TrainFlying.ToString().Contains("Master") &&
                Me.Level < 80)
                return false;

            var _trainerDataEntry = GetClosestTrainerEntry(SkillLine.Riding);
            if (_trainerDataEntry == null)
                return false;

            var _factionId = (uint)_trainerDataEntry[5];
            var _neededGold = GetFlyingGold(_factionId);
            if (Me.Gold < _neededGold)
                return false;

            /*
            if (!_inCapital)
            {
                if (!Settings.Hearth_Hearthstone ||
                    !Settings.Hearth_SetHearthstone ||
                    UnfavourableAreaSet())
                    return false;

                DevLog("Hearth.Capital: TrainFlying");
                Log("Preparing for training flying");
                HearthState = Hearth.Capital;
                return false;
            }
            */

            if (Me.GetSkill(SkillLine.Riding).CurrentValue == 225)
                Log("Learning \"Artisan Riding\"");
            else
                Log("Learning \"Master Riding\"");

            /*
            var _npcId = (Me.IsHorde)
                                ? 44919
                                : 43769;
            var _npc = GetNpc(_npcId);
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            */

            var _npcId = (int)_trainerDataEntry[0];
            var _npc = GetNpc(_npcId);
            var _npcLocation = (Vector3)_trainerDataEntry[4];

            var _preSkill = Me.GetSkill(SkillLine.Riding).CurrentValue;
            while (Me.GetSkill(SkillLine.Riding).CurrentValue == _preSkill && !breakOut)
            {
                DevLog("WHILE: TrainFlying");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;

                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();

                    await Coroutine.Wait(5000, () => IsFrameVisible("ClassTrainerFrame"));
                    await ClickButton("ClassTrainerTrainButton");
                    await Coroutine.Wait(5000, () => Me.GetSkill(SkillLine.Riding).CurrentValue != _preSkill);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private int GetSpecIndex()
        {
            var _classProfiles = Styx.CommonBot.CharacterManagement.CharacterManager.ClassProfiles
                        .Where(p => p.Class == Me.Class);
            var _currentSpec = Me.Specialization;
            var _currentClassProfile = _classProfiles.FirstOrDefault(p => p.GetSpec() == _currentSpec);
            return _classProfiles.ToList().IndexOf(_currentClassProfile) + 1;

        }

        private async Task<bool> HandleSpec()
        {
            if (Me.IsFlying)
                return false;

            if (!Me.IsAlive)
                return false;

            var _setting = Settings.Quest_Specialisation;
            if (_setting == 0)
                return false;

            if (Me.Level < 10)
                return false;

            if (_spec == _setting)
                return false;

            Log("Changing specialization");
            while (GetSpecIndex() != _setting && !breakOut)
            {
                DevLog("WHILE: ChangeSpec");

                if (!Me.IsCasting)
                    Lua.DoString("SetSpecialization(" + _setting + ")");
                await Coroutine.Wait(12000, () => !Me.IsCasting);
                await Sleep();
                await Coroutine.Yield();
            }

            var _currentSpec = Me.Specialization;
            var _currentClassProfileSpec = CharacterManager.CurrentClassProfile.GetSpec();
            var _newClassProfile = CharacterManager.ClassProfiles
                                                        .FirstOrDefault(p => p.GetSpec() == _currentSpec);

            if (_currentClassProfileSpec != _currentSpec)
                CharacterManager.SetClassProfile(_newClassProfile);

            _spec = _setting;
            return true;
        }

        private async Task<bool> HandleAbandonQuest()
        {
            if (!Settings.Quest_AbandonQuests)
                return false;

            var _questCount = QuestLog.Instance.GetAllQuests().Count();
            if (_questCount == 0)
                return false;

            var _levelThreshold = (Settings.Quest_QuestingPack.ToString() == "New")
                                            ? 3
                                            : 5;

            var _lowLevelQuest = QuestLog.Instance.GetAllQuests().FirstOrDefault(q => q.Level > 0 &&
                                                                                      q.Level <= Me.Level - _levelThreshold);
            if (_lowLevelQuest == null)
                return false;

            Log("Abandoning low level quest");
            while (_lowLevelQuest != null && !breakOut)
            {
                DevLog("WHILE: HandleAbandonQuest");
                Log("     {0} (Id: {1})", _lowLevelQuest.Name, _lowLevelQuest.Id);

                QuestLog.Instance.AbandonQuestById(_lowLevelQuest.Id);
                await Coroutine.Wait(3 * 1000, () => !QuestLog.Instance.ContainsQuest(_lowLevelQuest.Id));

                _lowLevelQuest = QuestLog.Instance.GetAllQuests().FirstOrDefault(q => q.Level > 0 &&
                                                                                      q.Level <= Me.Level - _levelThreshold);
                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleSorting()
        {
            if (!Settings.Bag_Sort)
                return false;

            Log("Sorting bags");
            Lua.DoString("SortBags()");

            return true;
        }

        private bool HasToy(uint _itemId)
        {
            if (!_toys.ContainsKey((int)_itemId))
                return false;

            return _toys[(int)_itemId];
        }

        private bool IsToy(uint _itemId)
        {
            return _toys.ContainsKey((int)_itemId);
        }

        private Dictionary<int, bool> GetToys()
        {
            var _toys = new Dictionary<int, bool>();
            var _maxToys = Lua.GetReturnVal<int>("return C_ToyBox.GetNumToys()", 0);
            var _showAll = "C_ToyBox.SetUncollectedShown(true);" +
                           "C_ToyBox.SetCollectedShown(true);" +
                           "C_ToyBox.SetAllSourceTypeFilters(true)";

            Lua.DoString(_showAll);
            for (int i = 1; i <= _maxToys; i++)
            {
                var _return = Lua.GetReturnVal<int>($"return C_ToyBox.GetToyFromIndex({i - 1})", 0);
                if (_return != -1)
                {
                    var _hasToy = Lua.GetReturnVal<bool>($"return PlayerHasToy({_return})", 0);
                    _toys.Add(_return, _hasToy);
                }
            }

            return _toys;
        }

        private async Task<bool> HandleToyItems()
        {
            RuntimeLog(">>");

            if (_toys.Count() == 0)
                _toys = GetToys();

            RuntimeLog(">>");

            if (!Settings.Bag_LearnToys &&
                !Settings.Bag_RemoveToys)
                return false;

            if (await LearnToyItems())
                return true;

            if (await RemoveToyItems())
                return true;

            return false;
        }

        private async Task<bool> LearnToyItems()
        {
            if (!Settings.Bag_LearnToys)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        IsToy(i.Entry) &&
                                                        !HasToy(i.Entry));
            if (_item == null)
                return false;

            Log("Learning toy items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: LearnToyItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                if (await UseItem(_item))
                    await Coroutine.Wait(2000, () => !Me.BagItems.Any(i => i.Entry == _item.Entry));
                else
                    break;

                _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        IsToy(i.Entry) &&
                                                        !HasToy(i.Entry));
                await Coroutine.Yield();
            }
            _toys = GetToys();

            await HandleSorting();
            return true;
        }

        private bool StartsQuest(WoWItem _item)
        {
            if (_item == null)
                return false;

            if (_item.ItemInfo == null)
                return false;

            var _questId = _item.ItemInfo.BeginQuestId;
            if (_questId == 0)
                return false;

            return true;

            /*
            if (Tooltip((int)_item.Entry).Contains(_startsQuestText))
                return true;
                */
        }

        private string Tooltip(int _itemId)
        {
            var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        i.Entry == _itemId);
            if (_item == null)
                return null;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null)
                return null;

            var _text = Lua.GetReturnVal<string>(
                    "local f = CreateFrame('GameTooltip', 'tooltip', UIParent, 'GameTooltipTemplate') " +
                    "local s = '' " +
                    "f:SetOwner(UIParent, 'ANCHOR_NONE') " +
                   $"f:SetBagItem({_item.BagIndex + 1}, {_item.BagSlot + 1}) " +

                    "for i=1, tooltip:NumLines() do " +
                    "	local t = _G['tooltipTextLeft'..i]:GetText() " +
                    "	if t ~= nil then " +
                    "		s = s .. t .. '\\n' " +
                    "	end " +
                    "end " +

                    "f:Hide()" +
                    "return s "
            , 0);

            return _text;
        }

        private async Task<bool> HandleQuestItems()
        {
            if (!Settings.Bag_RemoveQuestItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        IsQuestItem(i));
            if (_item == null)
                return false;

            Log("Removing low level quest item");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleQuestItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                _item.PickUp();
                Lua.DoString("DeleteCursorItem();");
                await Coroutine.Wait(2000, () => !Me.BagItems.Any(i => i.Entry == _item.Entry));

                _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        IsQuestItem(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> RemoveToyItems()
        {
            if (!Settings.Bag_RemoveToys)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        IsToy(i.Entry) &&
                                                        HasToy(i.Entry));
            if (_item == null)
                return false;

            Log("Removing already known toy items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: RemoveToyItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                _item.PickUp();
                Lua.DoString("DeleteCursorItem();");
                await Coroutine.Wait(2000, () => !Me.BagItems.Any(i => i.Entry == _item.Entry));

                _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        IsToy(i.Entry) &&
                                                        HasToy(i.Entry));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleCombinableItems()
        {
            if (!Settings.Bag_CombineItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsCombinable(i));
            if (_item == null)
                return false;

            if (Me.Mounted)
                return false;

            Log("Combining items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: CombineItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Coroutine.Wait(2000, () => GetItem((int)(_item.Entry)) == null ||
                                                     _item.CooldownTimeLeft.TotalSeconds == 0);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IsCombinable(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleOpenableItems()
        {
            if (!Settings.Bag_OpenItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsOpenable(i));
            if (_item == null)
                return false;

            if (Me.Mounted)
                return false;

            Log("Opening items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: OpenItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Loot();
                        await Coroutine.Wait(3 * 1000, () => GetItem((int)(_item.Entry)) == null ||
                                                             _item.CooldownTimeLeft.TotalSeconds == 0);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IsOpenable(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleMailItems()
        {
            if (!Settings.Bag_MailItems)
                return false;

            if (CharacterSettings.Instance.MailRecipient == "")
                return false;

            if (!IsFrameVisible("MailFrame"))
                return false;

            var _items = Me.BagItems.Where(i => i.ItemInfo.RequiredLevel < Me.Level - 10 &&
                                                ProtectedItemsManager.Contains(i.Entry) &&
                                                !i.IsSoulbound &&
                                                i.Entry != 5020);
            if (_items.Count() == 0)
                return false;

            Log("Mailing items");
            WoWItem[] _itemsToAttach = _items.Skip(0).Take(12).ToArray();
            var _preItemCount = (int)Me.BagItems.Sum(i => i.StackCount);
            var _lastItemsToAttach = new WoWItem[0];
            while (_items.Count() > 0 && !breakOut)
            {
                DevLog("WHILE: MailItems");

                if (_itemsToAttach != _lastItemsToAttach)
                {
                    foreach (WoWItem _i in _itemsToAttach)
                    {
                        Log("     {0} (Id: {1})", _i.Name, _i.Entry);
                    }
                    _lastItemsToAttach = _itemsToAttach;
                }

                MailFrame.Instance.SendMail(CharacterSettings.Instance.MailRecipient, _itemsToAttach[0].Name, null, 0, _itemsToAttach);

                await Coroutine.Wait(2000, () => _preItemCount != (int)Me.BagItems.Sum(i => i.StackCount));
                await Sleep();

                _items = Me.BagItems.Where(i => i.IsValid &&
                                                i.ItemInfo.RequiredLevel < Me.Level - 10 &&
                                                ProtectedItemsManager.Contains(i.Entry) &&
                                                !i.IsSoulbound);

                _itemsToAttach = _items.Skip(0).Take(12).ToArray();
                _preItemCount = (int)Me.BagItems.Sum(i => i.StackCount);
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> UseItem(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            if (_item == null)
                return false;

            _item.Use();
            return true;
        }

        private async Task<bool> HandleContainerItems()
        {
            if (!Settings.Bag_ManageContainers)
                return false;

            var _containers = Me.CarriedItems
                                .Where(c => c.IsValid &&
                                            c.ItemInfo.ContainerClass == WoWItemContainerClass.Bag)
                                .Cast<WoWContainer>();
            if (_containers.Count() == 0)
                return false;

            var _containersInBag = _containers
                                        .Where(c => c.BagSlot >= 0)
                                        .OrderByDescending(c => c.Slots);
            if (_containersInBag.Count() == 0)
                return false;

            var _equippedContainers = _containers
                                            .Where(c => c.BagSlot < 0)
                                            .OrderBy(c => c.Slots);

            if (_equippedContainers.Count() == 4 &&
                _equippedContainers.First().Slots >= _containersInBag.First().Slots)
                return false;

            var _autoEquip = CharacterSettings.Instance.AutoEquip;
            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = false;

            if (_equippedContainers.Count() < 4)
            {
                Log("Equipping container");

                var _item = _containersInBag.First();
                while (_equippedContainers.Count() < 4 && !breakOut)
                {
                    var _preEquippedContainersCount = _equippedContainers.Count();
                    DevLog("WHILE: EquipContainer");
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);

                    await Sleep();
                    _item.UseContainerItem();
                    if (!_item.IsSoulbound)
                    {
                        await Coroutine.Wait(2000, () => IsFrameVisible("StaticPopup1"));
                        while (IsFrameVisible("StaticPopup1"))
                        {
                            DevLog("WHILE: EquipContainer Click");
                            await ClickButton("StaticPopup1Button1");
                            await Sleep();
                            await Coroutine.Yield();
                        }
                    }
                    await Coroutine.Wait(3 * 1000, () => _preEquippedContainersCount != _equippedContainers.Count());

                    if (_containersInBag.Count() == 0)
                        return false;
                    _item = _containersInBag.First();
                    await Coroutine.Yield();
                }
            }

            if (_equippedContainers.Count() == 0)
                return false;

            if (_equippedContainers.First().Slots < _containersInBag.First().Slots)
            {
                Log("Exchanging container");

                var _item = _equippedContainers.First();
                var _betterItem = _containersInBag.First();
                while (_equippedContainers.First().Slots < _containersInBag.First().Slots && !breakOut)
                {
                    DevLog("WHILE: ExchangeContainer", _equippedContainers.Count());
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    Log("          <=> {0} (Id: {1})", _betterItem.Name, _betterItem.Entry);

                    if (_betterItem.ToItem().BagIndex == _item.BagIndex)
                    {
                        DevLog("WHILE ExchangeContainer > ToBackpack");
                        _betterItem.PickUp();
                        Lua.DoString("PickupContainerItem(0, 1)");
                        await Coroutine.Wait(3 * 1000, () => _betterItem.ToItem().BagIndex != _item.BagIndex);
                    }

                    var _slot = 20 + _item.BagIndex;
                    Lua.DoString("PickupBagFromSlot(" + _slot + ")");
                    _betterItem.PickUp();

                    if (!_betterItem.IsSoulbound)
                    {
                        await Coroutine.Wait(2000, () => IsFrameVisible("StaticPopup1"));
                        while (IsFrameVisible("StaticPopup1") && !breakOut)
                        {
                            DevLog("WHILE: EquipContainer Click");
                            await ClickButton("StaticPopup1Button1");
                            await Coroutine.Yield();
                        }
                    }
                    await Coroutine.Wait(3 * 1000, () => _betterItem != _containersInBag.First() ||
                                                     _item != _equippedContainers.First());

                    _item = _equippedContainers.First();
                    _betterItem = _containersInBag.First();
                    await Coroutine.Yield();
                }
            }

            await HandleSorting();

            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = true;

            return true;
        }

        private List<int> _draenorItemsData = new List<int>
        {
            111349,     // Mining
		    111350,     // Herbalism
		    111364,     // First Aid
            111387,     // Cooking
            109586,     // Archaeology
            111351,     // Skinning
            111356,     // Fishing
	    };

        private async Task<bool> HandleDraenorItems()
        {
            var _item = Me.BagItems.FirstOrDefault(i => _draenorItemsData.Contains((int)i.Entry));
            if (_item == null)
                return false;

            Log("Learning Draenor item");
            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleDraenorItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                if (await UseItem(_item))
                {
                    if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                        if (await Coroutine.Wait(15 * 1000, () => !Me.IsCasting))
                            await Coroutine.Wait(2 * 1000, () => !HasItem((int)_item.Entry));
                }
                else
                    break;

                _item = Me.BagItems.FirstOrDefault(i => _draenorItemsData.Contains((int)i.Entry));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleItems()
        {
            if (await HandleMailItems())
                return true;

            RuntimeLog(">");

            if (_checkedItems)
                return false;

            RuntimeLog(">");

            if (await HandleContainerItems())
                return true;

            RuntimeLog(">");

            if (await HandleCombinableItems())
                return true;

            RuntimeLog(">");

            if (await HandleOpenableItems())
                return true;

            RuntimeLog(">");

            if (await HandleToyItems())
                return true;

            RuntimeLog(">");

            if (await HandleQuestItems())
                return true;

            RuntimeLog(">");

            if (await HandleDraenorItems())
                return true;

            RuntimeLog(">");

            return false;
        }

        bool SkillNeedsTraining(SkillLine _skill, bool _archaeology = false)
        {
            if (!StringContains(Settings.GatheringProfessions_Professions, "OR", _skill) && !_archaeology)
                return false;

            var _skillMax = Me.GetSkill(_skill).MaxValue;
            int _stage = _skillMax / 75;
            if (_stage >= 8)
                return false;

            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            if (_skillValue == 0)
                return true;

            _skillValue = ((Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism) /*||
                           (Me.Race == WoWRace.Worgen && _skill == SkillLine.Skinning)*/)
                                    ? _skillValue + 15
                                    : _skillValue;

            if (_skillValue < (_stage - 1) * 75 + 50)
                return false;

            if (Me.Level < _trainLevelData[_stage] && !_archaeology)
                return false;

            return true;
        }

        private List<int> _trainLevelData = new List<int>
        {
            0,	    // 75
		    0,	    // 150
		    10,     // 225
		    25,     // 300 CLASSIC
		    40,     // 375 BC
		    55,     // 450 WOTLK
		    75,     // 525 CATA
		    80      // 600 MOP
	    };

        private void CreateTrainerData()
        {
            foreach (var _entry in _trainerPreData)
            {
                var _npcId = (int)_entry[0];
                var _skill = (SkillLine)_entry[1];
                var _minLevel = (_entry.Count() >= 3)
                                        ? (int)_entry[2]
                                        : 0;

                var _npcResult = Query.GetNpcById((uint)_npcId);
                var _mapId = (_entry.Count() >= 4)
                                    ? (int)_entry[3]
                                    : _npcResult.MapId;
                var _npcLocation = (_entry.Count() >= 5)
                                            ? (Vector3)_entry[4]
                                            : _npcResult.Location;
                var _factionId = (_entry.Count() >= 6)
                                        ? (uint)(int)_entry[5]
                                        : _npcResult.Faction;
                var _isHostile = (_entry.Count() >= 6)
                                        ? false
                                        : _npcResult.IsHostile;

                var _realEntry = new object[] {
                                        _npcId,
                                        _skill,
                                        _minLevel,
                                        _mapId,
                                        _npcLocation,
                                        _factionId};

                if(!_isHostile)
                    _trainerData.Add(_realEntry);
            }

            /*
            string _log = "\n";
            foreach (var _e in _trainerData)
            {
                _log += "new object[] {{";
                foreach (var _r in _e)
                    _log += $"{_r}, \t";
                // new object[] { 98222,   SkillLine.Herbalism,    80,     1,  new Vector3(-6657.887f, -2044.802f, -253.1613f),    "AH"},
                _log += "}}, \n";
            }
            DevLog(_log);
            */
        }

        /**/

        private object[] GetClosestTrainerEntry(SkillLine _skill)
        {
            var _entry = _trainerData.Where(e => _skill == (SkillLine)e[1] &&
                                                 Me.Level >= (int)e[2] &&
                                                 Me.MapId == (int)e[3] &&
                                                 (int)Me.GetReputationLevelWith((uint)e[5]) >= 3)
                                      .OrderBy(e => Me.Location.Distance((Vector3)e[4]))
                                      .FirstOrDefault();

            return _entry;
        }

        /*/
         
        private object[] GetClosestTrainerEntry(SkillLine _skill)
        {
            var _entry = _trainerData.Where(e => _skill == (SkillLine)e[1] &&
                                                 Me.Level >= (int)e[2] &&
                                                 Me.MapId == (int)e[3])
                                      .OrderBy(e => Me.Location.Distance((Vector3)e[4]))
                                      .FirstOrDefault();

            return _entry;
        }

        /**/

        private List<object[]> _trainerPreData = new List<object[]>()
        {
            // KALIMDOR
                // HORDE
                    // ORGRIMMAR
                        new object[] { 44919,   SkillLine.Riding},
                        new object[] { 47571,   SkillLine.Archaeology},    
                        new object[] { 3357,    SkillLine.Mining},     
                        new object[] { 46741,   SkillLine.Herbalism},   
                    // THUNDERBLUFF 
                        new object[] { 3013,    SkillLine.Herbalism},
                        new object[] { 3001,    SkillLine.Mining}, 
                // ALLIANCE  
                    // DARKSHORE 
                        new object[] { 43431,   SkillLine.Mining}, 
                // BOTH                   
                    // MOONGLADE
                        new object[] { 12025,   SkillLine.Herbalism,    80},
                    // UN'GORO CRATER
                        new object[] { 98222,   SkillLine.Herbalism,    80,     1,  new Vector3(-6657.887f, -2044.802f, -253.1613f),    35},
                    // TANARIS
                        new object[] { 8128,    SkillLine.Mining,       80},

            // EASTERN KINGDOMS
                // HORDE
                    // UNDERCITY
                        new object[] { 4598,    SkillLine.Mining},
                        new object[] { 4614,    SkillLine.Herbalism},
                // ALLIANCE
                    // STORMWIND
                        new object[] { 44238,   SkillLine.Archaeology},
                        new object[] { 43769,   SkillLine.Riding},
                        new object[] { 5513,    SkillLine.Mining},
                        new object[] { 5566,    SkillLine.Herbalism},
                    // IRONFORGE
                        new object[] { 4254,    SkillLine.Mining},
                        new object[] { 5137,    SkillLine.Herbalism},
                // BOTH
                    // THE CAPE OF STRANGLETHORN
                        new object[] { 908,     SkillLine.Herbalism},

            // OUTLAND
                // HORDE
                    // HELLFIRE PENINSULA
                        new object[] { 18747,   SkillLine.Mining},
                        new object[] { 18748,   SkillLine.Herbalism},
                        new object[] { 35093,   SkillLine.Riding},
                // ALLIANCE
                    // HELLFIRE PENINSULA
                        new object[] { 18779,   SkillLine.Mining},
                        new object[] { 18776,   SkillLine.Herbalism},
                        new object[] { 35100,   SkillLine.Riding},
                // BOTH
                    // SHATTRATH
                        // THE ALDORS
                            new object[] { 33682,   SkillLine.Mining},
                            new object[] { 33678,   SkillLine.Herbalism},
                        // THE SCRYERS
                            new object[] { 33617,   SkillLine.Mining},
                            new object[] { 33616,   SkillLine.Herbalism},

            // NORTHREND
                // HORDE
                    // BOREAN TUNDRA
                        new object[] { 35135,   SkillLine.Riding},
                        new object[] { 26976,   SkillLine.Mining},
                        new object[] { 26994,   SkillLine.Herbalism},
                // ALLIANCE
                    // BOREAN TUNDRA
                        new object[] { 35133,   SkillLine.Riding},
                        new object[] { 26999,   SkillLine.Mining},
                        new object[] { 26974,   SkillLine.Herbalism},
                    // HOWLING FJORD
                        new object[] { 26912,   SkillLine.Mining},
                        new object[] { 26910,   SkillLine.Herbalism},
                // BOTH
                    // SHOLAZAR BASIN
                        new object[] { 28746,   SkillLine.Riding},
                        new object[] { 98223,   SkillLine.Herbalism,  0,  571,    new Vector3(5858.911f, 4145.91f, -94.90401f),     35},
                    // DALARAN
                        new object[] { 31238,   SkillLine.Riding},
                        new object[] { 28698,   SkillLine.Mining},
                        new object[] { 28704,   SkillLine.Herbalism},

            // PANDARIA
                // HORDE
                    // THE JADE FOREST
                        new object[] { 66979,   SkillLine.Mining},
                        new object[] { 66980,   SkillLine.Herbalism},
                // ALLIANCE
                    // THE JADE FOREST
                        new object[] { 67024,   SkillLine.Mining},
                        new object[] { 67025,   SkillLine.Herbalism},
                // BOTH
                    // VALLEY OF THE FOUR WINDS
                        new object[] { 65877,   SkillLine.Herbalism},

            /* NOT IN HB DB > ADD MANUALLY ++++
            // BROKEN ISLES
                // BOTH
                    // DALARAN
                        new object[] { 93189,   SkillLine.Mining},
                        new object[] { 92459,   SkillLine.Herbalism},
                    // VAL'SHARAH
                        new object[] { 98135,   SkillLine.Herbalism},
                    // HIGHMOUNTAIN
                        new object[] { 93691,   SkillLine.Mining},
           */
        };

        // NpcId, Skill, MapId, Location, FactionId, MinLevel
        /*
        private List<object[]> _trainerData = new List<object[]>
        {
            // ORGRIMMAR
                new object[] { 1637, SkillLine.Mining,      3357 },
                new object[] { 1637, SkillLine.Herbalism,   46741 },
              //  new object[] { 1637, SkillLine.Skinning,    7088 },
                new object[] { 1637, SkillLine.Archaeology, 47571 },
            // STORMWIND
                new object[] { 1519, SkillLine.Mining,      5513 },
                new object[] { 1519, SkillLine.Herbalism,   5566 },
               // new object[] { 1519, SkillLine.Skinning,    1292 },
                new object[] { 1519, SkillLine.Archaeology, 44238 }
        };
        */

        private List<SkillLine> _gatheringProfessions = new List<SkillLine>
        {
            SkillLine.Herbalism,
            SkillLine.Mining,
          //  SkillLine.Skinning
        };

        private async Task<bool> HandleTrainSkill()
        {
            if (!Settings.GatheringProfessions_TrainSkill)
                return false;

            if (Me.IsNeutral)       // PANDAREN
                return false;

            if (TreeRoot.GoalText != null && StringContains(TreeRoot.GoalText, "OR", "Escort", "UseTransport"))
                return false;

            var _nodeId = (Me.IsHorde)
                            ? 536
                            : 582;
            var _checkNode = KnownFlightNodesManager.EnumerateKnownFlightNodes()
                                    .FirstOrDefault(n => n.Id == _nodeId);
            if (Me.Level <= 10 && !_inCapital && (!_inCapitalSurroundingZone || _checkNode == null) )
                return false;

            if (_checkedGatheringSkills)
                return false;

            if (Me.IsActuallyInCombat)
                return false;

            var _skill = _gatheringProfessions.FirstOrDefault(s => SkillNeedsTraining(s));
            if (_skill == 0)
            {
                _checkedGatheringSkills = true;
                return false;
            }

            /*
            if (!_inCapital)
            {
                if (!Settings.Hearth_Hearthstone ||
                    !Settings.Hearth_SetHearthstone ||
                    UnfavourableAreaSet())
                    return false;

                if (Me.Level < 10)
                    return false;

                DevLog("Hearth.Capital: TrainGatheringSkill");
                Log("Preparing for training {0}", _skill);
                HearthState = Hearth.Capital;
                return false;
            }
            

            var _trainerDataEntry = _trainerData.FirstOrDefault(e => (int)(e[0]) == Me.ZoneId &&
                                                                     (SkillLine)(e[1]) == _skill);
            var _npcId = (int)_trainerDataEntry[2];
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            */

            var _trainerDataEntry = GetClosestTrainerEntry(_skill);
            if (_trainerDataEntry == null)
                return false;

            var _npcId = (int)_trainerDataEntry[0];
            var _npcLocation = (Vector3)_trainerDataEntry[4];

            await TrainSkill(_skill, _npcId, _npcLocation);
            return true;
        }

        private SkillLine GetLowestNodeSkill()
        {
            var _availableSkills = _gatheringProfessions
                                        .Where(s => Me.GetSkill(s).CurrentValue > 0)
                                        .OrderBy(s => (Me.Race == WoWRace.Tauren && s == SkillLine.Herbalism)
                                                            ? Me.GetSkill(s).CurrentValue + 15
                                                            : Me.GetSkill(s).CurrentValue);

            if (_availableSkills.Count() == 0)
                return 0;

            return _availableSkills.First();
        }

        private List<int> _maxLevelData = new List<int>
        {
            60,	 // CLASSIC
			70,	 // BC
			80,  // WOTLK
			85,  // CATA
			90   // PANDA
	    };

        private double GetOptimumSkillValue()
        {
            double _optimumSkillValue = 0;
            if (Me.Level < 60)
            {
                _optimumSkillValue = (Me.Level + 1) * 5;
            }
            else if (Me.Level >= 90)
            {
                var _maxExpansionSkillLevel = (_accountExpansionLevel >= 6)
                                                    ? 800
                                                    : 700;
                _optimumSkillValue = Math.Min(_maxExpansionSkillLevel, 600 + ((Me.Level - 90) + 1) * 10);
            }
            else
            {
                var _nextMaxLevel = _maxLevelData.FirstOrDefault(l => Me.Level < l);
                var _lastMaxLevel = _maxLevelData
                                        .OrderByDescending(l => l)
                                        .FirstOrDefault(l => Me.Level >= l);
                var _index = _maxLevelData.IndexOf(_lastMaxLevel);

                var _startSkill = 300 + _index * 75;
                var _skillPerLevel = 75f / (_nextMaxLevel - _lastMaxLevel);
                var _deltaLevel = Me.Level - _lastMaxLevel;
                _optimumSkillValue = Math.Ceiling(_startSkill + (_deltaLevel + 1) * _skillPerLevel);
            }

            return _optimumSkillValue;
        }

        public string ToMD5Hash(string _string)

        {
            MD5 _md5 = MD5.Create();
            byte[] _stringBytes = Encoding.ASCII.GetBytes(_string);
            byte[] _hash = _md5.ComputeHash(_stringBytes);

            StringBuilder _sb = new StringBuilder();

            for (int i = 0; i < _hash.Length; i++)
            {
                _sb.Append(_hash[i].ToString("X2"));
            }

            return _sb.ToString();
        }

        private string SafeGuid(WoWGuid _guid)
        {
            var _md5 = ToMD5Hash(_guid.ToString());
            return String.Format("{0}{1}", _md5.Substring(0, 5),
                                           _md5.Substring(_md5.Length - 5));
        }

        private bool IsNodeValuable(WoWGameObject _node)
        {
            if (!_ignoredNodeGuids.ContainsKey(_node.Guid))
                _ignoredNodeGuids.Add(_node.Guid, 3);

            if (_ignoredNodeGuids[_node.Guid] == 0)
                return false;

            var _mobsNearNode = ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                                    .Where(u => u.IsValid &&
                                                u.IsAlive &&
                                                u.IsHostile &&
                                                u.Location.Distance(_node.Location) < 30 &&
                                                u.Level > Me.Level - 2);
            if (_mobsNearNode.Count() > 5)
                return false;

            var _mobsNearNodeClosest = _mobsNearNode
                                            .Where(u => u.Location.Distance(_node.Location) < 15);
            if (_mobsNearNodeClosest.Count() > 3)
                return false;

            var _correctLocation = (Me.IsOutdoors)
                                        ? _node.IsOutdoors
                                        : _node.IsIndoors;
            if (!_correctLocation)
                return false;

            var _skill = (_node.IsHerb)
                            ? SkillLine.Herbalism
                            : SkillLine.Mining;
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                ? _skillValue + 15
                                : _skillValue;
            int _greySkillLevel = -1;
            _node.GetDataSlot(GameObjectDataSlot.GreySkillLevel, out _greySkillLevel);
            if (_skillValue >= _greySkillLevel)
            {
                DevLog("Node blacklisted");
                DevLog("     Name: {0} (Id: {1})", _node.Name, _node.Entry);
                DevLog("     Guid: {0}", SafeGuid(_node.Guid));
                DevLog("     SkillValue >= GreySkillvalue: {0} >= {1}", _skillValue, _greySkillLevel);

                _ignoredNodeGuids[_node.Guid] = 0;
                return false;
            }

            var _requiredSkillValue = _node.RequiredSkill;
            if (_skillValue < _requiredSkillValue)
            {
                /*
                DevLog("Node not (yet) valuable");
                DevLog("     Name: {0} (Id: {1})", _node.Name, _node.Entry);
                DevLog("     Guid: {0}", SafeGuid(_node.Guid));
                DevLog("     SkillValue < RequiredSkillvalue: {0} < {1}", _skillValue, _requiredSkillValue);
                */
                return false;
            }

            var _optimumSkillValue = GetOptimumSkillValue();
            if (_skillValue >= _optimumSkillValue)
            {
                DevLog("Node blacklisted");
                DevLog("     Name: {0} (Id: {1})", _node.Name, _node.Entry);
                DevLog("     Guid: {0}", SafeGuid(_node.Guid));
                DevLog("     SkillValue >= OptimumSkillvalue: {0} >= {1}", _skillValue, _optimumSkillValue);

                _ignoredNodeGuids[_node.Guid] = 0;
                return false;
            }

            return true;
        }

        private async Task<bool> HandleGetNode()
        {
            if ((int)Settings.GatheringProfessions_HandleNodes == 0)
                return false;

            if (!Settings.GatheringProfessions_Professions.ToString().Contains("Herbalism") &&
                !Settings.GatheringProfessions_Professions.ToString().Contains("Mining"))
                return false;

            RuntimeLog(">>");

            if ((Me.IsActuallyInCombat && !Me.Mounted) || DeltaTime(_timeInterrupted, "s") < 5)
                return false;

            RuntimeLog(">>");

            if (_node == null)
                return false;

            if (_witheredHungerer != null)
                return false;

            RuntimeLog(">>"); /*

            if ((Me.IsActuallyInCombat && !Me.Mounted) || DeltaTime(_timeInterrupted, "s") < 5)
                return false;

                    RuntimeLog(">>"); */

            if (TreeRoot.GoalText != null && StringContains(TreeRoot.GoalText, "OR", "Escort", "UseTransport") )
                return false;

            RuntimeLog(">>");

            if (Me.HasAura("Resurrection Sickness"))
                return false;

            RuntimeLog(">>");

            if (Me.CurrentMap.ExpansionId == 5 &&
                !IsQuestCompleted((Me.IsHorde)
                                            ? 31766
                                            : 31733))
                return false;

            RuntimeLog(">>");

            var _skill = (_node.IsHerb)
                              ? SkillLine.Herbalism
                              : SkillLine.Mining;
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                            ? _skillValue + 15
                            : _skillValue;
            var _skillMax = Me.GetSkill(_skill).MaxValue;
            var _skillMaxWithBonus = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                                ? _skillMax + 15
                                                : _skillMax;

            Log("Gathering {0}", (_skill == SkillLine.Herbalism)
                                    ? "Herb"
                                    : "Ore");
            if (GetOptimumSkillValue() > _skillMaxWithBonus)
                Log("     OptimumSkillValue: {0} ({1})", _skillMaxWithBonus, GetOptimumSkillValue());
            else
                Log("     OptimumSkillValue: {0}", GetOptimumSkillValue());
            Log("     PreSkillValue: {0}", _skillValue);

            var _nodeLocation = _node.Location;
            var _nodeInteractRange = _node.InteractRange;
            var _nodeGuid = _node.Guid;
            var _nodeIsHerb = _node.IsHerb;

            var _preGatheringItemCount = _gatheringItemCount;
            var _startGathering = DateTime.Now;

            if (!_ignoredNodeGuids.ContainsKey(_nodeGuid))
                _ignoredNodeGuids.Add(_nodeGuid, 3);
            else
                _ignoredNodeGuids[_nodeGuid] = Math.Max(0, _ignoredNodeGuids[_nodeGuid] - 1);

            while (_gatheringItemCount == _preGatheringItemCount && !breakOut)
            {
                RuntimeLog("Start");

                if (_nodeTimer.IsFinished)
                {
                    DevLog("WHILE: GatherNode");
                    DevLog("     Guid: {0}", SafeGuid(_nodeGuid));
                    DevLog("     GatheringTries left: {0}", (!_ignoredNodeGuids.ContainsKey(_nodeGuid))
                                                                    ? " -/- "
                                                                    : $"{_ignoredNodeGuids[_nodeGuid]}");
                    DevLog("     Distance: {0}", Math.Round(Me.Location.Distance(_nodeLocation)));
                    DevLog("     TimeLeft: {0}", Math.Ceiling(40 - DeltaTime(_startGathering, "s")));
                    _nodeTimer.Reset();
                }

                RuntimeLog();

                if (_nodeRefreshTimer.IsFinished && _node != null)
                {

                    if (_nodeGuid != _node.Guid)
                    {
                        _nodeGuid = _node.Guid;
                        DevLog("     >>> Switching node <<<");
                    }

                    _nodeLocation = _node.Location;
                    _nodeInteractRange = _node.InteractRange;

                    _nodeRefreshTimer.Reset();
                }

                /*
                if (Me.Location.Distance(_nodeLocation) < 50 && _node == null)
                {
                    DevLog("     Node == null");
                    _ignoredNodeGuids[_nodeGuid] = 0;
                    return false;
                }
				*/

                RuntimeLog();

                if (DeltaTime(_startGathering, "s") > 40 && !Me.IsCasting &&
                    (_node == null || !_node.WithinInteractRange))
                {
                    DevLog("WHILE: GatherNode");
                    DevLog("     MaxTime reached");
                    _ignoredNodeGuids[_nodeGuid] = 0;
                    return false;
                }

                RuntimeLog();

                if (Me.Location.Distance(_nodeLocation) > _nodeInteractRange)
                {
                    RuntimeLog();

                    if (!await MoveTo(_nodeLocation))
                    {
                        DevLog("     Moving failed");
                        _ignoredNodeGuids[_nodeGuid] = 0;
                        return false;
                    }

                    RuntimeLog();
                }
                else
                {
                    DevLog("     WithinInteractRange \nNode == null: {0} \nGuid: {1} \nGatheringTries left: {2}",
                                _node == null,
                                SafeGuid(_nodeGuid),
                                (!_ignoredNodeGuids.ContainsKey(_nodeGuid))
                                            ? " -/- "
                                            : $"{_ignoredNodeGuids[_nodeGuid]}");

                    if (_ignoredNodeGuids[_nodeGuid] == 0)   // ++++
                        _ignoredNodeGuids[_nodeGuid] = 2;

                    if (!Me.IsCasting)
                    {
                        if (Me.IsFlying)
                        {
                            if (!await CommonCoroutines.LandAndDismount(null, false, _nodeLocation, 8))
                                await CommonCoroutines.LandAndDismount(null, true, _nodeLocation, 8);
                        }
                        await CommonCoroutines.StopMoving();

                        if (_node != null)
                        {
                            if (!IsQuestCompleted(38786))
                            {
                                var _questItem = Me.BagItems.FirstOrDefault(i => i.Entry == 128767);
                                if (_questItem != null)
                                {
                                    _questItem.Interact();
                                    await Sleep();
                                }
                            }
                            // await Sleep();
                            _node.Interact();

                            if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                            {
                                if (await Coroutine.Wait(10 * 1000, () => !Me.IsCasting &&
                                                                          (
                                                                            DeltaTime(_timeInterrupted, "s") < 5 ||
                                                                            DeltaTime(_timeSucceeded, "s") < 5
                                                                           )))
                                {
                                    if (DeltaTime(_timeSucceeded, "s") < 5)
                                    {
                                        if (await Loot())
                                        {
                                            await Coroutine.Wait(2 * 1000, () => _gatheringItemCount != _preGatheringItemCount);
                                            DevLog("     Succeeded");
                                            _ignoredNodeGuids[_nodeGuid] = 0;

                                            if (Me.Shapeshift == ShapeshiftForm.Travel && !Me.HasAura("Stag Form"))
                                                await Move("JF", 250, true);

                                            break;
                                        }
                                    }
                                    else
                                    {
                                        _ignoredNodeGuids[_nodeGuid] += 0.5;
                                        DevLog("     Interrupted");
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }

                RuntimeLog("Sum");
                await Coroutine.Yield();
            }

            DevLog("     GatheringTries left: {0}", _ignoredNodeGuids[_nodeGuid]);

            _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                ? _skillValue + 15
                                : _skillValue;

            Log("     PostSkillValue: {0}", _skillValue);
            _lowestNodeSkill = GetLowestNodeSkill();
            _checkedGatheringSkills = false;

            return true;
        }

        private async Task<bool> HandleGraphicMode()
        {
            if (!Settings.Performance_LowGraphicMode &&
                _renderScale == Settings.Performance_DefaultRenderScale / 100 &&
                _maxFps == Settings.Performance_MaxFps &&
                _maxFpsBk == Settings.Performance_MaxFpsBk)
                return false;

            if (Settings.Performance_LowGraphicMode &&
                _renderScale == 0.1 &&
                _maxFps == 20 &&
                _maxFpsBk == 20)
                return false;

            if (!Settings.Performance_LowGraphicMode)
            {
                Log("Restoring graphic mode");
                LowGraphicMode(false);
            }
            else
            {
                Log("Enabling Low Graphic Mode");
                LowGraphicMode();
            }

            return true;
        }

        private async Task<bool> HandleGatheringSettings()
        {
            var _herbalism = Settings.GatheringProfessions_Professions.ToString().Contains("Herbalism");
            var _mining = Settings.GatheringProfessions_Professions.ToString().Contains("Mining");
            //   var _skinning = Settings.GatheringProfessions_Professions.ToString().Contains("Skinning");
            var _preCharacterSettings = CharacterSettings.Instance.GetSettings();
            var _preGatherbuddySettings = Bots.Gatherbuddy.GatherbuddySettings.Instance.GetSettings();

            RuntimeLog(">");

            CharacterSettings.Instance.LootMobs = (Settings.GatheringProfessions_HarvestMobs)
                                                        ? true
                                                        : CharacterSettings.Instance.LootMobs;
            /*
            CharacterSettings.Instance.SkinMobs = (_skinning)
                                                        ? true
                                                        : false; */

            RuntimeLog(">");

            if (_herbalism && (int)Settings.GatheringProfessions_HandleNodes == 1)
            {
                CharacterSettings.Instance.HarvestHerbs = false;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherHerbs = false;
            }
            else
            {
                CharacterSettings.Instance.HarvestHerbs = true;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherHerbs = true;
            }

            RuntimeLog(">");

            if (_mining && (int)Settings.GatheringProfessions_HandleNodes == 1)
            {
                CharacterSettings.Instance.HarvestMinerals = false;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherMinerals = false;
            }
            else
            {
                CharacterSettings.Instance.HarvestMinerals = true;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherMinerals = true;
            }

            RuntimeLog(">");

            if (CharacterSettings.Instance.LootMobs         != (bool)_preCharacterSettings["LootMobs"] ||
                CharacterSettings.Instance.HarvestHerbs     != (bool)_preCharacterSettings["HarvestHerbs"] ||
                CharacterSettings.Instance.HarvestMinerals  != (bool)_preCharacterSettings["HarvestMinerals"])
                            CharacterSettings.Instance.Save();

            RuntimeLog(">");

            if (BotManager.Current.ToString() != "Gatherbuddy2")
            {
                if (Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherMinerals    != (bool)_preGatherbuddySettings["GatherMinerals"] ||
                    Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherHerbs       != (bool)_preGatherbuddySettings["GatherHerbs"] )
                            Bots.Gatherbuddy.GatherbuddySettings.Instance.Save();
            }

            RuntimeLog(">");

            return true;
        }

        private List<object[]> _legionRankQuestData = new List<object[]>
        {
                                    // Skill,       Type, TypeId, QuestId, TurnInId, AcceptCondition, DoAction

            // MINING
                // LEYSTONE
                    // RANK 1
                        new object[] { "Mining",    "Item", 124489, 38777,  93189 },
                        new object[] { "Mining",    "Item", 124490, 38784,  93189 },
                        new object[] { "Mining",    "Item", 124491, 38785,  93189 },
                    // RANK 2
                        new object[] { "Mining",    "Npc",  93189,  38888,  93691,  38777 },
                        new object[] { "Mining",    "Npc",  93189,  38888,  93691,  38784 },
                        new object[] { "Mining",    "Npc",  93189,  38888,  93691,  38785 },                // || or && ?
                        new object[] { "Mining",    "Npc",  93691,  38786,  93691,  38888 },                // PICK ON NODES
                        new object[] { "Mining",    "Npc",  93691,  38787,  93691,  38786,  "Gossip" },     // GOSSIP
                        new object[] { "Mining",    "Item", 124492, 38789,  93691 },
                        new object[] { "Mining",    "Item", 124493, 38790,  93691 },
                        new object[] { "Mining",    "Item", 124494, 38791,  93691 },
                    // RANK 3
                        new object[] { "Mining",    "Npc",  93735,  38792,  93691 },               // RETHU
                        new object[] { "Mining",    "Npc",  93743,  38793,  93691 },               // RETHU
                        new object[] { "Mining",    "Npc",  93744,  38794,  93691 },               // RETHU
                // FELSLATE
                    // RANK 1
                        new object[] { "Mining",    "Item", 124496, 38795, 93189 },
                        new object[] { "Mining",    "Item", 124497, 38796, 93189 },
                        new object[] { "Mining",    "Item", 124498, 38797, 93189 },
                    // RANK 2
                        new object[] { "Mining",    "Npc",  93189,  38901, 92264,   38795 },
                        new object[] { "Mining",    "Npc",  93189,  38901, 92264,   38796 },
                        new object[] { "Mining",    "Npc",  93189,  38901, 92264,   38797 },      // || or && ?
                        new object[] { "Mining",    "Npc",  92264,  38798, 92264,   38901 },       //+++++++++++
                        new object[] { "Mining",    "Npc",  92264,  38799, 92264,   38798 },       //+++++++++++
                        new object[] { "Mining",    "Item", 124501, 38800, 92264 },
                        new object[] { "Mining",    "Item", 124500, 38801, 92264 },
                        new object[] { "Mining",    "Item", 124499, 38802, 92264 },
                    // RANK 3

                                // +++++++++++

                // INFERNAL BRIMSTONE
                    // RANK 1
                        new object[] { "Mining",    "Item", 124502, 38806, 93189 },
                    // RANK 2
                        new object[] { "Mining",    "Npc",  92464,  38807, 92464,   38806 },    // ++++++
                    // RANK 3
                        new object[] { "Mining",    "Npc",  92464,  39790, 97360,   38807 },
                        new object[] { "Mining",    "Npc",  97360,  39763, 97360,   39790 },    // ++++++
                        new object[] { "Mining",    "Npc",  97360,  39817, 92464,   39763 },
                        new object[] { "Mining",    "Npc",  92464,  39830, 92464,   39763 },    // ++++++

            // HERBALISM
                // AETHRIL
                    // RANK 1
                        new object[] { "Herbalism", "Item", 129117, 40013, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40014, 92464,   40013 },
                    // RANK 2
                        new object[] { "Herbalism", "Item", 129135, 40015, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40016, 92464,   40015 },
                    // RANK 3
                        new object[] { "Herbalism", null,   0,      40017, 92464,   0 },            // AUTO ACCEPTED
                // DREAMLEAF
                    // RANK 1
                        new object[] { "Herbalism", "Item", 129118, 40018, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40019, 92464,   40018 },
                    // RANK 2
                        new object[] { "Herbalism", "Item", 129136, 40020, 98135 },
                        new object[] { "Herbalism", "Npc",  98135,  40021, 98135,   40020 },
                    // RANK 3
                        new object[] { "Herbalism", "Item", 129141, 40022, 98135 },
                        new object[] { "Herbalism", "Npc",  98135,  40023, 98135,   40022 },    // +++++++
                // FOXFLOWER
                    // RANK 1
                        new object[] { "Herbalism", "Item", 129119, 40024, 98135 },
                    // RANK 2
                        new object[] { "Herbalism", "Item", 129137, 40025, 98135 },
                        new object[] { "Herbalism", "Npc",  98135,  40026, 98135,   40025 },    // +++++++
                    // RANK 3
                        new object[] { "Herbalism", "Item", 129278, 40028, 98135 },
                // FJARNSKAGGL
                    // RANK 1
                        new object[] { "Herbalism", "Item", 129120, 40029, 92464 },
                    // RANK 2
                        new object[] { "Herbalism", "Item", 129138, 40030, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40031, 92464,   40030 },    // +++++
                    // RANK 3
                        new object[] { "Herbalism", "Item", 129142, 40032, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40033, 92464,   40032 },    // +++++
                // STARLIGHT ROSE
                    // RANK 1
                        new object[] { "Herbalism", "Item", 129121, 40034, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40035, 92464,   40034 },
                    // RANK 2
                        new object[] { "Herbalism", "Item", 129140, 40036, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40037, 92464,   40036 },    // +++++
                    // RANK 3
                        new object[] { "Herbalism", "Item", 129143, 40038, 92464 },
                        new object[] { "Herbalism", "Npc",  92464,  40039, 92464,   40038 },    // +++++
                // FELWORT
                    // RANK 1
                        new object[] { "Herbalism", "Item", 129122, 40040, 92464 },
                    // RANK 2
                        new object[] { "Herbalism", "Npc",  92464,  40041, 92464,   40040 },    // ++++++
                    // RANK 3
                        new object[] { "Herbalism", "Npc",  92464,  40042, 92464,   40041 },    // +++++
        };

        private Vector3 GetNpcLocation(int _npcId)
        {
            var _entry = _questNpcData.FirstOrDefault(e => _npcId == (int)e[0]);
            if (_entry == null)
                return Vector3.Zero;

            return (Vector3)_entry[1];
        }

        private List<object[]> _questNpcData = new List<object[]>()
        {
            new object[] { 93189, new Vector3(-729.5382f,   4493.964f,  738.8925f) },
            new object[] { 93691, new Vector3(2990.198f,    3794.458f,  1347.548f) },
            new object[] { 92264, new Vector3(1465.878f,    4993.25f,   182.8172f) },
            new object[] { 92464, new Vector3(-769.3635f,   4519.786f,  732.4017f) },
            new object[] { 98135, new Vector3(2327.461f,    6584.921f,  147.7281f) } 
           //  new object[] { 97360, new Vector3() },                                           +++++           
        };

        private bool CanAcceptQuest(object[] _questEntry)
        {
                  RuntimeLog(">>>>");

            var _skill = (string)_questEntry[0];
            if (!Settings.GatheringProfessions_Professions.ToString().Contains(_skill))
                return false;

                    RuntimeLog(">>>>");

            var _acceptCondition = _questEntry.Length < 6 || 
                                   IsQuestCompleted((int)_questEntry[5]);
            if (!_acceptCondition)
                return false;

                    RuntimeLog(">>>>");

            var _questId = (int)_questEntry[3];
            if (HasQuest(_questId) || IsQuestCompleted(_questId))
                return false;

                    RuntimeLog(">>>>");

            var _quest = Quest.FromId((uint)_questId);
            if (_quest == null || _quest.RequiredLevel > Me.Level)
                return false;

                    RuntimeLog(">>>>");

            var _type = (string)_questEntry[1];
            if (_type == "Npc")
            {
                var _npcId = (int)_questEntry[2];
                var _npc = GetNpc(_npcId);
                if (_npc == null || _npc.QuestGiverStatus != QuestGiverStatus.Available)
                    return false;
            }
            else if (_type == "Item")
            {
                var _itemId = (int)_questEntry[2];
                var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                            i.Entry == _itemId);
                if (_item == null)
                    return false;
            }

            return true;
        }

        private bool IsQuestCompleted(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _isDoneAndCompleted = _completedQuests.Any(id => id == (uint)_id);
                if (_isDoneAndCompleted)
                    return true;

                var _isActiveAndCompleted = _allQuests.Any(q => q.Id == (uint)_id &&
                                                                q.IsCompleted);
                if (_isActiveAndCompleted)
                    return true;
            }

            return false;
        }

        private static int[] StringToArray(string _string)
        {
            var _regex = new Regex("\\d+");
            var _result = _regex.Matches(_string);
            var _list = new List<int>();

            foreach (var _entry in _result)
            {
                _list.Add(Convert.ToInt32(_entry.ToString()));
            }

            return _list.ToArray();
        }

        private bool HasItem(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _hasItem = Me.BagItems.Any(i => i.IsValid &&
                                                    i.Entry == (uint)_id);
                if (_hasItem)
                    return true;
            }

            return false;
        }

        private bool HasQuest(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _hasQuest = _allQuests.Any(q => q.Id == (uint)_id);
                if (_hasQuest)
                    return true;
            }

            return false;
        }

        private async Task<bool> HandleAcceptQuest()
        {
            RuntimeLog(">>>");

            if (!Settings.GatheringProfessions_Professions.ToString().Contains("Mining") &&
                !Settings.GatheringProfessions_Professions.ToString().Contains("Herbalism"))
                return false;

            RuntimeLog(">>>");

            var _rethu = GetNpc(93735, 93743, 93744);
            if (!_checkedItems && !_checkedQuests && _rethu == null)
                return false;

            RuntimeLog(">>>");

            var _questDataEntry = _legionRankQuestData
                                        .FirstOrDefault(e => CanAcceptQuest(e));

            RuntimeLog(">>>");

            if (_questDataEntry == null)
            {
                RuntimeLog(">>>");
                return false;
            }

            RuntimeLog(">>>");

            DevLog("     {0}", String.Join(" - ", _questDataEntry) );

            var _type = (string)_questDataEntry[1];
            var _questId = (int)_questDataEntry[3];
            if (_type == "Npc")
            {
                var _npcId = (int)_questDataEntry[2];
                var _npc = GetNpc(_npcId);
                var _npcResult = Query.GetNpcById((uint)_npcId);

                if (_npc == null && _npcResult == null)
                    return false;

                var _npcLocation = _npcResult.Location;

                Log("Accepting quest from {0}", _npc.Name);
                Log("     QuestId: {0}", _questId);
                Log("     QuestGiverStatus: {0}", _npc.QuestGiverStatus);
                while (!HasQuest(_questId) && !breakOut)
                {
                    DevLog("WHILE: HandleAcceptQuest > Npc");

                    _npc = GetNpc(_npcId);
                    if (_npc != null)
                        _npcLocation = _npc.Location;
                    if (_npc == null || !_npc.WithinInteractRange)
                        await MoveTo(_npcLocation, true);
                    else
                    {
                        await CommonCoroutines.StopMoving();

                        await Sleep();
                        _npc.Interact();
                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("QuestFrame") || IsFrameVisible("GossipFrame"));

                        if (IsFrameVisible("GossipFrame"))
                        {
                            var _availableQuests = GossipFrame.Instance.AvailableQuests;
                            var _questEntry = _availableQuests.FirstOrDefault(e => e.Id == _questId);
                            await Sleep();
                            GossipFrame.Instance.SelectAvailableQuest(_questEntry.Index);
                            await Coroutine.Wait(3 * 1000, () => IsFrameVisible("QuestFrame"));
                        }

                        QuestFrame.Instance.AcceptQuest();
                        await Coroutine.Wait(3 * 1000, () => HasQuest(_questId));
                        await Sleep();
                        QuestFrame.Instance.Hide();
                    }

                    await Coroutine.Yield();
                }
            }
            else if (_type == "Item")
            {
                var _itemId = (int)_questDataEntry[2];
                var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                            i.Entry == _itemId);

                Log("Accepting quest from {0}", _item.Name);
                Log("     QuestId: {0}", _questId);
                while (!HasQuest(_questId) && !breakOut)
                {
                    DevLog("WHILE: HandleAcceptQuest > Item");

                    if (await UseItem(_item))
                    {
                        if (await Coroutine.Wait(3 * 1000, () => IsFrameVisible("QuestFrame")))
                        {
                            QuestFrame.Instance.AcceptQuest();
                            QuestFrame.Instance.Hide();
                            await Coroutine.Wait(3 * 1000, () => HasQuest(_questId));
                        }
                    }
                    else
                        break;

                    await Coroutine.Yield();
                }
            }
            
            return true;
        }

        private async Task<bool> TurnInQuest(int _questId, int _npcId)
        {
            if (!HasQuest(_questId))
                return false;

            var _questName = QuestLog.Instance.GetAllQuests()
                                    .FirstOrDefault(q => q.Id == _questId).Name;
            var _npcResult = Query.GetNpcById((uint)_npcId);
            var _npcLocation = (_npcResult == null)
                                    ? GetNpcLocation(_npcId)
                                    : _npcResult.Location;            
            var _npc = GetNpc(_npcId);
            if (_npc == null && _npcLocation == Vector3.Zero)
                return false;

            Log("Turning in quest");
            Log("     {0} (Id: {1})", _questName, _questId);

            while (HasQuest(_questId) && !breakOut)
            {
                DevLog("WHILE: TurnInQuest");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;
                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation, true);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();
                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("QuestFrame") || IsFrameVisible("GossipFrame"));

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _activeQuests = GossipFrame.Instance.ActiveQuests;
                        var _questEntry = _activeQuests.FirstOrDefault(e => e.Id == _questId);
                        await Sleep();
                        GossipFrame.Instance.SelectActiveQuest(_questEntry.Index);
                        await Coroutine.Wait(2000, () => IsFrameVisible("QuestFrame"));

                        if (!IsFrameVisible("QuestFrame"))
                        {
                            await Sleep();
                            GossipFrame.Instance.SelectActiveGossipQuest(_questEntry.Index);
                            await Coroutine.Wait(2000, () => IsFrameVisible("QuestFrame"));
                        }
                    }

                    while (IsFrameVisible("QuestFrame") && HasQuest(_questId) && !breakOut)
                    {
                        await Sleep();
                        QuestFrame.Instance.CompleteQuest();
                        await Coroutine.Wait(2000, () => !HasQuest(_questId));
                        await Coroutine.Yield();
                    }
                    QuestFrame.Instance.Hide();
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleTurnInQuest()
        {
            RuntimeLog(">>>");

            if (!Settings.GatheringProfessions_Professions.ToString().Contains("Mining") &&
                !Settings.GatheringProfessions_Professions.ToString().Contains("Herbalism"))
                return false;

            RuntimeLog(">>>");

            var _questDataEntry = _legionRankQuestData.FirstOrDefault(e => IsQuestCompleted((int)(e[3])));
            if (_questDataEntry == null)
                return false;

            DevLog("     {0}", String.Join(" - ", _questDataEntry));

            var _questId = (int)_questDataEntry[3];
            var _npcId = (int)_questDataEntry[4];
            await TurnInQuest(_questId, _npcId);

            return true;
        }

        private async Task<bool> HandleLegionRanks()
        {
            RuntimeLog(">>");

            if (await HandleAcceptQuest())
                return true;

            RuntimeLog(">>");

            if (await HandleTurnInQuest())
                return true;

            RuntimeLog(">>");

            return false;
        }

        private WoWUnit _witheredHungerer
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         (
                            u.Entry == 98232 ||
                            u.Entry == 98233
                         ) &&
                         (
                            (
                                u.IsAlive && 
                                !u.TaggedByMe
                            ) ||
                            (
                                u.CanLoot &&
                                u.Lootable &&
                                !u.Looting &&
                                !u.IsAlive
                            )
                         ) &&
                         u.IsValid
                     orderby
                        u.Distance
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private async Task<bool> HandleWitheredHungerer()
        {
            if (_witheredHungerer == null)
                return false;
            
            Log("{0} withered hungerer", (_witheredHungerer.IsAlive)
                                                ? "Attacking"
                                                : "Looting");
            while (_witheredHungerer != null && !breakOut)
            {
                DevLog("WHILE: HandleWitheredHungerer");

                if (!_witheredHungerer.WithinInteractRange)
                    await MoveTo(_witheredHungerer.Location);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await CommonCoroutines.LandAndDismount(null, true);

                    if (!_witheredHungerer.IsAlive)
                    {
                        _witheredHungerer.Interact();
                        await Loot();
                        await Coroutine.Wait(2000, () => _witheredHungerer == null);
                    }
                    else
                    {
                        _witheredHungerer.Target();
                        Lua.DoString("StartAttack()");
                        await Coroutine.Wait(2000, () => _witheredHungerer.TaggedByMe);
                        return false;
                    }
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleGatheringProfessions()
        {
            if (Settings.GatheringProfessions_Professions.ToString() == "None")
                return false;

            if (!Me.IsAlive)
                return false;

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return false;

            await HandleGatheringSettings();

            RuntimeLog(">");

            if (await HandleGetNode())
                return true;

            RuntimeLog(">");

            if (await HandleTrainSkill())
                return true;

            RuntimeLog(">");

            if (Me.CurrentMap.ExpansionId == 6)
                return false; 

            RuntimeLog(">");

            if (await HandleLegionRanks())
                return true;

            RuntimeLog(">");

            if (await HandleWitheredHungerer())
                return true;

            RuntimeLog(">");

            return false;
        }

        private bool _inShrine
        {
            get
            {
                return (Me.IsHorde)
                            ? Me.ZoneId == 6141 || Me.ZoneId == 6554
                            : Me.ZoneId == 6142 || Me.ZoneId == 6553;
            }
        }

        private bool _inCapital
        {
            get
            {
                return (Me.IsHorde)
                            ? Me.ZoneId == 1637
                            : Me.ZoneId == 1519;
            }
        }

        private bool _inCapitalSurroundingZone
        {
            get
            {
                return (Me.IsHorde)
                            ? Me.ZoneId == 14
                            : Me.ZoneId == 12;
            }
        }

        private int _classicHearthstoneAreaId
        {
            get
            {
                return (Me.IsHorde)
                            ? 5170
                            : 5148;
            }
        }

        private int _pandariaHearthstoneAreaId
        {
            get
            {
                return (Me.IsHorde)
                            ? 6519
                            : 6484;
            }
        }

        /*
        private async Task<bool> HandleHearthstoneArea(bool _useTaxi = false, bool _ignoreCapital = false)
        {
            if (!Settings.Hearth_Hearthstone ||
                !Settings.Hearth_SetHearthstone)
                return false;

            var _inValeOfEternalBlossoms =
                 Me.ZoneId == 5840 || ((Me.IsHorde)
                                        ? Me.ZoneId == 6554 || Me.ZoneId == 6141
                                        : Me.ZoneId == 6553 || Me.ZoneId == 6142);
            if (!_ignoreCapital &&
                !_inValeOfEternalBlossoms &&
                !_inCapital &&
                !_inCapitalSurroundingZone)
                return false;

            var _hearthstone = Me.BagItems.FirstOrDefault(i => i.Entry == 6948);
            if (_hearthstone == null)
                return false;

            if (Me.HearthstoneAreaId == _classicHearthstoneAreaId && !_inValeOfEternalBlossoms)
                return false;
            if (Me.HearthstoneAreaId == _pandariaHearthstoneAreaId)
                return false;

            Log("Setting Hearthstone");
            var _npcId = (Me.IsHorde)
                                ? (_inValeOfEternalBlossoms)
                                        ? 63008
                                        : 6929
                                : (_inValeOfEternalBlossoms)
                                        ? 64149
                                        : 6740;
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            var _npc = GetNpc(_npcId);
            var _newHearthstoneAreaId = (_inValeOfEternalBlossoms)
                                                ? _pandariaHearthstoneAreaId
                                                : _classicHearthstoneAreaId;
            while (Me.HearthstoneAreaId != _newHearthstoneAreaId && !breakOut)
            {
                DevLog("WHILE: HearthstoneArea");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;
                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation, _useTaxi);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();
                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("GossipFrame") || IsFrameVisible("StaticPopup1"));

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _binderEntry = GossipFrame.Instance.GossipOptionEntries
                                                .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Binder);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_binderEntry.Index);
                        await Coroutine.Wait(5000, () => IsFrameVisible("StaticPopup1"));
                    }

                    if (IsFrameVisible("StaticPopup1"))
                    {
                        await Sleep();
                        await ClickButton("StaticPopup1Button1");
                    }

                    await Coroutine.Wait(5000, () => Me.HearthstoneAreaId == _newHearthstoneAreaId);
                }

                await Coroutine.Yield();
            }
            _checkedItems = false;
            _checkedQuests = false;
            _checkedGatheringSkills = false;

            return true;
        }
        */

        bool IsSkillMaxedOut(SkillLine _skill)
        {
            var _skillValue = Me.GetSkill(_skill).CurrentValue;

            if (_skillValue == 0)
                return false;

            var _skillMaxValue = (_accountExpansionLevel >= 6)
                                        ? 800
                                        : 700;
            if (_skillValue == _skillMaxValue)
                return true;

            return false;
        }

        private void LowGraphicMode(bool _enable = true)
        {
            if (_enable)
            {
                _renderScale = 0.1;
                _maxFps = 20;
                _maxFpsBk = 20;
            }
            else
            {
                _renderScale = Settings.Performance_DefaultRenderScale / 100;
                _maxFps = Settings.Performance_MaxFps;
                _maxFpsBk = Settings.Performance_MaxFpsBk;
            }
            Lua.DoString($"SetCVar('RenderScale', {_renderScale})");
            Lua.DoString($"SetCVar('MaxFPS', {_maxFps})");
            Lua.DoString($"SetCVar('MaxFPSBk', {_maxFpsBk})");
        }

        private async Task<bool> HandleStop()
        {
            if (Settings.Quest_StopLevel == 0 &&
                Settings.Quest_StopItem == "0" &&
                Settings.Quest_StopQuest == "0" &&
                !Settings.Quest_StopSkillMax)
                return false;

            var _stopLevel = Settings.Quest_StopLevel != 0 && Me.Level >= Settings.Quest_StopLevel;
            var _stopQuest = Settings.Quest_StopQuest != "0" && HasQuest(StringToArray(Settings.Quest_StopQuest));
            var _stopItem = Settings.Quest_StopItem != "0" && HasItem(StringToArray(Settings.Quest_StopItem));

            RuntimeLog(">");

            var _professionsMaxSkilled = _gatheringProfessions
                                                .Where(s => IsSkillMaxedOut(s))
                                                .Count();

            RuntimeLog(">");

            var _stopSkillMax = (Settings.GatheringProfessions_Professions.ToString().Contains("And"))
                                        ? _professionsMaxSkilled == 2
                                        : _professionsMaxSkilled == 1;

            RuntimeLog(">");

            if (!_stopLevel && !_stopQuest && !_stopSkillMax && !_stopItem)
                return false;

            var _stopReason = (_stopItem)
                                    ? "StopItem received"
                                    : ((_stopLevel)
                                            ? "StopLevel reached"
                                            : ((_stopQuest)
                                                    ? "StopQuest accepted"
                                                    : "Gathering Professions max skilled"));

            Log("Stopping HB as {0}", _stopReason);
            LowGraphicMode(false);

            if (Settings.Quest_StopHBRelog)
                Bots.Professionbuddy.Dynamic.Helpers.HBRelog.StopProfile(Bots.Professionbuddy.Dynamic.Helpers.HBRelog.CurrentProfileName);
            else
            {
                Settings.General_InfiniteMode = false;
                TreeRoot.Stop("KeepOnMoving");
            }

            if (Settings.Quest_StopAction != "0")
                Bots.Professionbuddy.Dynamic.Helpers.HBRelog.StartProfile(Settings.Quest_StopAction);

            return true;
        }

        /*
        private async Task<bool> HandleMounts()
        {
            if (Settings.G_GroundMount == "0" && Settings.G_FlyingMount == "0")
                return false;

            var _groundMountSpellId = CharacterSettings.Instance.GroundMountSpellId.ToString();
            var _flyingMountSpellId = CharacterSettings.Instance.FlyingMountSpellId.ToString();

            if (Settings.G_GroundMount != "0" && _groundMountSpellId == Settings.G_GroundMount &&
                Settings.G_FlyingMount != "0" && _flyingMountSpellId == Settings.G_FlyingMount)
                return false;

            if (Settings.G_GroundMount != "0" && _groundMountSpellId != Settings.G_GroundMount)
            {
                var _newSpellId = -1;
                if (Settings.G_GroundMount.All(char.IsDigit))
                    _newSpellId = Convert.ToInt32(Settings.G_GroundMount);
                else
                {
                    var _mount = Mount.GroundMounts.FirstOrDefault(m => m.Name == Settings.G_GroundMount);
                    if (_mount != null)
                        _newSpellId = _mount.CreatureSpellId;
                }

                if (_newSpellId != -1 && SpellManager.CanCast(_newSpellId))
                {
                    CharacterSettings.Instance.GroundMountSpellId = _newSpellId;
                    CharacterSettings.Instance.Save();
                }
            }

            if (Settings.G_FlyingMount != "0" && _flyingMountSpellId != Settings.G_FlyingMount)
            {
                var _newSpellId = -1;
                if (Settings.G_FlyingMount.All(char.IsDigit))
                    _newSpellId = Convert.ToInt32(Settings.G_FlyingMount);
                else
                {
                    var _mount = Mount.FlyingMounts.FirstOrDefault(m => m.Name == Settings.G_FlyingMount);
                    if (_mount != null)
                        _newSpellId = _mount.CreatureSpellId;
                }

                if (_newSpellId != -1 && SpellManager.CanCast(_newSpellId))
                {
                    CharacterSettings.Instance.FlyingMountSpellId = _newSpellId;
                    CharacterSettings.Instance.Save();
                }
            }

            return true;
        }
        */

        private async Task<bool> HandleJump()
        {
            if (!Settings.Jump_Enable)
                return false;

            if (!Me.IsMoving)
                return false;

            if (Settings.Jump_MinPause > Settings.Jump_MaxPause)
                return false;

            if (Me.IsFlying)
                return false;

            if (Me.IsSwimming)
                return false;

            if (Me.Combat && !Me.Mounted)
                return false;

            var _clickPos = WoWMovement.ClickToMoveInfo.ClickPos;
            if (_clickPos.Distance(Me.Location) < 20)
                return false;

            var _inFront = WoWMathHelper.CalculatePointInFront(Me.Location, Me.Rotation, _clickPos.Distance(Me.Location));
            if (_inFront.Distance(_clickPos) > 10)
                return false;

            if (Me.Mounted && Mount.Current.IsFlyingMount && !Me.HasAura("Stag Form"))
                return false;

            if (DeltaTime(_timeLastJump, "ms") <= _deltaNextJump)
                return false;

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return false;

            _timeLastJump = DateTime.Now;
            _deltaNextJump = _rnd.Next(Settings.Jump_MinPause, Settings.Jump_MaxPause);
            await Move("J", 50);

            return true;
        }

        private async Task<bool> Loot()
        {
            /*
            if (DeltaTime(_timeDialogOpened, "ms") > DeltaTime(_timeDialogClosed, "ms"))
                return false;
                */

            if (!await Coroutine.Wait(1000, () => IsFrameVisible("LootFrame") ||
                                                  DeltaTime(_timeDialogOpened, "ms") > DeltaTime(_timeDialogClosed, "ms")))
                return false;

            LootFrame.Instance.LootAll();
            return true;
        }

        private async Task<bool> HandleLoot()
        {
            if (!Settings.General_SpeedLoot && !Settings.GatheringProfessions_HarvestMobs)
                return false;

            if (!CharacterSettings.Instance.LootMobs)
                return false;

            if (_lootableMob == null && (Settings.GatheringProfessions_HarvestMobs && _interactableMob == null))
                return false;

            var _mob = (_lootableMob != null)
                            ? _lootableMob
                            : Settings.GatheringProfessions_HarvestMobs
                                                ? _interactableMob
                                                : null;

            while (_mob != null && !breakOut)
            {
                DevLog("WHILE: HandleLoot");
                if (!Me.IsAlive)
                    break;

                try
                {
                    var _harvest = _mob.CanSkin;

                    while(!_mob.WithinInteractRange && !breakOut)
                    {
                        DevLog("WHILE: HandleLoot > GoTo");

                        Navigator.MoveTo(_mob.Location);
                        await Coroutine.Yield();
                    }
                    await CommonCoroutines.StopMoving();
                    _mob.Interact();

                    if(await Loot())
                    { 
                        if (await Coroutine.Wait(4000, () => (_harvest)
                                                                ? !_mob.CanSkin
                                                                : !_mob.CanLoot))
                        {
                            if (_harvest)
                                break;
                        }
                    }
                }
                catch(Exception)
                {
                    //
                }

                _mob = (_lootableMob != null)
                            ? _lootableMob
                            : Settings.GatheringProfessions_HarvestMobs
                                                ? _interactableMob
                                                : null;
                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleResurrect()
        {
            if (Settings.General_ResurrectionTimer == 0)
                return false;

            if (!Me.IsGhost)
                return false;

            if (_timeDied == DateTime.MinValue)
                _timeDied = DateTime.Now;

            var _beginResurrect = Settings.General_ResurrectionTimer;
            if (DeltaTime(_timeDied, "s") < _beginResurrect)
            {
                var _passedTime = Math.Floor(DeltaTime(_timeDied, "s"));
                if (_passedTime >= 10 && _logTimer.IsFinished)
                {
                    var _remainingTime = _beginResurrect - _passedTime;

                    DevLog("RESURRECT in {0}", TimeString(_remainingTime));
                    DevLog("     Dead for {0}", TimeString(_passedTime));

                    _logTimer.Reset();
                }
                return false;
            }

            if (_failedReviveCount < 2 && DeltaTime(_timeDied, "s") < Settings.General_ResurrectionTimer)
                return false;

            Log("Resurrecting at graveyard");
            while (!Me.IsAlive && !_init)
            {
                DevLog("WHILE: HandleResurrect Start");

                while (_spirithealer == null || Me.Location.Distance(_spirithealer.Location) > 50)
                {
                    DevLog("WHILE: Resurrect PortBack");

                    await ClickButton("GhostFrame");
                    await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld && _spirithealer != null);
                    await Coroutine.Yield();
                }

                if (!_spirithealer.WithinInteractRange)
                    await MoveTo(_spirithealer.Location);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _spirithealer.Interact();
                    await Coroutine.Wait(5000, () => IsFrameVisible("StaticPopup1"));
                    while (IsFrameVisible("StaticPopup1") && !_init)
                    {
                        DevLog("WHILE: Resurrect Click");
                        await ClickButton("StaticPopup1Button1");
                        await Coroutine.Yield();
                    }
                }
                await Coroutine.Yield();
            }

            _failedReviveCount = 0;
            return true;
        }

        private bool StringContains(object _text, string _operator = "OR", params object[] _strings)
        {
            var _count = 0;
            foreach(var _string in _strings)
            {
                if(_text.ToString().Contains(_string.ToString()))
                {
                    _count++;
                    if (_operator == "OR")
                        return true;
                }
            }

            return (_count == _strings.Count())
                            ? true
                            : false;
        }

        private async Task<bool> HandleFrameBug()
        {
            RuntimeLog(">");

            if (!IsFrameVisible("QuestFrame") && !IsFrameVisible("GossipFrame") && !IsFrameVisible("TaxiFrame"))
            {
                RuntimeLog(">");

                _lastFrameText = null;
                _timeFrameChange = DateTime.Now;
                return false;
            }

            if(IsFrameVisible("TaxiFrame") && StringContains(BotPoi.Current.Type, "OR", "QuestTurnIn", "QuestPickUp") 
               && DeltaTime(_timeDialogOpened, "s") > 5)
            {
                TaxiFrame.Instance.Hide();
                return false;
            }

            var _frameText = (IsFrameVisible("QuestFrame"))
                                    ? Lua.GetReturnVal<string>("return GetQuestText()", 0)
                                    : GossipFrame.Instance.GossipText;
            if (_frameText != _lastFrameText)
            {
                _lastFrameText = _frameText;
                _timeFrameChange = DateTime.Now;
            }

            if (DeltaTime(_timeFrameChange, "s") < 8)
                return false;

            if (IsFrameVisible("QuestFrame"))
            {
                Log("Closing QuestFrame");
                QuestFrame.Instance.Hide();
                await Coroutine.Wait(2000, () => !IsFrameVisible("QuestFrame"));
            }
            else
            {
                Log("Closing GossipFrame");
                GossipFrame.Instance.Hide();
                await Coroutine.Wait(2000, () => !IsFrameVisible("GossipFrame"));
            }

            return true;
        }

        private async Task<bool> HandlePreventHovering()
        {
            if (!Settings.Quest_PreventHovering)
                return false;

            if (Me.IsSwimming)
                return false;

            if (!Flightor.CanFly)
                return false;

            if (!Me.Mounted)
                return false;

            if (!Me.IsMoving)
                return false;

            if (!WoWMovement.ClickToMoveInfo.IsClickMoving)
                return false;

            if (!Mount.Current.IsFlyingMount)
                return false;

            if (Me.HasAura("Stag Form"))
                return false;

            if (Me.IsIndoors)
                return false;

            var _target = Me.CurrentTarget;
            if (_target != null && Me.Location.Distance(_target.Location) < 10)
                return false;

            if (Me.IsFlying || _timeStartFlyingMountRun == DateTime.MinValue)
            {
                _timeStartFlyingMountRun = DateTime.Now;
                return false;
            }

            if (DeltaTime(_timeStartFlyingMountRun, "ms") < 1500)
                return false;

            if (DeltaTime(_timeGettingOff, "ms") > 3000)
            {
                Log("Getting off ground");
                var _endLocation = ExtractVector3(TreeRoot.GoalText);
                var _CTMLocation = (_endLocation != Vector3.Zero)
                                            ? _endLocation
                                            : WoWMovement.ClickToMoveInfo.ClickPos;

                var _distance = (int)Math.Round(Me.Location.Distance(_CTMLocation));
                var _interval = Math.Max(100, Math.Min(2000, _distance * 6));

                await Move("JF", _interval);
                Navigator.Clear();
                BotPoi.Current.NavType = NavType.Fly;
                ProfileManager.CurrentProfile.NavType = NavType.Fly;
                Flightor.MoveTo(_CTMLocation);
            }
            else
            {
                Log("Switching to ground mount");
                await CommonCoroutines.LandAndDismount(null, true);
                await CommonCoroutines.SummonGroundMount();    
            }


            _timeGettingOff = DateTime.Now;
            return true;
        }

        private async Task<bool> TakeTaxi(Vector3 _location)
        {
            RuntimeLog(">>>>");

            if (Flightor.CanFly)
                return false;

            var _knownNodes = KnownFlightNodesManager.EnumerateKnownFlightNodes()
                                        .Where(n => n.MapId == Me.MapId);
            if (_knownNodes.Count() == 0)
                return false;

            RuntimeLog(">>>>");

            var _closestNodeToMe = _knownNodes.OrderBy(n => n.Location.Distance(Me.Location))
                                              .FirstOrDefault();
            var _closestNodeToLocation = _knownNodes.OrderBy(n => n.Location.Distance(_location))
                                                    .FirstOrDefault();

            if (_closestNodeToMe == _closestNodeToLocation)
                return false;

            RuntimeLog(">>>>");

            var _taxiPathDistance = _closestNodeToMe.Location.Distance(Me.Location) +
                                    _closestNodeToMe.Location.Distance(_closestNodeToLocation.Location) / 1.8 +
                                    _closestNodeToLocation.Location.Distance(_location);
            var _directPathDistance = Me.Location.Distance(_location);

            if (_taxiPathDistance >= _directPathDistance)
                return false;

            RuntimeLog(">>>>");

            Log("Taking taxi");
            Log("     < {0} \t\t\t({1})", _closestNodeToMe.Name, 
                                    _closestNodeToMe.Location);
            Log("     > {0} \t\t\t({1})", _closestNodeToLocation.Name,
                                    _closestNodeToLocation.Location);
            var _goTo = _closestNodeToMe.Location;

            while (!Me.OnTaxi && !breakOut)
            {
                DevLog("WHILE: TakeTaxi");

                RuntimeLog(">>>>");

                if (_flightmaster != null)
                    _goTo = _flightmaster.Location;
                if (_flightmaster == null || !_flightmaster.WithinInteractRange)
                    await MoveTo(_goTo);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _flightmaster.Interact();

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _taxiEntry = GossipFrame.Instance.GossipOptionEntries
                                                    .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Taxi);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_taxiEntry.Index);
                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"));
                    }

                    if (IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"))
                    {
                        var _mapNodes = WoWTaxi.GetTaxiMapNodes().ToList();
                        var _flyToNode = _mapNodes.FirstOrDefault(n => n.NodeId == _closestNodeToLocation.Id);
                        await Sleep();
                        _flyToNode.TakeFlight();
                        await Coroutine.Wait(3 * 1000, () => Me.OnTaxi);
                    }
                }

                RuntimeLog(">>>>");

                await Coroutine.Yield();
            }
            DevLog("TakeTaxi > OnTaxi");
            await Coroutine.Wait(1800000, () => !Me.OnTaxi || !_init);
            DevLog("TakeTaxi > OffTaxi");

            return true;
        }

        private double DistanceToLine(Vector3 _location, Vector3 _lineStart, Vector3 _lineEnd)
        {
            var _u = _lineStart - _lineEnd;
            var _a = _lineStart;
            var _p = _location;

            return Vector3.Cross(Vector3.Subtract(_p, _a), _u).Length() / _u.Length();
        }

        private double DistanceToPath(Vector3 _location, params Vector3[] _path)
        {
            double _closestDistance = _location.Distance(_path[0]);

            for (int i = 1; i <= _path.Count() - 1; i++)
            {
                var _start = _path[i - 1];
                var _end = _path[i];
                var _length = _start.Distance(_end);

                var _toStart = _start.Distance(Me.Location);
                var _toEnd = _end.Distance(Me.Location);
                var _toLine = DistanceToLine(_location, _start, _end);

                var _distance = (_toStart + _toEnd < 2 * _length)
                                        ? _toLine
                                        : Math.Min(_toStart, _toEnd);
                if (_distance < _closestDistance)
                    _closestDistance = _distance;
            }

            return _closestDistance;
        }

        private List<object[]> _navigationBlackspotsData = new List<object[]>
        {
            // EASTERN KINGDOMS            
                // EBON HOLD: FLY WAY OUT
                    new object[] {0,    "EbonHold_FlyWayOut",               35, new Vector3(2416.43f, -5623.803f, 461.1342f) },
                // SWAMP OF SORROWS: TOWER
                    new object[] {0,    "SwampOfSorrows_Tower",              2, new Vector3(-10406.78f, -2614.18f, 21.88813f) },

            // KALIMDOR
                // ORGRIMMAR: CLEFT OF SHADOWS
                    new object[] {1,    "Orgrimmar_CleftOfShadows",         20,  new Vector3(1806.51f, -4335.822f, 42.4572f) },
                // ULDUM: CHAMBER OF THE SUN
                    new object[] {1,    "Uldum_ChamberOfTheSun",            60,  new Vector3(-10427.58f, -304.6555f, 304.4986f) },

            // OUTLAND            
                // SHATTRATH : TERACE OF LIGHT
                    new object[] {530, "Shattrath_TeraceOfLight",           20, new Vector3(-1864.536f, 5431.456f, 13.34797f)  },
                // HELLFIRE PENINSULA: ELITES AT PORTAL
                    new object[] {530, "HellfirePeninsula_ElitesAtPortal",  30, new Vector3(-250.8762f, 1090.677f, 47.98906f) },

            // NORTHREND
                // GRIZZLY HILLS: GRIZZLEMAW
                    new object[] {571, "GrizzlyHills_Grizzlemaw",           60, new Vector3(4019.972f, -3690.464f, 351.2816f) },
                // STORM PEAK: MINE
                    new object[] {571, "StormPeak_Mine",                    25, new Vector3(6902.337f, -1155.754f, 832.1863f) },

            // PANDARIA
                // VALLEY OF FOUR WINDS: CLIFF
                    new object[] {870, "ValleyOfFourWinds_Cliff",           30, new Vector3(-684.1757f, -309.7902f, 116.1328f) ,
                                                                                new Vector3(-686.4291f, -252.9519f, 132.2081f) ,
                                                                                new Vector3(-672.5904f, -301.6006f, 116.7788f) ,
                                                                                new Vector3(-721.488f, -110.343f, 102.8925f) ,
                                                                                new Vector3(-717.047f, 40.55526f, 87.42505f) ,
                                                                                new Vector3(-905.7255f, 243.4491f, 82.85305f) ,
                                                                                new Vector3(-950.6279f, 971.4322f, 79.7547f) ,
                                                                                new Vector3(-1008.135f, 1185.015f, 69.11998f) ,
                                                                                new Vector3(-1144.735f, 1497.213f, 59.05566f) ,
                                                                                new Vector3(-1092.256f, 1731.296f, 73.07876f) ,
                                                                                new Vector3(-1136.38f, 1954.926f, 78.24606f) }
        };

        private async Task<bool> FollowPath(string _logDetail = null, int _pathTolerance = 10, bool _preventBlackspot = true, params Vector3[] _wayPath)
        {
            if (_logDetail != null)
                DevLog(_logDetail);

            foreach (var _moveTo in _wayPath)
            {
                while (Me.Location.Distance(_moveTo) > _pathTolerance && !breakOut)
                {
                    DevLog("WHILE: FollowPath{0}", (_logDetail != null)
                                                    ? $" > {_logDetail}"
                                                    : "");

                    await MoveTo(_moveTo, _preventBlackspot, _preventBlackspot);
                    await Coroutine.Yield();
                }
            }

            return true;
        }

        private async Task<bool> HandleNavigationBlackspots()
        {
            var _navigationBlackspot = _navigationBlackspotsData
                                            .FirstOrDefault(b => (int)(b[0]) == Me.MapId &&
                                                                 DistanceToPath(Me.Location, b.Skip(3).Take(b.Count() - 3).Cast<Vector3>().ToArray()) < (int)b[2]);
            if (_navigationBlackspot == null)
                return false;

            Vector3 _flyTo;
            var _task = (string)_navigationBlackspot[1];

            Log("Avoiding blackspot by taking alternative route");
            switch (_task)
            {
                case "Uldum_ChamberOfTheSun":
                    await FollowPath(_task, 10, false,
                                     new Vector3(-10250.65f, -331.821f, 271.3362f),
                                     new Vector3(-10165.33f, -311.9824f, 328.9292f));
                    break;

                case "Shattrath_TeraceOfLight":
                    await FollowPath(_task, 10, false,
                                    new Vector3(-1850.099f, 5402.949f, -2.474063f),
                                    new Vector3(-1828.878f, 5359.747f, -5.543963f),
                                    new Vector3(-1816.314f, 5312.777f, 16.35586f));
                    break;

                case "StormPeak_Mine":
                    await FollowPath(_task, 10, false,
                                     new Vector3(6896.477f, -1123.058f, 823.0789f),
                                     new Vector3(6898.772f, -1123.095f, 871.7703f));
                    break;


                case "Orgrimmar_CleftOfShadows":
                    await FollowPath(_task, 10, false,
                                     new Vector3(1839.717f, -4409.466f, 14.28226f),
                                     new Vector3(1888.178f, -4423.875f, 21.56979f),
                                     new Vector3(1894.588f, -4513.915f, 44.03855f));
                    break;


                case "HellfirePeninsula_ElitesAtPortal":
                    _flyTo = ExtractVector3(TreeRoot.GoalText);
                    _flyTo = (_flyTo == Vector3.Zero)
                                        ? (Me.IsHorde)
                                                ? new Vector3(225.7677f, 2634.509f, 87.71759f)
                                                : new Vector3(-669.2073f, 2711.317f, 94.43723f)
                                        : _flyTo;

                    await FollowPath(_task, 10, false, _flyTo);
                    break;

                case "GrizzlyHills_Grizzlemaw":
                    await FollowPath(_task, 10, false, new Vector3(4131.344f, -3686.705f, 350.3196f));
                    break;

                case "SwampOfSorrows_Tower":
                    await FollowPath(_task, 2, false, new Vector3(-10400.3f, -2610.552f, 22.6615f),
                                                      new Vector3(-10410.17f, -2610.601f, 25.42214f),
                                                      new Vector3(-10408.05f, -2620.31f, 29.35211f));
                    break;

                case "ValleyOfFourWinds_Cliff":
                    await FollowPath(_task, 10, false, WoWMathHelper.CalculatePointInFront(Me.Location, Me.Rotation, 100) +
                                                       new Vector3(0, 0, 100f));
                    break;

                case "EbonHold_FlyWayOut":
                    await FollowPath(_task, 10, false, new Vector3(2339.165f, -5676.859f, 447.7418f));
                    break;
            }

            return true;
        }

        private async Task<bool> HandleInit()
        {
            var _plugin = PluginManager.Plugins.FirstOrDefault(p => p.Name == Name && p.Enabled);
            if (_plugin == null)
                return false;

            _init = true;
            return true;
        }

        async Task<bool> UseTransport(Vector3 _wait,
                                      Vector3 _stand,
                                      Vector3 _exit,
                                      int _transportId,
                                      Vector3 _transportStart,
                                      Vector3 _transportEnd)
        {
            while (Me.Location.Distance(_wait) > 2 && !breakOut)
            {
                DevLog("WHILE: UseTransport > GoToWait");
                await MoveTo(_wait, true);
                await Coroutine.Yield();
            }
            await CommonCoroutines.LandAndDismount(null, true);

            DevLog("UseTransport > WaitForTransportStart");
            Func<bool> _atStart = () =>
            {
                if (breakOut)
                    return true;

                var _transport = ObjectManager.GetObjectsOfType<WoWGameObject>().FirstOrDefault(o => o.Entry == _transportId);
                return _transport != null && _transport.Location.Distance(_transportStart) < 2;
            };
            await Coroutine.Wait(10 * 60 * 1000, _atStart);

            while (Me.Location.Distance(_stand) > 2 && !breakOut)
            {
                DevLog("WHILE: UseTransport > GoToStand");
                if (!await MoveTo(_stand))
                    WoWMovement.ClickToMove(_stand);
                await Coroutine.Yield();
            }

            DevLog("UseTransport > WaitForTransportEnd");
            Func<bool> _atEnd = () =>
            {
                if (breakOut)
                    return true;

                var _transport = ObjectManager.GetObjectsOfType<WoWGameObject>().FirstOrDefault(o => o.Entry == _transportId);
                return _transport != null && _transport.Location.Distance(_transportEnd) < 2;
            };
            await Coroutine.Wait(10 * 60 * 1000, _atEnd);

            while (Me.Location.Distance(_exit) > 2 && !breakOut)
            {
                DevLog("WHILE: UseTransport > GoToExit");
                if (!await MoveTo(_exit))
                    WoWMovement.ClickToMove(_exit);
                await Coroutine.Yield();
            }

            return true;
        }

        private List<object[]> _transportBugData = new List<object[]>
        {
            // STORMWIND: SHIP TO NORTHREND
                new object[] {  0, "Stormwind_ShipToNorthrend",     new Vector3(-7747.651f, 1858.269f, 9.289278f) },
            // ORGRIMMAR: ZEPPELIN TO NORTHREND
                new object[] {  1, "Orgrimmar_ZeppelinToNorthrend", new Vector3(1992.438f, -4129.078f, 227.6518f) },
        };

        private async Task<bool> HandleTransportBug()
        {
            var _transportBugLocation = _transportBugData
                                            .FirstOrDefault(l => (int)(l[0]) == Me.MapId &&
                                                                 Me.Location.Distance((Vector3)(l[2])) < 40);

            if (_transportBugLocation == null || _timeAtTransportBugLocation == DateTime.MinValue)
            {
                _timeAtTransportBugLocation = DateTime.Now;
                return false;
            }

            if (DeltaTime(_timeAtTransportBugLocation, "s") < 10)
                return false;

            var _task = (string)_transportBugLocation[1];
            Log("Taking alternative route as transport seems to be bugged");
            switch (_task)
            {
                case "Orgrimmar_ZeppelinToNorthrend":
                    DevLog("Orgrimmar_ZeppelinToNorthrend");

                    if (Flightor.CanFly)
                    {
                        await MountUp();
                        await FollowPath("Orgrimmar_ZeppelinToNorthrend > FlyTo", 10, true,
                                         new Vector3(2017.099f, -4171.22f, 225.1838f));
                    }
                    else
                    {
                        Log("     Waiting for zeppelin to dock again");
                        DevLog("Orgrimmar_ZeppelinToNorthrend > WaitToDockAgain");
                        var _getOff = new Vector3(1768.282f, -4293.236f, 133.1137f);

                        await Coroutine.Wait(240000, () => Me.Location.Distance(_getOff) < 15 || breakOut);
                    }

                    Log("     Taking zeppelin to Undercity");
                    DevLog("Orgrimmar_ZeppelinToNorthrend > OrgrimmarToUndercity");
                    await UseTransport(new Vector3(1843.144f, -4391.853f, 135.2331f),
                                        new Vector3(1834.708f, -4385.79f, 135.0204f),
                                        new Vector3(2066.38f, 287.1158f, 97.03137f),
                                        164871,
                                        new Vector3(1833.509f, -4391.543f, 152.7679f),
                                        new Vector3(2062.376f, 292.998f, 114.973f));

                    Log("     Taking zeppelin to Northrend");
                    DevLog("Orgrimmar_ZeppelinToNorthrend > UndercityToNorthrend");
                    await UseTransport(new Vector3(2061.519f, 355.6173f, 82.44997f),
                                        new Vector3(2059.365f, 371.6779f, 82.5022f),
                                        new Vector3(1975.594f, -6097.703f, 67.15379f),
                                        181689,
                                        new Vector3(2056.493f, 381.6347f, 100.362f),
                                        new Vector3(1989.353f, -6082.776f, 85.59646f));
                    break;
                case "Stormwind_ShipToNorthrend":
                    DevLog("Stormwind_ShipToNorthrend");

                    if (!Flightor.CanFly)
                    {
                        Log("     Taking tram to Ironforge");
                        DevLog("Stormwind_ShipToNorthrend > EnterTram");
                        var _tramEntrance = new Vector3(-8354.604f, 511.9481f, 84.4087f);
                        while (StyxWoW.IsInWorld && !breakOut)
                        {
                            await MoveTo(_tramEntrance);
                            await Coroutine.Yield();
                        }
                        await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);

                        DevLog("Stormwind_ShipToNorthrend > UseTram");
                        await UseTransport(new Vector3(14.45237f, 2470.982f, -4.294516f),
                                           new Vector3(5.2006f, 2469.381f, -4.484428f),
                                           new Vector3(12.54623f, -10.97226f, -3.989917f),
                                           218208,
                                           new Vector3(4.498615f, 2470.302f, 5.983582f),
                                           new Vector3(4.498832f, -11.34751f, 7.011068f));

                        DevLog("Stormwind_ShipToNorthrend > ExitTram");
                        var _tramExit = new Vector3(72.52491f, 10.46509f, -4.29681f);
                        while (StyxWoW.IsInWorld && !breakOut)
                        {
                            await MoveTo(_tramExit);
                            await Coroutine.Yield();
                        }
                        await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);

                        await FollowPath("Stormwind_ShipToNorthrend > ExitTunnel", 2, true,
                                         new Vector3(-4833.128f, -1283.088f, 501.8689f));
                    }

                    Log("     Taking ship from Menethil to Northrend");
                    DevLog("Stormwind_ShipToNorthrend > MenethilToNorthrend");
                    await UseTransport(new Vector3(-3685.863f, -600.9024f, 4.384491f),
                                       new Vector3(-3675.086f, -593.4379f, 9.47117f),
                                       new Vector3(592.4054f, -5102.106f, 5.260393f),
                                       181688,
                                       new Vector3(-3674.43f, -586.774f, 0.0499605f),
                                       new Vector3(584.0138f, -5118.855f, 0.002798604f));
                    break;
            }

            ReloadProfile();
            Reset();

            return true;
        }

        private async Task<bool> HandleMountUp()
        {
            RuntimeLog(">");

            if (Me.Class != WoWClass.Druid && Me.Combat)
                return false;

            RuntimeLog(">");

            if (Settings.General_MountUpDistance == 0)
                return false;

            RuntimeLog(">");

            if (Me.Mounted)
                return false;

            RuntimeLog(">");

            if (!WoWMovement.ClickToMoveInfo.IsClickMoving)
                return false;

            RuntimeLog(">");

            if (Me.Location.Distance(WoWMovement.ClickToMoveInfo.ClickPos) < Settings.General_MountUpDistance)
                return false;

            RuntimeLog(">");

            if (await MountUp())
                DevLog("HandleMountUp");

            RuntimeLog(">");

            return true;
        }

        private async Task<bool> HandleAutoLoot()
        {
            if (_autoLoot == 1)
                return false;

            Lua.DoString("SetCVar('AutoLootDefault', 1)");
            _autoLoot = 1;

            return true;
        }

        private async Task<bool> HandleDuelInvite()
        {
            if (!Settings.Dialog_DeclineDuelInvites)
                return false;

            if (DialogState != Dialog.Duel)
                return false;

            Log("Declining duel invite");
            await ClickButton("StaticPopup1Button2", 3000);

            return true;
        }

        private async Task<bool> HandlePartyInvite()
        {
            if (!Settings.Dialog_DeclinePartyInvites)
                return false;

            if (DialogState != Dialog.Party)
                return false;

            Log("Declining party invite");
            await ClickButton("StaticPopup1Button2", 3000);

            return true;
        }

        private async Task<bool> HandleGuildInvite()
        {
            var _setting = Settings.Dialog_DeclineGuildInvites;
            if (_declineGuildInvite == _setting)
                return false;

            _declineGuildInvite = _setting;
            var _luaString = string.Format("SetAutoDeclineGuildInvites({0})", (_setting)
                                                                                    ? "true"
                                                                                    : "false");
            Lua.DoString(_luaString);

            return true;
        }

        private async Task<bool> HandleDialog()
        {
            if (!_declineGuildInvite &&
                !Settings.Dialog_DeclineGuildInvites &&
                !Settings.Dialog_DeclinePartyInvites &&
                !Settings.Dialog_DeclineDuelInvites)
                return false;

            await HandleGuildInvite();

            if (DialogState == Dialog.None)
                return false;

            if (!IsFrameVisible("StaticPopup1"))
            {
                DialogState = Dialog.None;
                return false;
            }

            await HandlePartyInvite();
            await HandleDuelInvite();

            return true;
        }

        private async Task<bool> HandleDig()
        {
            if (!Me.IsAlive)
                return false;

            if (Me.Combat)
                return false;

            if (BotManager.Current.ToString() != "ArchaeologyBuddy")
                return false;

            var _auraId = 210837;
            if (!Me.HasAura(_auraId))
                return false;

            Log("Digging");
            while (Me.HasAura(_auraId) && !breakOut)
            {
                DevLog("WHILE: HandleDig");

                //await CommonCoroutines.LandAndDismount(null, true, Me.Location, 5);
                await CommonCoroutines.Dismount(null, true);
                await CommonCoroutines.StopMoving();

                if (!Me.IsCasting)
                {
                    await Coroutine.Wait(4000, () => SpellManager.CanCast(80451));
                    SpellManager.CastSpellById(80451);
                }

                await Coroutine.Wait(2000, () => !Me.HasAura(_auraId));
            }

            _checkedArchaeologySkill = false;
            return true;
        }

        private async Task<bool> TakePortal(int _portalId, Vector3 _portalLocation)
        {
            var _portal = GetObject(_portalId);
            if (_portal != null)
                _portalLocation = _portal.Location;

            if (_portal == null || !_portal.WithinInteractRange)
                await MoveTo(_portalLocation, false, true);
            else
            {
                await CommonCoroutines.StopMoving();
                await Sleep();
                _portal.Interact();

                await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);
            }

            return true;
        }

        private List<int> _continentsSkillData = new List<int>
        {
            525,    // Pandaria
		    600,	// Draenor
		    700    // BrokenIsles
	    };

        private async Task<bool> HandleSwitchContinents()
        {
            if (Settings.Archaeology_GoToPandaria && ((Me.IsHorde)
                                                ? Me.MapId == 0
                                                : Me.MapId == 1))
            {
                Log("Char needs to be placed in {0} in order to switch to Pandaria.",
                                        (Me.IsHorde)
                                              ? "Kalimdor"
                                              : "Eastern Kingdoms");
                Settings.General_InfiniteMode = false;
                Log("     STOPPING bot...");
                TreeRoot.Stop("KeepOnMoving");
            }

            var _continentEntry = _continentsSkillData.LastOrDefault(c => GetCondition(((Hearth)(_continentsSkillData.IndexOf(c) + 3)).ToString(), true) &&
                                                                          Me.GetSkill(SkillLine.Archaeology).CurrentValue >= c);
            if (_continentEntry == 0)
                return false;

            /*
            if (_continentEntry == 525 &&
                (!Settings.Hearth_Hearthstone ||
                 !Settings.Hearth_SetHearthstone ||
                 UnfavourableAreaSet()))
                return false;
            */

            var _index = _continentsSkillData.IndexOf(_continentEntry) + 3;
            if (Me.CurrentMap.ExpansionId == _index + 1)
                return false;

            if (_index >= 4)        // Draenor + BrokenIsles
            {
                var _continent = ((Hearth)_index).ToString();

                DevLog("Hearth.{0}: Archaeology", _continent);
                Log("Continuing archaeology on next continent");
                HearthState = (Hearth)_index;

                await HandleSellTrash(true);
                return false;
            }

            var _portalId = (Me.IsHorde)
                                ? 215424
                                : 215457;
            var _portalLocation = (Me.IsHorde)
                                        ? new Vector3(2014.819f, -4700.274f, 28.62439f)
                                        : new Vector3(-8197.333f, 528.3328f, 117.16f);
            var _portal = GetObject(_portalId);

            Log("Taking portal to Pandaria");
            while (Me.CurrentMap.ExpansionId != 4 && !breakOut)
            {
                DevLog("WHILE: HandleSwitchContinents");

                await TakePortal(_portalId, _portalLocation);
                await Coroutine.Yield();
            }
            await WaitForMapChange();
            return true;
        }

        private async Task<bool> WaitForMapChange()
        {
            DevLog("Waiting for map change...");
            if (await Coroutine.Wait(10 * 1000, () => DeltaTime(_timeMapChanged, "ms") < 5000))
            {
                DevLog("Map changed");

                // Reset Digsites
                if (BotManager.Current.ToString() == "ArchaeologyBuddy")
                {
                    DevLog("Restart ArchaeologyBuddy");
                    ArchBuddy.Instance.Stop();
                    ArchBuddy.Instance.Start();
                }

                return true;
            }

            return false;
        }

        private async Task<bool> TrainSkill(SkillLine _skill, int _npcId, Vector3 _npcLocation)
        {
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = ((Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism) /*||
                            Me.Race == WoWRace.Worgen && _skill == SkillLine.Skinning   */)
                                    ? _skillValue + 15
                                    : _skillValue;

            if(_skill != SkillLine.Riding)
                Log("Training {0}", _skill);
            DevLog("     CurrentValue: {0}", _skillValue);
            DevLog("     NpcId: {0}", _npcId);

            var _preMax = Me.GetSkill(_skill).MaxValue;
            var _npc = GetNpc(_npcId);
            while (Me.GetSkill(_skill).MaxValue == _preMax && !breakOut)
            {
                DevLog("WHILE: TrainSkill{0}", _skill);

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;

                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation, true, true);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();

                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("ClassTrainerFrame") || IsFrameVisible("GossipFrame"));
                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _trainerEntry = GossipFrame.Instance.GossipOptionEntries
                                                .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Trainer);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_trainerEntry.Index);
                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("ClassTrainerFrame"));
                    }

                    TrainerFrame.Instance.SetServiceFilter(TrainerServiceFilter.Available, true);
                    TrainerFrame.Instance.SetServiceFilter(TrainerServiceFilter.Unavailable, false);
                    TrainerFrame.Instance.SetServiceFilter(TrainerServiceFilter.Used, false);

                    while (IsFrameVisible("ClassTrainerScrollFrameButton1") && !breakOut)
                    {

                        await ClickButton("ClassTrainerFrameSkillStepButton");
                        await ClickButton("ClassTrainerTrainButton");

                        if (IsFrameVisible("StaticPopup1"))
                            await ClickButton("StaticPopup1Button1");

                        if (_skill == SkillLine.Riding)
                            break;

                        await Coroutine.Yield();
                    }

                    await Coroutine.Wait(3 * 1000, () => Me.GetSkill(_skill).MaxValue != _preMax);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleTrainArchaeology()
        {
            if (!Settings.Archaeology_Train)
                return false;

            /*
            if (Me.GetSkill(SkillLine.Archaeology).CurrentValue == 0)
                return false;
                */

            if (Me.GetSkill(SkillLine.Archaeology).MaxValue >= 700)
                return false;

            RuntimeLog(">>");

            /*
            if (!SkillNeedsTraining(SkillLine.Archaeology, true) && false)
            {
                if (HearthState == Hearth.Off)
                    _checkedArchaeologySkill = true;
                return false;
            }

            if (!_inCapital)
            {
                if (Settings.Hearth_Hearthstone &&
                    !Settings.Hearth_SetHearthstone &&
                    UnfavourableAreaSet())
                    return false;

                DevLog("Hearth.Capital: TrainArchaeologySkill");
                Log("Preparing for training archaeology skill");
                HearthState = Hearth.Capital;
                return false;

            }

            var _skill = SkillLine.Archaeology;
            var _trainerDataEntry = _trainerData.FirstOrDefault(e => (int)(e[0]) == Me.ZoneId &&
                                                                     (SkillLine)(e[1]) == SkillLine.Archaeology);
            var _npcId = (int)_trainerDataEntry[2];
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            */

            var _skill = SkillLine.Archaeology;
            var _trainerDataEntry = GetClosestTrainerEntry(_skill);
            if (_trainerDataEntry == null)
                return false;

            var _npcId = (int)_trainerDataEntry[0];
            var _npcLocation = (Vector3)_trainerDataEntry[4];

            await TrainSkill(_skill, _npcId, _npcLocation);

            return true;
        }

        private WoWUnit _vendor
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         u.IsRepairMerchant &&
                         u.WithinInteractRange &&
                         u.IsValid &&
                         u.IsAlive
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private IEnumerable<WoWItem> _trash
        {
            get
            {
                return Me.BagItems.Where(i => i.IsValid &&
                                                i.Quality == WoWItemQuality.Poor);
            }

        }

        private async Task<bool> HandleSellTrash(bool _force = false)
        {
            RuntimeLog(">>");

            if (_trash.Count() == 0)
                return false;

            if (_trash.Count() < 10 && !_force)
                return false;

            RuntimeLog(">>");

            var _vendorMountSpells = new List<int>()
            {
                122708,
                61447,
                61425
            };
            var _vendorMountSpellId = _vendorMountSpells.FirstOrDefault(s => SpellManager.HasSpell(s) &&
                                                                             SpellManager.CanCast(s));
            if (_vendorMountSpellId == 0)
                return false;

            RuntimeLog(">>");

            Log("Selling trash");
            while (!IsFrameVisible("MerchantFrame") && !breakOut)
            {
                DevLog("WHILE: HandleSellTrash");

                if (_vendor == null)
                {
                    await CommonCoroutines.LandAndDismount();
                    await CommonCoroutines.StopMoving();

                    if (!Me.IsCasting)
                        SpellManager.CastSpellById(_vendorMountSpellId);
                    await Coroutine.Wait(5000, () => _vendor != null);

                    await Coroutine.Yield();
                }
                else
                {
                    _vendor.Interact();
                    await Coroutine.Wait(5000, () => IsFrameVisible("MerchantFrame"));
                }

                await Coroutine.Yield();
            }

            if (IsFrameVisible("MerchantFrame"))
            {
                await Vendors.SellItems(_trash);
                if (!await Coroutine.Wait(5000, () => _trash == null))
                {
                    var _item = _trash.FirstOrDefault();
                    while (_item != null && !breakOut)
                    {
                        DevLog("WHILE: HandleSellTrash > Destroy");

                        await Sleep();
                        _item.PickUp();
                        Lua.DoString("DeleteCursorItem();");
                        await Coroutine.Wait(2000, () => !HasItem((int)_item.Entry));

                        _item = _trash.FirstOrDefault();
                        await Coroutine.Yield();
                    }
                }
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleArchaeologyLoot()
        {
            if (!CharacterSettings.Instance.LootMobs)
                return false;

            CharacterSettings.Instance.LootMobs = false;

            return true;
        }

        private WoWUnit _ancientHaunt
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         u.Entry == 62596 &&
                         (
                            u.IsAlive ||
                            (
                                u.CanLoot &&
                                u.Lootable &&
                                !u.Looting &&
                                !u.IsAlive
                            )
                         ) &&
                         u.IsValid
                     orderby
                        u.Distance
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private async Task<bool> HandleAncientHaunt()
        {
            if (_ancientHaunt == null)
                return false;

            Log("Looting ancient haunt");
            while (_ancientHaunt != null && !breakOut)
            {
                DevLog("WHILE: HandleAncientHaunt");

                if (!_ancientHaunt.WithinInteractRange)
                    await MoveTo(_ancientHaunt.Location);
                else
                {
                    await CommonCoroutines.StopMoving();

                    _ancientHaunt.Interact();
                    await Loot();
                    await Coroutine.Wait(3000, () => _ancientHaunt == null);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleBuyFragments()
        {
            if ((int)Settings.Archaeology_Fragments == 0)
                return false;

            if (Me.CurrentMap.ExpansionId != 4)
                return false;

            if (!Flightor.CanFly)
                return false;

            if (Me.GetSkill(SkillLine.Archaeology).CurrentValue < 600)
                return false;

            var _restoredArtifactId = 87399;
            var _restoredArtifact = GetItem(_restoredArtifactId);
            if (_restoredArtifact == null)
                return false;

            var _npcId = 64922;
            var _npc = GetNpc(_npcId);
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            if (Me.Location.Distance(_npcLocation) > 600 && GetItemNum(_restoredArtifactId) < 10)
                return false;

            var _fragmentContainerId = (int)Settings.Archaeology_Fragments;
            var _fragmentContainer = GetItem(_fragmentContainerId);

            Log("Buying fragments");
            while (_restoredArtifact != null && !breakOut)
            {
                DevLog("WHILE: HandleBuyFragments");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;

                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation);
                else
                {
                    if (!IsFrameVisible("MerchantFrame"))
                    {
                        await CommonCoroutines.StopMoving();
                        await Sleep();
                        _npc.Interact();

                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("MerchantFrame") || IsFrameVisible("GossipFrame"));
                        if (IsFrameVisible("GossipFrame"))
                        {
                            var _vendorEntry = GossipFrame.Instance.GossipOptionEntries
                                                    .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Vendor);
                            await Sleep();
                            GossipFrame.Instance.SelectGossipOption(_vendorEntry.Index);
                            await Coroutine.Wait(3 * 1000, () => IsFrameVisible("MerchantFrame"));
                        }
                    }
                    else
                    {
                        if (_trash.Count() > 0)
                            await Vendors.SellItems(_trash);

                        var _allItems = MerchantFrame.Instance.GetAllMerchantItems().ToList();
                        var _index = _allItems.FirstOrDefault(i => i.ItemId == _fragmentContainerId).Index;
                        MerchantFrame.Instance.BuyItem(_index, GetItemNum(_restoredArtifactId));

                        await Coroutine.Wait(3000, () => GetItem(_restoredArtifactId) == null);
                        _restoredArtifact = GetItem(_restoredArtifactId);
                    }
                }
                _checkedItems = false;
                await Coroutine.Yield();
            }

            return true;
        }

        private List<int> _races = new List<int>
        {
            1174,   // 1 - Demonic
            1173,   // 2 - HighmountainTauren
            1172,   // 3 - Highborne
            828,    // 4 - Ogre
            821,    // 5 - DraenorClans
            829,    // 6 - Arakkoa
            677,    // 7 - Mogu
            676,    // 8 - Pandaren
            754,    // 9 - Mantid
            399,    // 10 - Vyrkul
            385,    // 11 - Troll
            401,    // 12 - Tol'vir
            397,    // 13 - Orc
            400,    // 14 - Nerubian
            394,    // 15 - NightElf
            393,    // 16 - Fossil
            398,    // 17 - Draenei
            384     // 18 - Dwarf
        };

        private int GetRaceIndex(WoWCurrency _currency)
        {
            return _races.IndexOf((int)_currency.CurrencyType) + 1;
        }

        private bool CurrencyIsCritical(WoWCurrency _currency)
        {
            return _currency.CurrencyType.ToString().Contains("ArchaeologyFragment") &&
                   _currency.Amount >= 190;
        }

        private async Task<bool> HandleSolveArtifact()
        {
            var _currency = WoWCurrency.GetDiscoveredCurrencies()
                                       .FirstOrDefault(c => CurrencyIsCritical(c));
            if (_currency == null)
                return false;

            Log("Solving artifact");
            WoWCurrency _lastCurrency = null;
            while (_currency != null && !breakOut)
            {
                DevLog("WHILE: HandleSolveArtifact");

                if (_currency != _lastCurrency)
                {
                    Log("     {0} (Id: {1})", _currency.Name, _currency.Entry);
                    _lastCurrency = _currency;
                }

                await Sleep();
                Lua.DoString($"SolveArtifact({GetRaceIndex(_currency)})");

                var _preAmount = _currency.Amount;
                if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                    if (await Coroutine.Wait(5 * 1000, () => !Me.IsCasting))
                        await Coroutine.Wait(2 * 1000, () => _preAmount != _currency.Amount);

                _currency = WoWCurrency.GetDiscoveredCurrencies()
                                       .FirstOrDefault(c => CurrencyIsCritical(c));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleArchaeology()
        {
            if (BotManager.Current.ToString() != "ArchaeologyBuddy")
                return false;

            RuntimeLog(">");

            if (await HandleBuyFragments())
                return true;

            RuntimeLog(">");

            if (await HandleSellTrash())
                return true;

            RuntimeLog(">");

            if (await HandleAncientHaunt())
                return true;

            RuntimeLog(">");

            await HandleArchaeologyLoot();

            RuntimeLog(">");

            if (_checkedArchaeologySkill)
                return false;

            RuntimeLog(">");

            if (await HandleTrainArchaeology())
                return true;

            RuntimeLog(">");

            if (await HandleSwitchContinents())
                return true;

            RuntimeLog(">");

            return false;
        }

        private string TimeString(double _seconds)
        {
            _seconds = Math.Round(_seconds);

            if (_seconds < 60)
                return $"{_seconds} seconds";
            else
            {
                var _minutes = Math.Floor(_seconds / 60);
                _seconds = _seconds % 60;

                return String.Format("{0}:{1} minutes", _minutes, (_seconds < 10)
                                                                        ? $"0{_seconds}"
                                                                        : $"{_seconds}");
            }
        }

        private async Task<bool> HandleObjectiveChange()
        {
            if (Settings.Motionless_ObjectiveTimer == 0)
                return false;

            if (_lastObjective != TreeRoot.GoalText || _timeLastObjectiveChange == DateTime.MinValue)
            {
                _timeLastMoved = DateTime.Now;
                _lastObjective = TreeRoot.GoalText;
                return false;
            }

            if (DeltaTime(_timeLastObjectiveChange, "s") < Settings.Motionless_ObjectiveTimer)
                return false;

            DevLog("ObjectiveChange");
            return true;
        }

        private async Task<bool> HandleNoMovement()
        {
            RuntimeLog(">>");

            if (Settings.Motionless_Timer == 0)
                return false;

            RuntimeLog(">>");

            if (Me.IsMoving || _timeLastMoved == DateTime.MinValue ||
                Me.OnTaxi || Me.IsOnTransport)                           //+++ || IsFrameVisible("OverrideActionBar"))
            {
                _timeLastMoved = DateTime.Now;
                return false;
            }

            RuntimeLog(">>");

            if (!Me.IsAlive)
                return false;

            RuntimeLog(">>");

            var _beginMotionless = (BotManager.Current.ToString() == "Questing" &&
                                    ProfileManager.XmlLocation != null &&
                                    ProfileManager.XmlLocation.ToLower().Contains("questing profile pack"))
                                                ? 330
                                                : Settings.Motionless_Timer;

            RuntimeLog(">>");

            if (DeltaTime(_timeLastMoved, "s") < _beginMotionless)
            {
                var _passedTime = Math.Floor(DeltaTime(_timeLastMoved, "s"));
                if (_passedTime >= 10 && _logTimer.IsFinished)
                {
                    var _remainingTime = _beginMotionless - _passedTime;

                    DevLog("MOTIONLESS in {0}", TimeString(_remainingTime));
                    DevLog("     No movement for {0}", TimeString(_passedTime));

                    _logTimer.Reset();
                }
                return false;
            }

            DevLog("NoMovement");
            return true;
        }

        private async Task<bool> HandleReloadStage(int _increaseDistanceFactor = 0)
        {
            if (StuckState != Stuck.Stage6 &&
                MotionlessState != Motionless.Stage1)
                return false;

            if (Me.Location.Distance(_lastStuckLocation) >= (_increaseDistanceFactor + 1) * _defaultSpeed)
            {
                Reset();
                return false;
            }

            if (DeltaTime(_timeReload, "s") < 45)
                return false;

            return true;
        }

        private async Task<bool> JumpToUnstuckStage()
        {
            DevLog("Stuck handling disabled: {0}", Settings.Stuck_Disable);
            Log("     Stage 2");

            MotionlessState = Motionless.Stage2;
            StuckState = Stuck.On;

            if (!await DoUnstuck(1, true))
            {
                DevLog("Motionless: Unstuck unsuccessful > Hearth");
                await JumpToHearthStage();
                return false;
            }
            else
            {
                DevLog("Motionless: Unstuck successful > Wait 20 seconds");

                await CommonCoroutines.StopMoving();
                if (!await Coroutine.Wait(20 * 1000, () => Me.IsMoving))
                {
                    DevLog("Motionless: Still not moving > Hearth");
                    await JumpToHearthStage();
                    return false;
                }
            }

            Reset();
            return true;
        }

        private async Task<bool> HandleMotionless()
        {
            RuntimeLog(">");

            if (await HandleReloadStage(1))
            {
                await JumpToUnstuckStage();
                return false;
            }
            else if ((int)MotionlessState >= 2)
                return false;

            RuntimeLog(">");

            if ((int)MotionlessState >= 2)
                return false;

            if (Me.HasAura("Resurrection Sickness") || IsFrameVisible("MerchantFrame") || Me.IsActuallyInCombat)
            {
                _timeLastMoved = DateTime.Now;
                _timeLastObjectiveChange = DateTime.Now;
                return false;
            }

            if (!await HandleNoMovement() &&
                !await HandleObjectiveChange())
                return false;

            RuntimeLog(">");

            Log("MOTIONLESS! Bringing back movement...");
            Log("     Stage 1");

            PrintInfo(false);

            MotionlessState = Motionless.Stage1;
            _lastStuckLocation = Me.Location;
            _timeReload = DateTime.Now;
            ReloadProfile();

            return true;
        }

        private async Task<bool> HandleFeralCombat()
        {
            if (Me.Class != WoWClass.Druid)
                return false;

            if (Me.Specialization != WoWSpec.DruidFeral)
                return false;

            if (Me.Shapeshift == ShapeshiftForm.Cat)
                return false;

            RuntimeLog(">");

            if (Me.Mounted && Mount.Current.IsFlyingMount && !Me.HasAura("Stag Form"))
                return false;

            RuntimeLog(">");

            if (!Me.Combat)
                return false;

            RuntimeLog(">");

            if (!Me.IsActuallyInCombat)
                return false;

            RuntimeLog(">");

            if (!SpellManager.CanCast(768))
                return false;

            RuntimeLog(">");

            Log("Forcing cat form");

            SpellManager.CastSpellById(768);
            await Coroutine.Wait(2000, () => Me.Shapeshift == ShapeshiftForm.Cat);

            return true;
        }

        private async Task<bool> HandleMeleeDismount()
        {
            if (!Me.IsMelee)
                return false;

            if (!_meleeDismount)
                return false;

            if (!Me.Mounted)
                return false;

            if (Me.CurrentTarget == null)
                return false;

            Log("Dismounting near target");

            var _targetLocation = Me.CurrentTarget.Location;
            while (Me.Mounted && !breakOut)
            {
                if (Me.Location.Distance(_targetLocation) > 5)
                    await MoveTo(_targetLocation);
                else
                {
                    await CommonCoroutines.LandAndDismount(null, true, _targetLocation);
                    await CommonCoroutines.StopMoving();

                    var _target = Me.CurrentTarget;
                    if (_target != null)
                        Lua.DoString("StartAttack()");
                }

                await Coroutine.Yield();
            }

            _meleeDismount = false;
            return true;
        }

        private async Task<bool> HandleReloadProfile()
        {
            if (!_reloadProfile)
                return false;

            if (Me.IsActuallyInCombat)
                return false;

            Log("Reloading profile");
            ReloadProfile();
            _reloadProfile = false;

            return true;
        }

        /*
        private bool QuestCanBeTurnedIn(PlayerQuest _quest)
        {
            if (!_quest.IsCompleted)
                return false;

            var _npcId = GetTurnInNpcId((int)_quest.Id);
            if (_npcId == -1)
                return false;

            var _npcResult = Query.GetNpcById((uint)_npcId);
            if (_npcResult == null)
                return false;

            if (_npcResult.MapId != Me.MapId)
                return false;

            return true;
        }        

        private double GetTurnInDistance(PlayerQuest _quest)
        {
            var _npcId = GetTurnInNpcId((int)_quest.Id);
            if (_npcId == -1)
                return -1;

            var _npcResult = Query.GetNpcById((uint)_npcId);
            if (_npcResult == null)
                return -1;

            var _npcLocation = _npcResult.Location;
            if (_npcLocation == Vector3.Zero)
                return -1;

            return Me.Location.Distance(_npcLocation);
        }

        private int GetTurnInQuestId()
        {
            var _quests = QuestLog.Instance.GetAllQuests()
                                    .Where(q => QuestCanBeTurnedIn(q) &&
                                                GetTurnInDistance(q) > -1)
                                    .OrderBy(q => GetTurnInDistance(q));

            if (_quests.Count() == 0)
                return -1;

            return (int)_quests.FirstOrDefault().Id;
        }

        private async Task<bool> HandleCompletedQuests(bool _force = false)
        {
            return false;

            if (!_force)
            {
                if (!_profileChange)
                    return false;

                if (Me.Combat)
                    return false;

                if (!Me.IsMoving)
                    return false;

                var _profile = ProfileManager.XmlLocation;
                if (_lastProfile == null && !_profile.ToLower().Replace("auto loader", "autoloader").Contains("autoloader"))
                    _lastProfile = _profile;

                if (_lastProfile == _profile)
                    return false;
            }

            var _questId = GetTurnInQuestId();
            if (_questId == -1)
                return false;

            while (_questId != -1 && !breakOut)
            {
                DevLog("WHILE: HandleCompletedQuests");

                var _npcId = GetTurnInNpcId(_questId);
                await TurnInQuest(_questId, _npcId);

                _questId = GetTurnInQuestId();
                await Coroutine.Yield();
            }

            _lastProfile = ProfileManager.XmlLocation;
            _profileChange = false;
            return true;
        }
        */

        private bool InVehicle()
        {
            if (_timeEnterVehicle == DateTime.MinValue &&
                _timeExitVehicle == DateTime.MinValue)
            {
                if (Me.InVehicle)
                    _timeEnterVehicle = DateTime.Now;
                else
                    _timeExitVehicle = DateTime.Now;
            }

            return DeltaTime(_timeEnterVehicle, "ms") < DeltaTime(_timeExitVehicle, "ms");
        }

        /*
        private async Task<bool> HandleNavType()
        {
            if (BotPoi.Current.NavType == NavType.Run &&
                ProfileManager.CurrentProfile.NavType == NavType.Run)
                return false;

            BotPoi.Current.NavType = NavType.Run;
            ProfileManager.CurrentProfile.NavType = NavType.Run;

            return true;
        }

        private async Task<bool> HandleLegionFlying()
        {
            await HandleNavType();
            await HandleFlightor();

            return true;
        }

        private async Task<bool> HandleFlightor()
        {
            Flightor.Clear();

            if (!_lastLogEntry.Contains("[Flightor] Set up for movement from"))
                return false;

            if (_intervalLastFlightorLog > 2000)
            {
                _intervalLastFlightorLog = 0;
                Log("Forcing ground navigation");
            }

            await MountUp(null, 0, true);

            var _endLocation = ExtractVector3(_lastLogEntry, 2);
            if (_endLocation == Vector3.Zero)
                return false;

            Navigator.MoveTo(_endLocation);

            return true;
        }
        */

        private Vector3 ExtractVector3(string _string, int _index = 1)
        {
            if (_string == null)
                return Vector3.Zero;

            var _regex = new Regex("(?<=<)[^a-z]+(?=>)");
            var _result = _regex.Matches(_string);
            if (_result.Count == 0)
                return Vector3.Zero;

            _regex = new Regex("[-0-9.]+");
            var _safeIndex = Math.Min(_index, _result.Count) - 1;
            _result = _regex.Matches(_result[_safeIndex].ToString());
            if (_result.Count != 3)
                return Vector3.Zero;

            var _vector = new Vector3(float.Parse(_result[0].ToString()),
                                      float.Parse(_result[1].ToString()),
                                      float.Parse(_result[2].ToString()));
            return _vector;
        }

        private async Task<bool> HandleTravelFormSetting()
        {
            var _setting = (int)Settings.General_TravelForm;
            if (_setting == 0)
                return false;

            if (!SpellManager.HasSpell(783))
                return false;

            var _groundMount = CharacterSettings.Instance.GroundMountSpellId;
            var _flyingMount = CharacterSettings.Instance.FlyingMountSpellId;

            if (_groundMount == 783 && _flyingMount == 783)
                return false;

            CharacterSettings.Instance.GroundMountSpellId = (_setting == 1 || _setting == 3)
                                                                        ? 783
                                                                        : _groundMount;
            CharacterSettings.Instance.FlyingMountSpellId = (_setting == 2 || _setting == 3)
                                                                        ? 783
                                                                        : _flyingMount;

            return true;
        }

        private async Task<bool> KOMloop()
        {
            if (!StyxWoW.IsInWorld)
                return false;

           if(DeltaTime(_timeRun, "s") > 5)
            {
                _timeLastMoved = DateTime.Now;
                _timeLastObjectiveChange = DateTime.Now;
            }

            RuntimeLog("Start");

            if (!await HandleInit())
                return false;

            RuntimeLog("#");

            if (await HandleStuck())
                return true;

            RuntimeLog("#");

            if (await HandleMeleeDismount())
                return true;

            RuntimeLog("#");

            if (await HandleDig())
                return true;

            RuntimeLog("#");

            await HandleJump();

            RuntimeLog("#");

            await HandleLoot();

            RuntimeLog("#");

            if (!await HandleTPS())
            {
                RuntimeLog("Sum");
                return false;
            }

            RuntimeLog();

            if (await HandleMotionless())
                return true;

            RuntimeLog();

            if (await HandleHearth())
                return true;

            RuntimeLog();

            await HandleGraphicMode();

            RuntimeLog();

            await HandleDisableAddons();

            RuntimeLog();

            await HandleKillBetweenHotspots();

            RuntimeLog();

            await HandleAutoLoot();

            RuntimeLog();

            if (await HandleReloadProfile())
                return true;

            RuntimeLog();

            if (await HandleResurrect())
                return true;

            RuntimeLog();

            if (!Me.IsAlive)
            {
                RuntimeLog("Sum");
                return false;
            }

            RuntimeLog();

            await HandleDialog();

            RuntimeLog();

            if (await HandleGatheringProfessions())
                return true;

            RuntimeLog();

            await HandleFrameBug();

            RuntimeLog();

            if (Me.Combat)
            {
                RuntimeLog("Sum");
                return false;
            }

            RuntimeLog();

            if (await HandleTransportBug())
                return true;

            RuntimeLog();

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
            {
                RuntimeLog("Sum");
                return false;
            }

            RuntimeLog();

            await HandleTravelFormSetting();

            RuntimeLog();

            await HandleFeralCombat();

            RuntimeLog();

            /*
            if (await HandleCompletedQuests())
                return true;
            */

            RuntimeLog();

            if (await HandleMountUp())
                return true;

            RuntimeLog();

            if (await HandlePreventHovering())
            {
                _timeStartFlyingMountRun = DateTime.Now;
                return true;
            }

            RuntimeLog();

            if (await HandleNavigationBlackspots())
                return true;

            RuntimeLog();

            if (await HandleStop())
                return true;

            RuntimeLog();

            if (await HandleSpec())
                return true;

            RuntimeLog();

            if (await HandleArchaeology())
                return true;

            RuntimeLog();

            if (await HandleAbandonQuest())
                return true;

            RuntimeLog();

            if (await HandleTaxiNodes())
                return true;

            RuntimeLog();

            if (await HandleNewMails())
                return true;

            RuntimeLog();

            if (await HandleItems())
                return true;

            RuntimeLog();

            if (await HandleGarrisonResources())
                return true;

            RuntimeLog();

            if (await HandleTrainFlying())
                return true;

            RuntimeLog("Sum");

            _checkedQuests = true;
            _checkedItems = true;
            _timeRun = DateTime.Now;

            return false;
        }

        private ulong _flyingGold
        {
            get
            {
                var _factionId = (Me.IsHorde)
                                    ? 76
                                    : 72;

                var _amount = 5000 - ((int)Me.GetReputationLevelWith((uint)_factionId) - 3) * 250;
                return (ulong)_amount;
            }
        }

        private ulong GetFlyingGold(uint _factionId)
        {
            var _base = 5000;
            var _levelAboveNeutral = Math.Max(0, (int)Me.GetReputationLevelWith(_factionId) - 3);
            var _discount = _levelAboveNeutral * 5;
            var _amount = (Me.Race == WoWRace.Goblin) 
                                    ? 4000
                                    : 5000 * (1 - _discount / 100);

            return (ulong)_amount;

        }

        private WoWUnit _flightmaster
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         ((UnitNPCFlags)u.NpcFlags).HasFlag(UnitNPCFlags.Flightmaster) &&
                         u.IsValid &&
                         u.CanInteractNow &&
                         u.IsAlive
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private WoWUnit _spirithealer
        {
            get {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         ((UnitNPCFlags)u.NpcFlags).HasFlag(UnitNPCFlags.Spirithealer) &&
                         u.IsAlive &&
                         u.IsValid
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private void ReloadProfile()
        {
            if (BotManager.Current.ToString() != "Questing")
                return;

            string _profile = ProfileManager.XmlLocation;
            DevLog("Reloading");
            DevLog("     Current Profile: {0}", (_profile == null)
                                                    ? "null"
                                                    : _profile);

            if ((Me.Race == WoWRace.Pandaren)
                        ? (IsQuestCompleted(31012) || IsQuestCompleted(30987))
                        : true)
                _profile = GetAutoLoader();

            DevLog("     New Profile: {0}", _profile);

            try
            {
                ProfileManager.LoadNew(_profile);
            }
            catch (Exception) { }
        }

        public override void OnDisable()
        {
            BotEvents.OnBotStarted -= BotEvents_OnOnBotStarted;
            BotEvents.OnBotStopped -= BotEvents_OnOnBotStopped;
            BotEvents.OnBotPaused -= BotEvents_OnOnBotPaused;
            BotEvents.OnBotResumed -= BotEvents_OnOnBotResumed;
            Logging.OnLogMessage -= Logging_OnLogMessage;
            BotEvents.Profile.OnNewOuterProfileLoaded -= Profile_OnNewOuterProfileLoaded;

            TreeHooks.Instance.RemoveHook("InGame_Check", _KOM);

            Lua.Events.DetachEvent("PLAYER_LEVEL_UP", LuaEvents_PlayerLevelUp);
            Lua.Events.DetachEvent("PLAYER_DEAD", LuaEvents_PlayerDead);
            Lua.Events.DetachEvent("PLAYER_UNGHOST", LuaEvents_PlayerUnghost);

            Lua.Events.DetachEvent("QUEST_WATCH_UPDATE", LuaEvents_QuestWatchUpdateQuestTurnedIn);
            Lua.Events.DetachEvent("QUEST_TURNED_IN", LuaEvents_QuestWatchUpdateQuestTurnedIn);
            Lua.Events.DetachEvent("QUEST_ACCEPTED", LuaEvents_QuestAccepted);

            Lua.Events.DetachEvent("BAG_UPDATE_DELAYED", LuaEvents_BagUpdateDelayed);

            Lua.Events.DetachEvent("DUEL_REQUESTED", LuaEvents_DuelRequested);
            Lua.Events.DetachEvent("PARTY_INVITE_REQUEST", LuaEvents_PartyInviteRequest);

            Lua.Events.DetachEvent("UI_ERROR_MESSAGE", LuaEvents_UiErrorMessage);

            Lua.Events.DetachEvent("UNIT_SPELLCAST_INTERRUPTED", LuaEvents_UnitSpellcastInterrupted);
            Lua.Events.DetachEvent("UNIT_SPELLCAST_SUCCEEDED", LuaEvents_UnitSpellcastSucceeded);

            Lua.Events.DetachEvent("TAXIMAP_OPENED", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("GOSSIP_SHOW", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("MAIL_SHOW", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("TRAINER_SHOW", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("LOOT_OPENED", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("CINEMATIC_START", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("QUEST_DETAIL", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("QUEST_PROGRESS", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("MERCHANT_SHOW", LuaEvents_DialogOpened);

            Lua.Events.DetachEvent("LOOT_CLOSED", LuaEvents_DialogClosed);

            Reset(true);
            Log("Disabled");
        }

        private bool IsPlayerInRange
        {
            get
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWPlayer>().Any(p => p.Distance <= 100);
            }
        }

        private void DisableAddons()
        {
            DevLog("DisableAddons (PreventDisableAddons: {0})", Settings.Quest_PreventDisableAddons);

            var _enabled = "Lua.DoString(\"for i=1,GetNumAddOns()";
            var _disabled = "Lua.DoString(\"--for i=1,GetNumAddOns()";

            var _questingFolder = $"{Application.StartupPath}\\Default Profiles\\Questing Profile Pack\\";
            var _files = Directory.GetFiles(_questingFolder, "*.xml", SearchOption.AllDirectories).ToList();
            foreach (var _file in _files)
            {
                var _content = File.ReadAllText(_file);
                var _echo = "EchoSchema.xsd";

                var _searchFor = (Settings.Quest_PreventDisableAddons)
                                        ? _enabled
                                        : _disabled;
                var _replaceWith = (Settings.Quest_PreventDisableAddons)
                                           ? _disabled
                                           : _enabled;

                if (StringContains(_content, "AND", _echo, _searchFor))
                {
                    var _result = _content.Replace(_searchFor, _replaceWith);
                    File.WriteAllText(_file, _result);
                }
            }

            _disableAddonsSetting = Convert.ToInt32(Settings.Quest_PreventDisableAddons);
        }

        private async Task<bool> HandleDisableAddons()
        {
            if (_disableAddonsSetting == Convert.ToInt32(Settings.Quest_PreventDisableAddons) )
                return false;

            DisableAddons();
            return true;
        }

        private void RemoveNameFromLog()
        {
            DevLog("RemoveNameFromLog");

            var _file = Logging.LogFilePath;
            var _result = Regex.Replace(File.ReadAllText(_file), Me.Name, "", RegexOptions.IgnoreCase);

            File.WriteAllText(_file, _result);
        }

        private void Profile_OnNewOuterProfileLoaded(BotEvents.Profile.NewProfileLoadedEventArgs _args)
        {
            if (_args.OldProfile != _args.NewProfile)
                _profileChange = true;
        }

        private void Logging_OnLogMessage(ReadOnlyCollection<Logging.LogMessage> _messages)
        {
            foreach (var _item in _messages)
            {
                _lastLogEntry = _item.Message;

                if (_item.Message.ToLower().Contains(Me.Name.ToLower()))
                    RemoveNameFromLog();

                if (_item.Message.Contains("[Flightor] Set up for movement from"))
                {
                    _intervalLastFlightorLog = DeltaTime(_timeLastFlightorLog, "ms");
                    _timeLastFlightorLog = DateTime.Now;
                }

                if (_item.Message.Contains("Changed maps to "))
                    _timeMapChanged = DateTime.Now;              

                if (_item.Message.Contains("Dismount to kill bot poi."))
                    _meleeDismount = true;

                if (_item.Message.Contains("Navigator unable to move") ||
                    _item.Message.Contains("[Flightor] We are indoors, but we could not find a take off spot. Retrying soon..") ||
                    _item.Message.Contains("[Flightor] Failed ground nav from "))
                    StuckState = Stuck.On;

                if (_item.Message.Contains("Stop and dismount. Reason: Using transport"))
                    _timeTransportDismount = DateTime.Now;


                if (_item.Message.Contains("Path distances request failed on map:"))
                {
                    if (DeltaTime(_timePathDistancesRequestCount, "ms") < 3000)
                    {
                        _failedPathDistancesRequestCount++;
                        DevLog("FailedPathDistancesRequestCount++ ({0})", _failedPathDistancesRequestCount);
                    }
                    else if (_failedPathDistancesRequestCount > 0)
                    {
                        DevLog("FailedPathDistancesRequestCount Reset");
                        _failedPathDistancesRequestCount = 0;
                    }

                    _timePathDistancesRequestCount = DateTime.Now;
                }

                if (_item.Message.Contains("Mount up:"))
                {
                    if (DeltaTime(_timeMountUp, "ms") < 6000)
                    {
                        _failedMountUpCount++;
                        DevLog("FailedMountUpCount++ ({0})", _failedMountUpCount);
                    }
                    else if (_failedMountUpCount > 0)
                    {
                        DevLog("FailedMountUpCount Reset");
                        _failedMountUpCount = 0;
                    }

                    _timeMountUp = DateTime.Now;
                }

                /*
                if (_item.Message.Contains("We are stuck"))
                    StuckState = Stuck.On;
                    */

                if (_item.Message.Contains("Done selling/repairing") || _item.Message.Contains("Done mailing"))
                {
                    if (Settings.Bag_Sort)
                    {
                        Log("Sorting bags");
                        Lua.DoString("SortBags()");
                    }
                }

                if (
                        (_item.Message.Contains("Moving into") && _item.Message.Contains("indoor area")) ||
                        _item.Message.Contains("Destination appears to be indoors")
                    )
                    _timeMovingInto = DateTime.Now;
            }
        }

        public void PrintSettings(bool _enable = false, bool _imported = false)
        {
            var _logString = "\n\n\n";

            if (!_enable)
                _logString += String.Format("Settings changed!{0}\n\n", (_imported)
                                                                            ? " (imported)"
                                                                            : "");

            string _onlineVersion = null;
            try
            {
                var _regex = new Regex("(?<=<span style=\"font-size: 18px\">v)[.0-9]+(?=<)");
                var _html = new KOMWebClient().DownloadString("https://www.thebuddyforum.com/threads/plugin-keeponmoving-unstuck-handling-and-more-in-development.249513/");
                var _result = _regex.Matches(_html);
                _onlineVersion = _result[0].ToString();
            }
            catch (Exception)
            {
                //
            }

            var _updateInformation = (_onlineVersion == null)
                                            ? ") [WebClient Timeout]"
                                            : (Version.ToString() != _onlineVersion)
                                                        ? $" != v{_onlineVersion})"
                                                        : ")";
            _logString += $"----- KoM-LOG (v{Version}{_updateInformation} -----\n\n";

            var _settings = Settings.GetSettings()
                                    .OrderBy(s => s.Key);

            var _changed = false;
            string _lastCategory = null;
            foreach (var _setting in _settings)
            {
                var _category = _setting.Key.Substring(0, _setting.Key.IndexOf("_"));
                if (_category != _lastCategory)
                {
                    _lastCategory = _category;
                    _logString += $"[{_category.ToUpper()}]\n";
                }

                if (!Settings._settingsDict.ContainsKey(_setting.Key))
                    Settings._settingsDict.Add(_setting.Key, _setting.Value);

                var _settingCaption = _setting.Key.Substring(_setting.Key.IndexOf("_") + 1);
                if (Settings._settingsDict[_setting.Key].ToString() == _setting.Value.ToString())
                    _logString += $"     {_settingCaption}: {_setting.Value}\n";
                else
                {
                    _changed = true;
                    _logString += $"  !  {_settingCaption}: {Settings._settingsDict[_setting.Key]} > {_setting.Value}\n";
                }

                Settings._settingsDict[_setting.Key] = _setting.Value;
            }

            _logString += "\n----- KoM-LOG -----\n\n";
            
            if (_enable || _changed || _imported)
                DevLog(_logString);

            if (_enable)
            {
                if (_onlineVersion != null && Version.ToString() != _onlineVersion)
                {
                    Log("----- New version available: v{0} -----", _onlineVersion);

                    if (Settings.General_UpdateSound)
                    {
                        _soundPlayer.SoundLocation = $"{Application.StartupPath}\\Plugins\\KeepOnMoving\\update.wav";
                        _soundPlayer.Play();
                    }
                }
            }
        }

        /*
        private int GetTurnInNpcId(int _questId, string _mode = "PFW")
        {
            if (_mode.Contains("P"))        // last (P)rofile
            {
                try
                {
                    var _doc = new XmlDocument();
                    _doc.Load(_lastProfile);
                    var _node = _doc.SelectSingleNode($"//TurnIn[@QuestId='{_questId}']");
                    return Convert.ToInt32(_node.Attributes["TurnInId"].Value);
                }
                catch (Exception)
                {
                    //
                }
            }

            if (_mode.Contains("F"))        // questing (F)olders
            {
                var _questingFolder = $"{Application.StartupPath}\\Default Profiles\\Questing Profile Pack\\";
                var _files = Directory.GetFiles(_questingFolder, "*.xml", SearchOption.AllDirectories).ToList();
                foreach (var _file in _files)
                {
                    try
                    {
                        var _doc = new XmlDocument();
                        _doc.Load(_file);
                        var _node = _doc.SelectSingleNode($"//TurnIn[@QuestId='{_questId}']");
                        return Convert.ToInt32(_node.Attributes["TurnInId"].Value);
                    }
                    catch (Exception)
                    {
                        //
                    }
                }
            }

            if (_mode.Contains("W"))        // (W)owhead
            {
                try
                {
                    var _url = $"http://www.wowhead.com/quest={_questId}";
                    var _html = new KOMWebClient().DownloadString(_url);

                    var _regex = new Regex("(?<=myMapper(.+)\"end\"(.+)\"id\":)\\d+");
                    var _result = _regex.Matches(_html);
                    return Convert.ToInt32(_result[0].ToString());
                }
                catch (Exception)
                {
                    //
                }
            }

            return -1;
        }
        */

        private object QueryOnline(string _mode, int _id, string _filter, string _returnType = "int")
        {
            string _html = null;
            var _wowhead = false;
            try
            {
                var _url = $"http://www.wowdb.com/{_mode}s/{_id}";
                _html = new KOMWebClient().DownloadString(_url);
            }
            catch(Exception)
            {
                try
                {
                    var _url = $"http://www.wowhead.com/{_mode}={_id}";
                    _html = new KOMWebClient().DownloadString(_url);
                    _wowhead = true;
                }
                catch(Exception)
                {
                    return null;
                }
            }

            var _regex = new Regex("");
            if (_wowhead)
            {
                switch (_filter)
                {
                    case "TurnInNpcId":
                        _regex = new Regex("(?<=myMapper(.+)\"end\"(.+)\"id\":)\\d+");
                        break;
                }
            }
            else
            {
                switch (_filter)
                {
                    case "TurnInNpcId":
                        _regex = new Regex("(?<=(.+)quest-end(.+))\\d+");
                        break;
                    case "FactionId":
                        _regex = new Regex("(?<=Faction:(.+))\\d+");
                        break;
                }
            }

            var _result = _regex.Matches(_html);
            try
            {
                if (_returnType == "int")
                    return Convert.ToInt32(_result[0].ToString());
            }
            catch(Exception)
            {
                //
            }

            return null;
        }

        public override void OnEnable()
        {
            Log("Enabled");
            PrintSettings(true);

            if (_trainerData.Count() == 0)
                CreateTrainerData();        

            try
            {
                BotEvents.OnBotStarted += BotEvents_OnOnBotStarted;
                BotEvents.OnBotStopped += BotEvents_OnOnBotStopped;
                BotEvents.OnBotPaused += BotEvents_OnOnBotPaused;
                BotEvents.OnBotResumed += BotEvents_OnOnBotResumed;
                Logging.OnLogMessage += Logging_OnLogMessage;
                BotEvents.Profile.OnNewOuterProfileLoaded += Profile_OnNewOuterProfileLoaded;

                TreeHooks.Instance.InsertHook("InGame_Check", 0, _KOM ?? (_KOM = new ActionRunCoroutine(ctx => KOMloop())));

                Lua.Events.AttachEvent("PLAYER_LEVEL_UP", LuaEvents_PlayerLevelUp);
                Lua.Events.AttachEvent("PLAYER_DEAD", LuaEvents_PlayerDead);
                Lua.Events.AttachEvent("PLAYER_UNGHOST", LuaEvents_PlayerUnghost);

                Lua.Events.AttachEvent("QUEST_WATCH_UPDATE", LuaEvents_QuestWatchUpdateQuestTurnedIn);
                Lua.Events.AttachEvent("QUEST_TURNED_IN", LuaEvents_QuestWatchUpdateQuestTurnedIn);
                Lua.Events.AttachEvent("QUEST_ACCEPTED", LuaEvents_QuestAccepted);

                Lua.Events.AttachEvent("BAG_UPDATE_DELAYED", LuaEvents_BagUpdateDelayed);

                Lua.Events.AttachEvent("DUEL_REQUESTED", LuaEvents_DuelRequested);
                Lua.Events.AttachEvent("PARTY_INVITE_REQUEST", LuaEvents_PartyInviteRequest);

                Lua.Events.AttachEvent("UI_ERROR_MESSAGE", LuaEvents_UiErrorMessage);
                Lua.Events.AttachEvent("UNIT_SPELLCAST_INTERRUPTED", LuaEvents_UnitSpellcastInterrupted);
                Lua.Events.AttachEvent("UNIT_SPELLCAST_SUCCEEDED", LuaEvents_UnitSpellcastSucceeded);

                Lua.Events.AttachEvent("TAXIMAP_OPENED", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("GOSSIP_SHOW", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("MAIL_SHOW", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("TRAINER_SHOW", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("LOOT_OPENED", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("CINEMATIC_START", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("QUEST_DETAIL", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("QUEST_PROGRESS", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("MERCHANT_SHOW", LuaEvents_DialogOpened);

                Lua.Events.AttachEvent("LOOT_CLOSED", LuaEvents_DialogClosed);

                Lua.Events.AttachEvent("UNIT_ENTERING_VEHICLE", LuaEvents_UnitEnteringVehicle);
                Lua.Events.AttachEvent("UNIT_EXITED_VEHICLE", LuaEvents_UnitExitedVehicle);
            }
            catch (Exception)
            {
                //
            }
        }

        private void LuaEvents_UnitExitedVehicle(object _sender, LuaEventArgs _args)
        {
            if ((string)_args.Args[0] == "player")
                _timeExitVehicle = DateTime.Now;
        }

        private void LuaEvents_UnitEnteringVehicle(object _sender, LuaEventArgs _args)
        {
            if ((string)_args.Args[0] == "player")
                _timeEnterVehicle = DateTime.Now;
        }

        private void LuaEvents_DialogClosed(object _sender, LuaEventArgs _args)
        {
            _timeDialogClosed = DateTime.Now;
            DevLog("DialogClosed ({0})", _args.EventName);
        }

        private void LuaEvents_DialogOpened(object _sender, LuaEventArgs _args)
        {
            _timeDialogOpened = DateTime.Now;
            DevLog("DialogOpened ({0})", _args.EventName);
        }

        private double GetGatheringSpellId(SkillLine _skill)
        {
            var _skillLineAbilities = SkillLineAbility.GetAbilities()
                .Where(a => a.SkillLine == _skill &&
                            SpellManager.HasSpell(a.SpellId));

            var _gatheringSpell = _skillLineAbilities
                .FirstOrDefault(a => WoWSpell.FromId(a.SpellId).SpellEffects.Count() > 0 &&
                                     WoWSpell.FromId(a.SpellId).SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.OpenLock));

            if (_gatheringSpell == null)
                return -1;

            return _gatheringSpell.SpellId;
        }

        private void LuaEvents_UnitSpellcastSucceeded(object _sender, LuaEventArgs _args)
        {
            var _spellId = (double)_args.Args[4];
            if (_spellId == GetGatheringSpellId(SkillLine.Herbalism) ||       // Herbalism
                _spellId == GetGatheringSpellId(SkillLine.Mining))            // Mining
                _timeSucceeded = DateTime.Now;
        }

        private void LuaEvents_UnitSpellcastInterrupted(object _sender, LuaEventArgs _args)
        {
            var _spellId = (double)_args.Args[4];
            if (_spellId == GetGatheringSpellId(SkillLine.Herbalism) ||       // Herbalism
                _spellId == GetGatheringSpellId(SkillLine.Mining))            // Mining
                _timeInterrupted = DateTime.Now;
        }

        private void LuaEvents_UiErrorMessage(object _sender, LuaEventArgs _args)
        {
            if ((string)_args.Args[1] == _mountErrorMessage)
                _timeMountError = DateTime.Now;
        }

        private void LuaEvents_DuelRequested(object _sender, LuaEventArgs _args)
        {
            DialogState = Dialog.Duel;
        }

        private void LuaEvents_PartyInviteRequest(object _sender, LuaEventArgs _args)
        {
            DialogState = Dialog.Party;
        }

        private void LuaEvents_PlayerDead(object _sender, LuaEventArgs _args)
        {
            _timeDied = DateTime.Now;
            DevLog("Died at {0}", DateTime.Now);

            if (_timeUnghost == DateTime.MinValue)
                return;

            var _deltaAlive = DeltaTime(_timeUnghost, "s");
            if (_deltaAlive < 60)
            {
                DevLog("*{0}  >  +{1} ({2}s)", _timeUnghost, _timeDied, Math.Round(_deltaAlive));
                _failedReviveCount++;
                DevLog("FailedReviveCount++ ({0})", _failedReviveCount);
            }
            else if (_failedReviveCount > 0)
            {
                _failedReviveCount = 0;
                DevLog("FailedReviveCount Reset");
            }
        }

        private void LuaEvents_PlayerUnghost(object _sender, LuaEventArgs _args)
        {
            _timeUnghost = DateTime.Now;
            Reset();
            DevLog("Unghost at {0}", DateTime.Now);
        }

        private void LuaEvents_QuestWatchUpdateQuestTurnedIn(object _sender, LuaEventArgs _args)
        {
            _checkedQuests = false;
            _completedQuests = QuestLog.Instance.GetCompletedQuests();
            _allQuests = QuestLog.Instance.GetAllQuests();

            double _questId = Convert.ToDouble(_args.Args[0]);
            if (_questId == 31012 || _questId == 30987)
                CreateTrainerData();
        }

        private void LuaEvents_QuestAccepted(object _sender, LuaEventArgs _args)
        {
            _allQuests = QuestLog.Instance.GetAllQuests();
        }

        private void LuaEvents_BagUpdateDelayed(object _sender, LuaEventArgs _args)
        {
            _checkedItems = false;
        }

        private void LuaEvents_PlayerLevelUp(object _sender, LuaEventArgs _args)
        {
            if (!GetAutoLoader().Contains("1-110 Autoloader"))
                _reloadProfile = true;

            _checkedItems = false;
            _checkedQuests = false;
            _checkedGatheringSkills = false;
        }

        public override void Pulse()
        {
        }

        private void BotEvents_OnOnBotPaused(object _sender, EventArgs e)
        {
            Lua.DoString("SetCVar('AutoInteract', 0)");
            if (!Settings.Performance_LowGraphicMode)
                return;

            Log("Restoring Graphic Mode");
            LowGraphicMode(false);
        }

        private void BotEvents_OnOnBotResumed(object _sender, EventArgs e)
        {
            if (!Settings.Performance_LowGraphicMode)
                return;

            Log("Enabling Low Graphic Mode");
            LowGraphicMode();
        }

        private void BotEvents_OnOnBotStarted(EventArgs args)
        {
            Reset(true);

            DisableAddons();

            string _profile = ProfileManager.XmlLocation;
            DevLog("Bot Start");
            DevLog("      Current Profile: {0}", (_profile == null)
                                                    ? "null"
                                                    : _profile);

            if (Settings.Quest_PandarenFaction.ToString() != "NotYetChosen" && Me.Race == WoWRace.Pandaren && (!IsQuestCompleted(31012) && !IsQuestCompleted(30987)))
            {
                if (_profile != null && _profile.ToLower().Contains("questing profile pack") && _profile.ToLower().Replace("auto loader", "autoloader").Contains("autoloader"))
                {
                    var _factionLetter = Settings.Quest_PandarenFaction.ToString().Substring(0, 1);
                    if (_profile.ToLower().Contains("store:///"))
                        _profile = $"store:///1-110 Questing Profile Pack/Classic/[{_factionLetter} - Quest] 1-58 [Kick]/[{_factionLetter} - Quest] 1-12 [Kick]/[{_factionLetter} - Quest] Panda 1-12 [Kick].xml";
                    else
                        _profile = $"{Application.StartupPath}\\Default Profiles\\Questing Profile Pack\\Classic\\[{_factionLetter} - Quest] 1-58 [Kick]\\[{_factionLetter} - Quest] 1-12 [Kick]\\[{_factionLetter} - Quest] Panda 1-12 [Kick].xml";

                    DevLog("      New Profile: {0}", _profile);

                    try
                    {
                        ProfileManager.LoadNew(_profile);
                    }
                    catch (Exception) { }
                }
            }
        }

        private string GetAutoLoader()
        {
            var _profile = ProfileManager.XmlLocation;
            if (_profile == null || _profile == "")
                return _profile;

            if (!Settings.Quest_AutoLoader)
                return _profile;

            if (_profile.ToLower().Replace("auto loader", "autoloader").Contains("autoloader"))
                return _profile;

            if (BotManager.Current.ToString() != "Questing")
                return _profile;

            if (Settings.Quest_QuestingPack.ToString() == "None")
                return _profile;

            if (Settings.Quest_QuestingPack.ToString() == "Old")
            {
                if (_profile.ToLower().Contains("store:///"))
                    return "store:///1-110 Questing Profile Pack/Auto Loader - v2.xml";
                else
                    return $"{Application.StartupPath}\\Default Profiles\\Questing Profile Pack\\Auto Loader - v2.xml";
            }
            else if (Settings.Quest_QuestingPack.ToString() == "New")
                return $"{Application.StartupPath}\\Default Profiles\\Questing Profile Pack\\1-110 Autoloader.xml";

            return _profile;
        }

        private void RestartBot()
        {
            Reset();
            TreeRoot.Stop();

            if (!Settings.General_InfiniteMode)
            {
                ThreadPool.QueueUserWorkItem(o =>
                {
                    TreeRoot.WaitForStop();
                    TreeRoot.Start();
                });
            }
        }

		private void BotEvents_OnOnBotStopped(EventArgs args)
		{
			if(Settings.Performance_LowGraphicMode)
            {
				Log("Restoring Graphic Mode");
                LowGraphicMode(false);
            }
            Lua.DoString("SetCVar('AutoInteract', 0)");

            if (Settings.General_InfiniteMode)
            {
                Reset();

				Log("Enabling Infinite Mode");

                var _profile = ProfileManager.XmlLocation;
                DevLog("Bot Stop");
                DevLog("      Current Profile: {0}", (_profile == null)
                                                        ? "null"
                                                        : _profile);

                if (Me.Race != WoWRace.Pandaren || IsQuestCompleted(31012) || IsQuestCompleted(30987))
                    _profile = GetAutoLoader();

				ThreadPool.QueueUserWorkItem(o =>
                {
					TreeRoot.WaitForStop();

					while(Me.InVehicle)
                    {
                        DevLog("WHILE: BotStop > VehicleExit");
                        Lua.DoString("VehicleExit()");
					}

                    if (IsFrameVisible("QuestFrame"))
                        Log("Closing QuestFrame");
                    else if (IsFrameVisible("GossipFrame"))
                        Log("Closing GossipFrame");

                    while (IsFrameVisible("QuestFrame") || IsFrameVisible("GossipFrame"))
                    {
                        DevLog("WHILE: BotStop > QuestFrame/GossipFrame");
                        if (IsFrameVisible("QuestFrame"))
                            QuestFrame.Instance.Hide();
                        else if (IsFrameVisible("GossipFrame"))
                            GossipFrame.Instance.Hide();
                    }

                    DevLog("      New Profile: {0}", _profile);

                    try
                    {
                        ProfileManager.LoadNew(_profile);
					}
					catch (Exception) {}

					TreeRoot.Start();
				});
			}
            else
                Reset(true);
        }
    }
}
